@@ -47,7 +47,7 @@ import org.apache.tomcat.util.buf.MessageBytes;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 1.14 $ $Date: 2004/08/27 23:56:11 $
+ * @version $Revision: 1.18 $ $Date: 2005/02/23 19:27:56 $
  */
 
 public class FormAuthenticator
@@ -65,6 +65,13 @@ public class FormAuthenticator
     protected static final String info =
         "org.apache.catalina.authenticator.FormAuthenticator/1.0";
 
+    /**
+     * Character encoding to use to read the username and password parameters
+     * from the request. If not set, the encoding of the request body will be
+     * used.
+     */
+    protected String characterEncoding = null;
+
 
     // ------------------------------------------------------------- Properties
 
@@ -79,6 +86,22 @@ public class FormAuthenticator
     }
 
 
+    /**
+     * Return the character encoding to use to read the username and password.
+     */
+    public String getCharacterEncoding() {
+        return characterEncoding;
+    }
+
+    
+    /**
+     * Set the character encoding to be used to read the username and password. 
+     */
+    public void setCharacterEncoding(String encoding) {
+        characterEncoding = encoding;
+    }
+
+
     // --------------------------------------------------------- Public Methods
 
 
@@ -223,6 +246,9 @@ public class FormAuthenticator
         // Yes -- Validate the specified credentials and redirect
         // to the error page if they are not correct
         Realm realm = context.getRealm();
+        if (characterEncoding != null) {
+            request.setCharacterEncoding(characterEncoding);
+        }
         String username = request.getParameter(Constants.FORM_USERNAME);
         String password = request.getParameter(Constants.FORM_PASSWORD);
         if (log.isDebugEnabled())
@@ -246,8 +272,8 @@ public class FormAuthenticator
         if (session == null)
             session = request.getSessionInternal(false);
         if (session == null) {
-            if (container.getLogger().isDebugEnabled())
-                container.getLogger().debug("User took so long to log on the session expired");
+            if (containerLog.isDebugEnabled())
+                containerLog.debug("User took so long to log on the session expired");
             response.sendError(HttpServletResponse.SC_REQUEST_TIMEOUT,
                                sm.getString("authenticator.sessionExpired"));
             return (false);
