@@ -47,7 +47,7 @@ import org.apache.tomcat.util.http.mapper.Mapper;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 1.14 $ $Date: 2004/10/13 08:17:47 $
+ * @version $Revision: 1.16 $ $Date: 2005/01/29 19:44:43 $
  */
 
 
@@ -708,7 +708,6 @@ public class Connector
     public void setScheme(String scheme) {
 
         this.scheme = scheme;
-        setProperty("scheme", scheme);
 
     }
 
@@ -733,7 +732,6 @@ public class Connector
     public void setSecure(boolean secure) {
 
         this.secure = secure;
-        setProperty("secure", String.valueOf(secure));
 
     }
 
@@ -876,6 +874,20 @@ public class Connector
 
     }
 
+    
+    protected ObjectName createObjectName(String domain, String type)
+            throws MalformedObjectNameException {
+        String encodedAddr = null;
+        if (getProperty("address") != null) {
+            encodedAddr = URLEncoder.encode(getProperty("address").toString());
+        }
+        String addSuffix = (getProperty("address") == null) ? "" : ",address="
+                + encodedAddr;
+        ObjectName _oname = new ObjectName(domain + ":type=" + type + ",port="
+                + getPort() + addSuffix);
+        return _oname;
+    }
+    
     /**
      * Initialize this connector (create ServerSocket here!)
      */
@@ -883,7 +895,8 @@ public class Connector
         throws LifecycleException
     {
         if (initialized) {
-            log.info(sm.getString("coyoteConnector.alreadyInitialized"));
+            if(log.isInfoEnabled())
+                log.info(sm.getString("coyoteConnector.alreadyInitialized"));
            return;
         }
 
@@ -893,20 +906,15 @@ public class Connector
             try {
                 // we are loaded directly, via API - and no name was given to us
                 StandardEngine cb=(StandardEngine)container;
-                String encodedAddr = null;
-                if (getProperty("address") != null) {
-                    encodedAddr = URLEncoder.encode(getProperty("address").toString());
-                }
-                String addSuffix=(getProperty("address")==null) ?"": ",address=" + encodedAddr;
-                oname=new ObjectName(cb.getName() + ":type=Connector,port="+
-                        getPort() + addSuffix);
+                oname = createObjectName(cb.getName(), "Connector");
                 Registry.getRegistry(null, null)
                     .registerComponent(this, oname, null);
                 controller=oname;
             } catch (Exception e) {
                 log.error( "Error registering connector ", e);
             }
-            log.debug("Creating name for connector " + oname);
+            if(log.isDebugEnabled())
+                log.debug("Creating name for connector " + oname);
         }
 
         // Initializa adapter
@@ -964,8 +972,9 @@ public class Connector
             initialize();
 
         // Validate and update our current state
-        if (started) {
-            log.info(sm.getString("coyoteConnector.alreadyStarted"));
+        if (started ) {
+            if(log.isInfoEnabled())
+                log.info(sm.getString("coyoteConnector.alreadyStarted"));
             return;
         }
         lifecycle.fireLifecycleEvent(START_EVENT, null);
@@ -977,14 +986,14 @@ public class Connector
             // We are registred - register the adapter as well.
             try {
                 Registry.getRegistry(null, null).registerComponent
-                    (protocolHandler, this.domain + ":type=protocolHandler,className="
-                     + protocolHandlerClassName, null);
+                    (protocolHandler, createObjectName(this.domain,"ProtocolHandler"), null);
             } catch (Exception ex) {
                 log.error(sm.getString
                           ("coyoteConnector.protocolRegistrationFailed"), ex);
             }
         } else {
-            log.info(sm.getString
+            if(log.isInfoEnabled())
+                log.info(sm.getString
                      ("coyoteConnector.cannotRegisterProtocol"));
         }
 
@@ -1001,8 +1010,12 @@ public class Connector
             //mapperListener.setEngine( service.getContainer().getName() );
             mapperListener.init();
             try {
+                ObjectName mapperOname = createObjectName(this.domain,"Mapper");
+                if (log.isDebugEnabled())
+                    log.debug(sm.getString(
+                            "coyoteConnector.MapperRegistration", mapperOname));
                 Registry.getRegistry(null, null).registerComponent
-                    (mapper, this.domain + ":type=Mapper", "Mapper");
+                    (mapper, createObjectName(this.domain,"Mapper"), "Mapper");
             } catch (Exception ex) {
                 log.error(sm.getString
                         ("coyoteConnector.protocolRegistrationFailed"), ex);
@@ -1028,14 +1041,14 @@ public class Connector
         started = false;
 
         try {
+            mapperListener.destroy();
             Registry.getRegistry(null, null).unregisterComponent
-                (new ObjectName(domain,"type", "Mapper"));
+                (createObjectName(this.domain,"Mapper"));
             Registry.getRegistry(null, null).unregisterComponent
-                (new ObjectName(domain
-                 + ":type=protocolHandler,className="
-                 + protocolHandlerClassName));
+                (createObjectName(this.domain,"ProtocolHandler"));
         } catch (MalformedObjectNameException e) {
-            log.info( "Error unregistering mapper ", e);
+            log.error( sm.getString
+                    ("coyoteConnector.protocolUnregistrationFailed"), e);
         }
         try {
             protocolHandler.destroy();
@@ -1100,7 +1113,8 @@ public class Connector
             ObjectName parentName=new ObjectName( domain + ":" +
                     "type=Service");
             
-            log.debug("Adding to " + parentName );
+            if(log.isDebugEnabled())
+                log.debug("Adding to " + parentName );
             if( mserver.isRegistered(parentName )) {
                 mserver.invoke(parentName, "addConnector", new Object[] { this },
                         new String[] {"org.apache.catalina.connector.Connector"});
@@ -1113,13 +1127,15 @@ public class Connector
             ObjectName engName=new ObjectName( domain + ":" + "type=Engine");
             if( mserver.isRegistered(engName )) {
                 Object obj=mserver.getAttribute(engName, "managedResource");
-                log.debug("Found engine " + obj + " " + obj.getClass());
+                if(log.isDebugEnabled())
+                      log.debug("Found engine " + obj + " " + obj.getClass());
                 container=(Container)obj;
                 
                 // Internal initialize - we now have the Engine
                 initialize();
                 
-                log.debug("Initialized");
+                if(log.isDebugEnabled())
+                    log.debug("Initialized");
                 // As a side effect we'll get the container field set
                 // Also initialize will be called
                 return;
@@ -1132,7 +1148,8 @@ public class Connector
     public void init() throws Exception {
 
         if( this.getService() != null ) {
-            log.debug( "Already configured" );
+            if(log.isDebugEnabled())
+                 log.debug( "Already configured" );
             return;
         }
         if( container==null ) {
@@ -1142,7 +1159,8 @@ public class Connector
 
     public void destroy() throws Exception {
         if( oname!=null && controller==oname ) {
-            log.debug("Unregister itself " + oname );
+            if(log.isDebugEnabled())
+                 log.debug("Unregister itself " + oname );
             Registry.getRegistry(null, null).unregisterComponent(oname);
         }
         if( getService() == null)
