@@ -74,7 +74,7 @@ import org.apache.commons.logging.Log;
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Revision: 1.16 $ $Date: 2004/10/26 15:42:05 $
+ * @version $Revision: 1.19 $ $Date: 2005/02/07 21:56:32 $
  */
 
 public class Request
@@ -863,6 +863,10 @@ public class Request
             if(attr != null) {
                 attributes.put(Globals.KEY_SIZE_ATTR, attr);
             }
+            attr = coyoteRequest.getAttribute(Globals.SSL_SESSION_ID_ATTR);
+            if(attr != null) {
+                attributes.put(Globals.SSL_SESSION_ID_ATTR, attr);
+            }
             attr = attributes.get(name);
         }
         return attr;
@@ -875,7 +879,8 @@ public class Request
     static boolean isSSLAttribute(String name) {
         return Globals.CERTIFICATES_ATTR.equals(name) ||
             Globals.CIPHER_SUITE_ATTR.equals(name) ||
-            Globals.KEY_SIZE_ATTR.equals(name);
+            Globals.KEY_SIZE_ATTR.equals(name)  ||
+            Globals.SSL_SESSION_ID_ATTR.equals(name);
     }
 
     /**
@@ -2191,7 +2196,15 @@ public class Request
               (sm.getString("coyoteRequest.sessionCreateCommitted"));
         }
 
-        session = manager.createSession();
+        // Attempt to reuse session id if one was submitted in a cookie
+        // Do not reuse the session id if it is from a URL, to prevent possible
+        // phishing attacks
+        if (connector.getEmptySessionPath() 
+                && isRequestedSessionIdFromCookie()) {
+            session = manager.createSession(getRequestedSessionId());
+        } else {
+            session = manager.createSession(null);
+        }
 
         // Creating a new session cookie based on that session
         if ((session != null) && (getContext() != null)
@@ -2280,7 +2293,10 @@ public class Request
 
         Parameters parameters = coyoteRequest.getParameters();
 
-        String enc = coyoteRequest.getCharacterEncoding();
+        // getCharacterEncoding() may have been overridden to search for
+        // hidden form field containing request encoding
+        String enc = getCharacterEncoding();
+
         boolean useBodyEncodingForURI = connector.getUseBodyEncodingForURI();
         if (enc != null) {
             parameters.setEncoding(enc);
