@@ -31,6 +31,10 @@ import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpSession;
 
+import org.apache.catalina.util.StringManager;
+
+
+import org.apache.catalina.security.SecurityUtil;
 
 /**
  * Facade class that wraps a Coyote request object.  
@@ -39,17 +43,16 @@ import javax.servlet.http.HttpSession;
  * @author Craig R. McClanahan
  * @author Remy Maucherat
  * @author Jean-Francois Arcand
- * @version $Revision: 1.5 $ $Date: 2004/06/23 08:24:57 $
+ * @version $Revision: 1.8 $ $Date: 2004/11/22 16:35:18 $
  */
 
-
-public class RequestFacade 
-    implements HttpServletRequest {
+public class RequestFacade implements HttpServletRequest {
         
         
     // ----------------------------------------------------------- DoPrivileged
     
-    private final class GetAttributePrivilegedAction implements PrivilegedAction{
+    private final class GetAttributePrivilegedAction
+            implements PrivilegedAction {
         
         public Object run() {
             return request.getAttributeNames();
@@ -57,7 +60,8 @@ public class RequestFacade
     }
      
     
-    private final class GetParameterMapPrivilegedAction implements PrivilegedAction{
+    private final class GetParameterMapPrivilegedAction
+            implements PrivilegedAction {
         
         public Object run() {
             return request.getParameterMap();
@@ -65,30 +69,38 @@ public class RequestFacade
     }    
     
     
-    private final class GetRequestDispatcherPrivilegedAction implements PrivilegedAction{
+    private final class GetRequestDispatcherPrivilegedAction
+            implements PrivilegedAction {
+
         private String path;
+
         public GetRequestDispatcherPrivilegedAction(String path){
             this.path = path;
         }
         
         public Object run() {   
             return request.getRequestDispatcher(path);
-       }           
+        }           
     }    
     
     
-    private final class GetParameterPrivilegedAction implements PrivilegedAction{
+    private final class GetParameterPrivilegedAction
+            implements PrivilegedAction {
+
         public String name;
+
         public GetParameterPrivilegedAction(String name){
             this.name = name;
         }
+
         public Object run() {       
             return request.getParameter(name);
         }           
     }    
     
      
-    private final class GetParameterNamesPrivilegedAction implements PrivilegedAction{
+    private final class GetParameterNamesPrivilegedAction
+            implements PrivilegedAction {
         
         public Object run() {          
             return request.getParameterNames();
@@ -96,26 +108,32 @@ public class RequestFacade
     } 
     
     
-    private final class GetParameterValuePrivilegedAction implements PrivilegedAction{
+    private final class GetParameterValuePrivilegedAction
+            implements PrivilegedAction {
+
         public String name;
+
         public GetParameterValuePrivilegedAction(String name){
             this.name = name;
         }
+
         public Object run() {       
             return request.getParameterValues(name);
         }           
     }    
   
     
-    private final class GetCookiesPrivilegedAction implements PrivilegedAction{
+    private final class GetCookiesPrivilegedAction
+            implements PrivilegedAction {
         
-       public Object run() {       
+        public Object run() {       
             return request.getCookies();
         }           
     }      
     
     
-    private final class GetCharacterEncodingPrivilegedAction implements PrivilegedAction{
+    private final class GetCharacterEncodingPrivilegedAction
+            implements PrivilegedAction {
         
         public Object run() {       
             return request.getCharacterEncoding();
@@ -123,8 +141,11 @@ public class RequestFacade
     }   
         
     
-    private final class GetHeadersPrivilegedAction implements PrivilegedAction{
+    private final class GetHeadersPrivilegedAction
+            implements PrivilegedAction {
+
         private String name;
+
         public GetHeadersPrivilegedAction(String name){
             this.name = name;
         }
@@ -135,7 +156,8 @@ public class RequestFacade
     }    
         
     
-    private final class GetHeaderNamesPrivilegedAction implements PrivilegedAction{
+    private final class GetHeaderNamesPrivilegedAction
+            implements PrivilegedAction {
 
         public Object run() {       
             return request.getHeaderNames();
@@ -143,7 +165,8 @@ public class RequestFacade
     }  
             
     
-    private final class GetLocalePrivilegedAction implements PrivilegedAction{
+    private final class GetLocalePrivilegedAction
+            implements PrivilegedAction {
 
         public Object run() {       
             return request.getLocale();
@@ -151,14 +174,17 @@ public class RequestFacade
     }    
             
     
-    private final class GetLocalesPrivilegedAction implements PrivilegedAction{
+    private final class GetLocalesPrivilegedAction
+            implements PrivilegedAction {
 
         public Object run() {       
             return request.getLocales();
         }           
     }    
     
-    private final class GetSessionPrivilegedAction implements PrivilegedAction{
+    private final class GetSessionPrivilegedAction
+            implements PrivilegedAction {
+
         private boolean create;
         
         public GetSessionPrivilegedAction(boolean create){
@@ -168,7 +194,8 @@ public class RequestFacade
         public Object run() {  
             return request.getSession(create);
         }           
-    }      
+    }
+
     // ----------------------------------------------------------- Constructors
 
 
@@ -193,6 +220,13 @@ public class RequestFacade
     protected Request request = null;
 
 
+    /**
+     * The string manager for this package.
+     */
+    protected static StringManager sm =
+        StringManager.getManager(Constants.Package);
+
+
     // --------------------------------------------------------- Public Methods
 
 
@@ -208,11 +242,23 @@ public class RequestFacade
 
 
     public Object getAttribute(String name) {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getAttribute(name);
     }
 
 
     public Enumeration getAttributeNames() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         if (System.getSecurityManager() != null){
             return (Enumeration)AccessController.doPrivileged(
                 new GetAttributePrivilegedAction());        
@@ -223,6 +269,12 @@ public class RequestFacade
 
 
     public String getCharacterEncoding() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         if (System.getSecurityManager() != null){
             return (String)AccessController.doPrivileged(
                 new GetCharacterEncodingPrivilegedAction());
@@ -233,28 +285,57 @@ public class RequestFacade
 
 
     public void setCharacterEncoding(String env)
-        throws java.io.UnsupportedEncodingException {
+            throws java.io.UnsupportedEncodingException {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         request.setCharacterEncoding(env);
     }
 
 
     public int getContentLength() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getContentLength();
     }
 
 
     public String getContentType() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getContentType();
     }
 
 
-    public ServletInputStream getInputStream()
-        throws IOException {
+    public ServletInputStream getInputStream() throws IOException {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getInputStream();
     }
 
 
     public String getParameter(String name) {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         if (System.getSecurityManager() != null){
             return (String)AccessController.doPrivileged(
                 new GetParameterPrivilegedAction(name));
@@ -265,6 +346,12 @@ public class RequestFacade
 
 
     public Enumeration getParameterNames() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         if (System.getSecurityManager() != null){
             return (Enumeration)AccessController.doPrivileged(
                 new GetParameterNamesPrivilegedAction());
@@ -276,13 +363,18 @@ public class RequestFacade
 
     public String[] getParameterValues(String name) {
 
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         String[] ret = null;
 
         /*
          * Clone the returned array only if there is a security manager
          * in place, so that performance won't suffer in the nonsecure case
          */
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             ret = (String[]) AccessController.doPrivileged(
                 new GetParameterValuePrivilegedAction(name));
             if (ret != null) {
@@ -297,6 +389,12 @@ public class RequestFacade
 
 
     public Map getParameterMap() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         if (System.getSecurityManager() != null){
             return (Map)AccessController.doPrivileged(
                 new GetParameterMapPrivilegedAction());        
@@ -307,52 +405,111 @@ public class RequestFacade
 
 
     public String getProtocol() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getProtocol();
     }
 
 
     public String getScheme() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getScheme();
     }
 
 
     public String getServerName() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getServerName();
     }
 
 
     public int getServerPort() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getServerPort();
     }
 
 
-    public BufferedReader getReader()
-        throws IOException {
+    public BufferedReader getReader() throws IOException {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getReader();
     }
 
 
     public String getRemoteAddr() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getRemoteAddr();
     }
 
 
     public String getRemoteHost() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getRemoteHost();
     }
 
 
     public void setAttribute(String name, Object o) {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         request.setAttribute(name, o);
     }
 
 
     public void removeAttribute(String name) {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         request.removeAttribute(name);
     }
 
 
     public Locale getLocale() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         if (System.getSecurityManager() != null){
             return (Locale)AccessController.doPrivileged(
                 new GetLocalePrivilegedAction());
@@ -363,6 +520,12 @@ public class RequestFacade
 
 
     public Enumeration getLocales() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         if (System.getSecurityManager() != null){
             return (Enumeration)AccessController.doPrivileged(
                 new GetLocalesPrivilegedAction());
@@ -373,11 +536,23 @@ public class RequestFacade
 
 
     public boolean isSecure() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.isSecure();
     }
 
 
     public RequestDispatcher getRequestDispatcher(String path) {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         if (System.getSecurityManager() != null){
             return (RequestDispatcher)AccessController.doPrivileged(
                 new GetRequestDispatcherPrivilegedAction(path));
@@ -388,24 +563,41 @@ public class RequestFacade
 
 
     public String getRealPath(String path) {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getRealPath(path);
     }
 
 
     public String getAuthType() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getAuthType();
     }
 
 
     public Cookie[] getCookies() {
 
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         Cookie[] ret = null;
 
         /*
          * Clone the returned array only if there is a security manager
          * in place, so that performance won't suffer in the nonsecure case
          */
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             ret = (Cookie[])AccessController.doPrivileged(
                 new GetCookiesPrivilegedAction());
             if (ret != null) {
@@ -420,16 +612,34 @@ public class RequestFacade
 
 
     public long getDateHeader(String name) {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getDateHeader(name);
     }
 
 
     public String getHeader(String name) {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getHeader(name);
     }
 
 
     public Enumeration getHeaders(String name) {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         if (System.getSecurityManager() != null){
             return (Enumeration)AccessController.doPrivileged(
                 new GetHeadersPrivilegedAction(name));
@@ -440,6 +650,12 @@ public class RequestFacade
 
 
     public Enumeration getHeaderNames() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         if (System.getSecurityManager() != null){
             return (Enumeration)AccessController.doPrivileged(
                 new GetHeaderNamesPrivilegedAction());
@@ -450,73 +666,156 @@ public class RequestFacade
 
 
     public int getIntHeader(String name) {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getIntHeader(name);
     }
 
 
     public String getMethod() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getMethod();
     }
 
 
     public String getPathInfo() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getPathInfo();
     }
 
 
     public String getPathTranslated() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getPathTranslated();
     }
 
 
     public String getContextPath() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getContextPath();
     }
 
 
     public String getQueryString() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getQueryString();
     }
 
 
     public String getRemoteUser() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getRemoteUser();
     }
 
 
     public boolean isUserInRole(String role) {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.isUserInRole(role);
     }
 
 
     public java.security.Principal getUserPrincipal() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getUserPrincipal();
     }
 
 
     public String getRequestedSessionId() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getRequestedSessionId();
     }
 
 
     public String getRequestURI() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getRequestURI();
     }
 
 
     public StringBuffer getRequestURL() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getRequestURL();
     }
 
 
     public String getServletPath() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.getServletPath();
     }
 
 
     public HttpSession getSession(boolean create) {
 
-        if (System.getSecurityManager() != null){
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
+        if (SecurityUtil.isPackageProtectionEnabled()){
             return (HttpSession)AccessController.
                 doPrivileged(new GetSessionPrivilegedAction(create));
         } else {
@@ -525,47 +824,101 @@ public class RequestFacade
     }
 
     public HttpSession getSession() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return getSession(true);
     }
 
 
     public boolean isRequestedSessionIdValid() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.isRequestedSessionIdValid();
     }
 
 
     public boolean isRequestedSessionIdFromCookie() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.isRequestedSessionIdFromCookie();
     }
 
 
     public boolean isRequestedSessionIdFromURL() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.isRequestedSessionIdFromURL();
     }
 
 
     public boolean isRequestedSessionIdFromUrl() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
         return request.isRequestedSessionIdFromURL();
     }
 
 
     public String getLocalAddr() {
-		return request.getLocalAddr();
-	}
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
+        return request.getLocalAddr();
+    }
 
 
-	public String getLocalName() {
-		return request.getLocalName();
-	}
+    public String getLocalName() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
+        return request.getLocalName();
+    }
 
 
-	public int getLocalPort() {
-		return request.getLocalPort();
-	}
+    public int getLocalPort() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
+        return request.getLocalPort();
+    }
 
 
-	public int getRemotePort() {
-		return request.getRemotePort();
-	}
+    public int getRemotePort() {
+
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
+        return request.getRemotePort();
+    }
 
 }
