@@ -44,6 +44,7 @@ import org.apache.catalina.Wrapper;
 import org.apache.catalina.util.CharsetMapper;
 import org.apache.catalina.util.DateTool;
 import org.apache.catalina.util.StringManager;
+import org.apache.catalina.security.SecurityUtil;
 import org.apache.tomcat.util.buf.CharChunk;
 import org.apache.tomcat.util.buf.UEncoder;
 import org.apache.tomcat.util.http.FastHttpDateFormat;
@@ -57,7 +58,7 @@ import org.apache.tomcat.util.compat.JdkCompat;
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Revision: 1.9 $ $Date: 2004/09/16 15:28:29 $
+ * @version $Revision: 1.10 $ $Date: 2004/11/22 16:35:18 $
  */
 
 public class Response
@@ -930,7 +931,7 @@ public class Response
         cookies.add(cookie);
 
         final StringBuffer sb = new StringBuffer();
-        if (System.getSecurityManager() != null) {
+        if (SecurityUtil.isPackageProtectionEnabled()) {
             AccessController.doPrivileged(new PrivilegedAction() {
                 public Object run(){
                     ServerCookie.appendCookieValue
@@ -1349,7 +1350,7 @@ public class Response
         if (hreq.isRequestedSessionIdFromCookie())
             return (false);
         
-        if (System.getSecurityManager() != null) {
+        if (SecurityUtil.isPackageProtectionEnabled()) {
             return ((Boolean)
                 AccessController.doPrivileged(new PrivilegedAction() {
 
@@ -1452,7 +1453,7 @@ public class Response
                     
                     String encodedURI = null;
                     final String frelativePath = relativePath;
-                    if (System.getSecurityManager() != null ){
+                    if (SecurityUtil.isPackageProtectionEnabled() ){
                         try{
                             encodedURI = (String)AccessController.doPrivileged( 
                                 new PrivilegedExceptionAction(){                                
