@@ -59,7 +59,7 @@ import org.apache.tomcat.util.http.mapper.MappingData;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 1.26 $ $Date: 2004/06/23 13:51:35 $
+ * @version $Revision: 1.27 $ $Date: 2005/03/17 10:53:44 $
  */
 
 public class ApplicationContext
@@ -417,6 +417,9 @@ public class ApplicationContext
              * purposes
              */
             int semicolon = path.indexOf(';');
+            if (pos >= 0 && semicolon > pos) {
+                semicolon = -1;
+            }
             uriCC.append(path, 0, semicolon > 0 ? semicolon : pos);
             context.getMapper().map(uriMB, mappingData);
             if (mappingData.wrapper == null) {
