@@ -750,16 +750,24 @@ public abstract class ContainerBase
                                                    child.getName() +
                                                    "' is not unique");
             child.setParent(this);  // May throw IAE
+            children.put(child.getName(), child);
+
+            // Start child
             if (started && (child instanceof Lifecycle)) {
+                boolean success = false;
                 try {
                     ((Lifecycle) child).start();
+                    success = true;
                 } catch (LifecycleException e) {
                     log.error("ContainerBase.addChild: start: ", e);
                     throw new IllegalStateException
                         ("ContainerBase.addChild: start: " + e);
+                } finally {
+                    if (!success) {
+                        children.remove(child.getName());
+                    }
                 }
             }
-            children.put(child.getName(), child);
 
             fireContainerEvent(ADD_CHILD_EVENT, child);
         }
@@ -972,7 +980,8 @@ public abstract class ContainerBase
 
         // Validate and update our current component state
         if (started) {
-            log.info(sm.getString("containerBase.alreadyStarted", logName()));
+            if(log.isInfoEnabled())
+                log.info(sm.getString("containerBase.alreadyStarted", logName()));
             return;
         }
         
@@ -1029,7 +1038,8 @@ public abstract class ContainerBase
 
         // Validate and update our current component state
         if (!started) {
-            log.info(sm.getString("containerBase.notStarted", logName()));
+            if(log.isInfoEnabled())
+                log.info(sm.getString("containerBase.notStarted", logName()));
             return;
         }
 
@@ -1128,7 +1138,8 @@ public abstract class ContainerBase
                 if( controller == oname ) {
                     Registry.getRegistry(null, null)
                         .unregisterComponent(oname);
-                    log.debug("unregistering " + oname);
+                    if(log.isDebugEnabled())
+                        log.debug("unregistering " + oname);
                 }
             } catch( Throwable t ) {
                 log.error("Error unregistering ", t );
@@ -1331,7 +1342,11 @@ public abstract class ContainerBase
         String loggerName = null;
         Container current = this;
         while (current != null) {
-            loggerName = "[" + current.getName() + "]" 
+            String name = current.getName();
+            if ((name == null) || (name.equals(""))) {
+                name = "/";
+            }
+            loggerName = "[" + name + "]" 
                 + ((loggerName != null) ? ("." + loggerName) : "");
             current = current.getParent();
         }
