@@ -25,7 +25,6 @@ import org.apache.catalina.Container;
 import org.apache.catalina.Context;
 import org.apache.catalina.Host;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.Realm;
 import org.apache.catalina.Valve;
 import org.apache.catalina.startup.HostConfig;
 import org.apache.catalina.valves.ValveBase;
@@ -39,7 +38,7 @@ import org.apache.commons.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 1.35 $ $Date: 2004/10/05 17:12:51 $
+ * @version $Revision: 1.39 $ $Date: 2005/01/29 19:38:37 $
  */
 
 public class StandardHost
@@ -676,9 +675,10 @@ public class StandardHost
             try {
                 realmName=new ObjectName( domain + ":type=Realm,host=" + getName());
                 if( mserver.isRegistered(realmName ) ) {
-                    Realm nrealm = (Realm)mserver.getAttribute(realmName,
-                                                       "managedResource");
-                    setRealm(nrealm);
+                    mserver.invoke(realmName, "init", 
+                            new Object[] {},
+                            new String[] {}
+                    );            
                 }
             } catch( Throwable t ) {
                 log.debug("No realm for this host " + realmName);
@@ -709,12 +709,12 @@ public class StandardHost
                      errorReportValveClass));
             }
         }
-
-        if (xmlValidation)
-            log.info( sm.getString("standardHost.validationEnabled"));
-        else
-            log.info( sm.getString("standardHost.validationDisabled"));
-
+        if(log.isInfoEnabled()) {
+            if (xmlValidation)
+                log.info( sm.getString("standardHost.validationEnabled"));
+            else
+                log.info( sm.getString("standardHost.validationDisabled"));
+        }
         super.start();
 
     }
@@ -761,13 +761,14 @@ public class StandardHost
                 HostConfig deployer = new HostConfig();
                 addLifecycleListener(deployer);                
                 if( mserver.isRegistered( serviceName )) {
-                    log.debug("Registering with the Engine");
+                    if(log.isDebugEnabled())
+                        log.debug("Registering "+ serviceName +" with the Engine");
                     mserver.invoke( serviceName, "addChild",
                             new Object[] { this },
                             new String[] { "org.apache.catalina.Container" } );
                 }
             } catch( Exception ex ) {
-                ex.printStackTrace();
+                log.error("Host registering failed!",ex);
             }
         }
         
@@ -776,17 +777,30 @@ public class StandardHost
             try {
                 StandardEngine engine=(StandardEngine)parent;
                 domain=engine.getName();
-                log.debug( "Register " + domain );
+                if(log.isDebugEnabled())
+                    log.debug( "Register host " + getName() + " with domain "+ domain );
                 oname=new ObjectName(domain + ":type=Host,host=" +
                         this.getName());
+                controller = oname;
                 Registry.getRegistry(null, null)
                     .registerComponent(this, oname, null);
             } catch( Throwable t ) {
-                log.info("Error registering ", t );
+                log.error("Host registering failed!", t );
             }
         }
     }
 
+    public void destroy() throws Exception {
+        // destroy our child containers, if any
+        Container children[] = findChildren();
+        super.destroy();
+        for (int i = 0; i < children.length; i++) {
+            if(children[i] instanceof StandardContext)
+                ((StandardContext)children[i]).destroy();
+        }
+      
+    }
+    
     public ObjectName preRegister(MBeanServer server, ObjectName oname ) 
         throws Exception
     {
