@@ -55,6 +55,7 @@ import org.apache.catalina.Wrapper;
 import org.apache.catalina.security.SecurityUtil;
 import org.apache.catalina.util.Enumerator;
 import org.apache.catalina.util.InstanceSupport;
+import org.apache.tomcat.util.IntrospectionUtils;
 import org.apache.tomcat.util.log.SystemLogHandler;
 import org.apache.commons.modeler.Registry;
 
@@ -65,7 +66,7 @@ import org.apache.commons.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 1.55 $ $Date: 2004/10/28 16:42:37 $
+ * @version $Revision: 1.58 $ $Date: 2005/03/11 16:34:58 $
  */
 public class StandardWrapper
     extends ContainerBase
@@ -243,7 +244,21 @@ public class StandardWrapper
     private StandardWrapperValve swValve;
     private long loadTime=0;
     private int classLoadTime=0;
-
+    
+    /**
+     * Static class array used when the SecurityManager is turned on and 
+     * <code>Servlet.init</code> is invoked.
+     */
+    private static Class[] classType = new Class[]{ServletConfig.class};
+    
+    
+    /**
+     * Static class array used when the SecurityManager is turned on and 
+     * <code>Servlet.service</code>  is invoked.
+     */                                                 
+    private static Class[] classTypeUsedInService = new Class[]{
+                                                         ServletRequest.class,
+                                                         ServletResponse.class};
     // ------------------------------------------------------------- Properties
 
 
@@ -603,6 +618,30 @@ public class StandardWrapper
     // --------------------------------------------------------- Public Methods
 
 
+    /**
+     * Extract the root cause from a servlet exception.
+     * 
+     * @param e The servlet exception
+     */
+    public static Throwable getRootCause(ServletException e) {
+        Throwable rootCause = e;
+        Throwable rootCauseCheck = null;
+        // Extra aggressive rootCause finding
+        do {
+            try {
+                rootCauseCheck = (Throwable)IntrospectionUtils.getProperty
+                                            (rootCause, "rootCause");
+                if (rootCauseCheck!=null)
+                    rootCause = rootCauseCheck;
+
+            } catch (ClassCastException ex) {
+                rootCauseCheck = null;
+            }
+        } while (rootCauseCheck != null);
+        return rootCause;
+    }
+
+
     /**
      * Refuse to add a child Container, because Wrappers are the lowest level
      * of the Container hierarchy.
@@ -960,7 +999,7 @@ public class StandardWrapper
             // Load the specified servlet class from the appropriate class loader
             Class classClass = null;
             try {
-                if (System.getSecurityManager() != null){
+                if (SecurityUtil.isPackageProtectionEnabled()){
                     final ClassLoader fclassLoader = classLoader;
                     final String factualClass = actualClass;
                     try{
@@ -992,8 +1031,6 @@ public class StandardWrapper
                 }
             } catch (ClassNotFoundException e) {
                 unavailable(null);
-
-
                 getServletContext().log( "Error loading " + classLoader + " " + actualClass, e );
                 throw new ServletException
                     (sm.getString("standardWrapper.missingClass", actualClass),
@@ -1043,12 +1080,13 @@ public class StandardWrapper
                                                   servlet);
 
                 if( System.getSecurityManager() != null) {
-                    Class[] classType = new Class[]{ServletConfig.class};
+
                     Object[] args = new Object[]{((ServletConfig)facade)};
                     SecurityUtil.doAsPrivilege("init",
                                                servlet,
                                                classType,
                                                args);
+                    args = null;
                 } else {
                     servlet.init(facade);
                 }
@@ -1062,13 +1100,12 @@ public class StandardWrapper
                     DummyResponse res = new DummyResponse();
 
                     if( System.getSecurityManager() != null) {
-                        Class[] classType = new Class[]{ServletRequest.class,
-                                                        ServletResponse.class};
                         Object[] args = new Object[]{req, res};
                         SecurityUtil.doAsPrivilege("service",
                                                    servlet,
-                                                   classType,
+                                                   classTypeUsedInService,
                                                    args);
+                        args = null;
                     } else {
                         servlet.service(req, res);
                     }
@@ -1522,24 +1559,6 @@ public class StandardWrapper
     }
 
 
-    /**
-     * Log the abbreviated name of this Container for logging messages.
-     */
-    protected String logName() {
-
-        StringBuffer sb = new StringBuffer("StandardWrapper[");
-        if (getParent() != null)
-            sb.append(getParent().getName());
-        else
-            sb.append("null");
-        sb.append(':');
-        sb.append(getName());
-        sb.append(']');
-        return (sb.toString());
-
-    }
-
-
     private Method[] getAllDeclaredMethods(Class c) {
 
         if (c.equals(javax.servlet.http.HttpServlet.class)) {
