@@ -61,7 +61,7 @@ import org.apache.commons.modeler.Registry;
  * <code>org.apache.catalina.core.StandardServer</code> component.</p>
  *
  * @author Amy Roh
- * @version $Revision: 1.31 $ $Date: 2004/09/29 09:55:38 $
+ * @version $Revision: 1.36 $ $Date: 2005/03/25 08:38:48 $
  */
 
 public class MBeanFactory extends BaseModelMBean {
@@ -117,8 +117,6 @@ public class MBeanFactory extends BaseModelMBean {
 
         if (type.equals("org.apache.catalina.core.StandardContext")) {
             return "StandardContext";
-        } else if (type.equals("org.apache.catalina.core.StandardDefaultContext")) {
-            return "DefaultContext";
         } else if (type.equals("org.apache.catalina.core.StandardEngine")) {
             return "Engine";
         } else if (type.equals("org.apache.catalina.core.StandardHost")) {
@@ -143,35 +141,8 @@ public class MBeanFactory extends BaseModelMBean {
         }
         return t;
     }
-
     
    /**
-     * Get Parent DefaultContext to add its child component 
-     * from parent's ObjectName
-     */
-    /*
-    private DefaultContext getDefaultContext(ObjectName pname) 
-        throws Exception {
-        
-        String type = pname.getKeyProperty("type");
-        if (type.equals("DefaultContext")) {
-            Service service = getService(pname);
-            StandardEngine engine = (StandardEngine) service.getContainer();
-            String hostName = pname.getKeyProperty("host");
-            if (hostName!=null) {
-                StandardHost host = 
-                    (StandardHost) engine.findChild(hostName);
-                return host.getDefaultContext();
-            } else {
-                return engine.getDefaultContext();
-            }
-        }
-        return null;
-    }
-    */
-    
-
-    /**
      * Get Parent ContainerBase to add its child component 
      * from parent's ObjectName
      */
@@ -304,10 +275,10 @@ public class MBeanFactory extends BaseModelMBean {
 
         // Create a new DataSourceRealm instance
         DataSourceRealm realm = new DataSourceRealm();
-	realm.setDataSourceName(dataSourceName);
-	realm.setRoleNameCol(roleNameCol);
-	realm.setUserCredCol(userCredCol);
-	realm.setUserNameCol(userNameCol);
+        realm.setDataSourceName(dataSourceName);
+        realm.setRoleNameCol(roleNameCol);
+        realm.setUserCredCol(userCredCol);
+        realm.setUserNameCol(userNameCol);
         realm.setUserRoleTable(userRoleTable);
         realm.setUserTable(userTable);
 
@@ -326,45 +297,6 @@ public class MBeanFactory extends BaseModelMBean {
 
     }
 
-    /**
-     * Create a new DefaultContext.
-     *
-     * @param parent MBean Name of the associated parent component
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-    public String createDefaultContext(String parent)
-        throws Exception {
-        // XXX FIXME
-        // Create a new StandardDefaultContext instance
-        /*
-        StandardDefaultContext context = new StandardDefaultContext();
-
-        // Add the new instance to its parent component
-        ObjectName pname = new ObjectName(parent);
-        Service service = getService(pname);       
-        Engine engine = (Engine) service.getContainer();
-        String hostName = pname.getKeyProperty("host");
-        if (hostName == null) { //if DefaultContext is nested in Engine
-            context.setParent(engine);
-            engine.addDefaultContext(context);
-        } else {                // if DefaultContext is nested in Host
-            Host host = (Host) engine.findChild(hostName);
-            context.setParent(host);
-            host.addDefaultContext(context);
-        }
-
-        // Return the corresponding MBean name
-        ManagedBean managed = registry.findManagedBean("DefaultContext");
-        ObjectName oname =
-            MBeanUtils.createObjectName(managed.getDomain(), context);
-        return (oname.toString());
-        */
-        return null;
-
-    }
-
-
     /**
      * Create a new HttpConnector
      *
@@ -443,10 +375,10 @@ public class MBeanFactory extends BaseModelMBean {
 
         // Create a new JDBCRealm instance
         JDBCRealm realm = new JDBCRealm();
-	realm.setDriverName(driverName);
-	realm.setConnectionName(connectionName);
-	realm.setConnectionPassword(connectionPassword);
-	realm.setConnectionURL(connectionURL);
+        realm.setDriverName(driverName);
+        realm.setConnectionName(connectionName);
+        realm.setConnectionPassword(connectionPassword);
+        realm.setConnectionURL(connectionURL);
 
         // Add the new instance to its parent component
         ObjectName pname = new ObjectName(parent);
@@ -775,8 +707,10 @@ public class MBeanFactory extends BaseModelMBean {
      * @exception Exception if an MBean cannot be created or registered
      */
     public String createStandardHost(String parent, String name,
-                                     String appBase, boolean autoDeploy,
-                                     boolean deployXML, boolean liveDeploy,                                      
+                                     String appBase,
+                                     boolean autoDeploy,
+                                     boolean deployOnStartup,
+                                     boolean deployXML,                                       
                                      boolean unpackWARs,
                                      boolean xmlNamespaceAware,
                                      boolean xmlValidation)
@@ -787,8 +721,8 @@ public class MBeanFactory extends BaseModelMBean {
         host.setName(name);
         host.setAppBase(appBase);
         host.setAutoDeploy(autoDeploy);
+        host.setDeployOnStartup(deployOnStartup);
         host.setDeployXML(deployXML);
-        host.setLiveDeploy(liveDeploy);
         host.setUnpackWARs(unpackWARs);
         host.setXmlNamespaceAware(xmlNamespaceAware);
         host.setXmlValidation(xmlValidation);
@@ -1006,6 +940,13 @@ public class MBeanFactory extends BaseModelMBean {
             Context context = (Context) host.findChild(pathStr);
             // Remove this component from its parent component
             host.removeChild(context);
+            if(context instanceof StandardContext)
+            try {
+                ((StandardContext)context).destroy();
+            } catch (Exception e) {
+                log.warn("Error during context [" + context.getName() + "] destroy ", e);
+           }
+   
         }
 
     }
@@ -1028,7 +969,12 @@ public class MBeanFactory extends BaseModelMBean {
         Host host = (Host) engine.findChild(hostName);
 
         // Remove this component from its parent component
-        engine.removeChild(host);
+        if(host!=null) {
+            if(host instanceof StandardHost)
+                ((StandardHost)host).destroy();
+            else
+                engine.removeChild(host);
+        }
 
     }
 
