@@ -44,7 +44,7 @@ import org.apache.catalina.util.StringManager;
 * @author Craig R. McClanahan
 * @author Carson McDonald
 * @author Ignacio Ortega
-* @version $Revision: 1.10 $
+* @version $Revision: 1.16 $
 */
 
 public class DataSourceRealm
@@ -268,8 +268,13 @@ public class DataSourceRealm
      *  authenticating this username
      */
     public Principal authenticate(String username, String credentials) {
-
-        Connection dbConnection = null;
+    	
+    	// No user - can't possibly authenticate, don't bother the database then
+    	if (username == null) {
+    		return null;
+    	}
+        
+    	Connection dbConnection = null;
 
         try {
 
@@ -279,34 +284,19 @@ public class DataSourceRealm
                 // If the db connection open fails, return "not authenticated"
                 return null;
             }
-
+            
             // Acquire a Principal object for this user
-            Principal principal = authenticate(dbConnection,
-                                               username, credentials);
-
-            if( !dbConnection.getAutoCommit() ) {
-                dbConnection.commit();             
-            }
-
-            // Release the database connection we just used
-            close(dbConnection);
-            dbConnection = null;
-
-            // Return the Principal (if any)
-            return (principal);
-
+            return authenticate(dbConnection, username, credentials);
+            
         } catch (SQLException e) {
-
             // Log the problem for posterity
-            container.getLogger().error(sm.getString("dataSourceRealm.exception"), e);
-
-            // Close the connection so that it gets reopened next time
-            if (dbConnection != null)
-                close(dbConnection);
+            containerLog.error(sm.getString("dataSourceRealm.exception"), e);
 
             // Return "not authenticated" for this request
             return (null);
 
+        } finally {
+        	close(dbConnection);
         }
 
     }
@@ -326,73 +316,36 @@ public class DataSourceRealm
      * @param username Username of the Principal to look up
      * @param credentials Password or other credentials to use in
      *  authenticating this username
-     *
-     * @exception SQLException if a database error occurs
      */
     protected Principal authenticate(Connection dbConnection,
                                                String username,
-                                               String credentials)
-        throws SQLException {
+                                               String credentials) throws SQLException{
 
-        ResultSet rs = null;
-        PreparedStatement stmt = null;
-        ArrayList list = null;
+        String dbCredentials = getPassword(dbConnection, username);
 
-        try {
-            // Look up the user's credentials
-            String dbCredentials = null;
-            stmt = credentials(dbConnection, username);
-            rs = stmt.executeQuery();
-            if (rs.next()) {
-                dbCredentials = rs.getString(1);
-            }
-            rs.close();
-            rs = null;
-            stmt.close();
-            stmt = null;
-            if (dbCredentials == null) {
-                return (null);
-            }
-            dbCredentials = dbCredentials.trim();
-    
-            // Validate the user's credentials
-            boolean validated = false;
-            if (hasMessageDigest()) {
-                // Hex hashes should be compared case-insensitive
-                validated = (digest(credentials).equalsIgnoreCase(dbCredentials));
-            } else
-                validated = (digest(credentials).equals(dbCredentials));
-    
-            if (validated) {
-                if (container.getLogger().isTraceEnabled())
-                    container.getLogger().trace(sm.getString("dataSourceRealm.authenticateSuccess",
-                                     username));
-            } else {
-                if (container.getLogger().isDebugEnabled())
-                    container.getLogger().trace(sm.getString("dataSourceRealm.authenticateFailure",
-                                     username));
-                return (null);
-            }
-    
-            // Accumulate the user's roles
-            list = new ArrayList();
-            stmt = roles(dbConnection, username);
-            rs = stmt.executeQuery();
-            while (rs.next()) {
-                String role = rs.getString(1);
-                if(role != null) {
-                    list.add(role.trim());
-                }
-            }
-        } finally {
-            if (rs != null) {
-                rs.close();
-            }
-            if (stmt != null) {
-                stmt.close();
-            }
+        // Validate the user's credentials
+        boolean validated = false;
+        if (hasMessageDigest()) {
+            // Hex hashes should be compared case-insensitive
+            validated = (digest(credentials).equalsIgnoreCase(dbCredentials));
+        } else
+            validated = (digest(credentials).equals(dbCredentials));
+
+        if (validated) {
+            if (containerLog.isTraceEnabled())
+                containerLog.trace(
+                    sm.getString("dataSourceRealm.authenticateSuccess",
+                                 username));
+        } else {
+            if (containerLog.isTraceEnabled())
+                containerLog.trace(
+                    sm.getString("dataSourceRealm.authenticateFailure",
+                                 username));
+            return (null);
         }
 
+        ArrayList list = getRoles(dbConnection, username);
+
         // Create and return a suitable Principal for this user
         return (new GenericPrincipal(this, username, credentials, list));
 
@@ -410,16 +363,24 @@ public class DataSourceRealm
         if (dbConnection == null)
             return;
 
+        // Commit if not auto committed
+        try {
+            if (!dbConnection.getAutoCommit()) {
+                dbConnection.commit();
+            }            
+        } catch (SQLException e) {
+            containerLog.error("Exception committing connection before closing:", e);
+        }
+
         // Close this database connection, and log any errors
         try {
             dbConnection.close();
         } catch (SQLException e) {
-            container.getLogger().error(sm.getString("dataSourceRealm.close"), e); // Just log it here
+            containerLog.error(sm.getString("dataSourceRealm.close"), e); // Just log it here
         }
 
     }
 
-
     /**
      * Open the specified database connection.
      *
@@ -441,11 +402,170 @@ public class DataSourceRealm
 	    return dataSource.getConnection();
         } catch (Exception e) {
             // Log the problem for posterity
-            container.getLogger().error(sm.getString("dataSourceRealm.exception"), e);
+            containerLog.error(sm.getString("dataSourceRealm.exception"), e);
         }  
         return null;
     }
 
+    /**
+     * Return a short name for this Realm implementation.
+     */
+    protected String getName() {
+
+        return (name);
+
+    }
+
+    /**
+     * Return the password associated with the given principal's user name.
+     */
+    protected String getPassword(String username) {
+
+        Connection dbConnection = null;
+
+        // Ensure that we have an open database connection
+        dbConnection = open();
+        if (dbConnection == null) {
+            return null;
+        }
+
+        try {
+        	return getPassword(dbConnection, username);        	
+        } finally {
+            close(dbConnection);
+        }
+    }
+    
+    /**
+     * Return the password associated with the given principal's user name.
+     * @param dbConnection The database connection to be used
+     * @param username Username for which password should be retrieved
+     */
+    protected String getPassword(Connection dbConnection, 
+								 String username) {
+
+        ResultSet rs = null;
+        PreparedStatement stmt = null;
+        String dbCredentials = null;
+
+        try {
+            stmt = credentials(dbConnection, username);
+            rs = stmt.executeQuery();
+            if (rs.next()) {
+                dbCredentials = rs.getString(1);
+            }
+
+            return (dbCredentials != null) ? dbCredentials.trim() : null;
+            
+        } catch(SQLException e) {
+            containerLog.error(
+                    sm.getString("dataSourceRealm.getPassword.exception",
+                                 username));
+        } finally {
+        	try {
+	            if (rs != null) {
+	                rs.close();
+	            }
+	            if (stmt != null) {
+	                stmt.close();
+	            }
+        	} catch (SQLException e) {
+                    containerLog.error(
+                        sm.getString("dataSourceRealm.getPassword.exception",
+        		             username));
+        		
+        	}
+        }
+        
+        return null;
+    }
+
+
+    /**
+     * Return the Principal associated with the given user name.
+     */
+    protected Principal getPrincipal(String username) {
+    	Connection dbConnection = open();
+        if (dbConnection == null) {
+            return new GenericPrincipal(this,username, null, null);
+        }
+        try {
+        	return (new GenericPrincipal(this,
+        			username,
+					getPassword(dbConnection, username),
+					getRoles(dbConnection, username)));
+        } finally {
+        	close(dbConnection);
+        }
+
+    }
+
+    /**
+     * Return the roles associated with the given user name.
+     * @param username Username for which roles should be retrieved
+     */
+    protected ArrayList getRoles(String username) {
+
+        Connection dbConnection = null;
+
+        // Ensure that we have an open database connection
+        dbConnection = open();
+        if (dbConnection == null) {
+            return null;
+        }
+
+        try {
+            return getRoles(dbConnection, username);
+        } finally {
+        	close(dbConnection);
+        }
+    }
+    
+    /**
+     * Return the roles associated with the given user name
+     * @param dbConnection The database connection to be used
+     * @param username Username for which roles should be retrieved
+     */
+    protected ArrayList getRoles(Connection dbConnection,
+                                     String username) {
+    	
+        ResultSet rs = null;
+        PreparedStatement stmt = null;
+        ArrayList list = null;
+    	
+        try {
+    		stmt = roles(dbConnection, username);
+    		rs = stmt.executeQuery();
+    		list = new ArrayList();
+    		
+    		while (rs.next()) {
+    			String role = rs.getString(1);
+    			if (role != null) {
+    				list.add(role.trim());
+    			}
+    		}
+    		return list;
+    	} catch(SQLException e) {
+            containerLog.error(
+                sm.getString("dataSourceRealm.getRoles.exception", username));
+        }
+    	finally {
+        	try {
+	            if (rs != null) {
+	                rs.close();
+	            }
+	            if (stmt != null) {
+	                stmt.close();
+	            }
+        	} catch (SQLException e) {
+                    containerLog.error(
+                        sm.getString("dataSourceRealm.getRoles.exception",
+                                     username));
+        	}
+        }
+    	
+    	return null;
+    }
 
     /**
      * Return a PreparedStatement configured to perform the SELECT required
@@ -467,39 +587,7 @@ public class DataSourceRealm
         return (credentials);
 
     }
-
-
-    /**
-     * Return a short name for this Realm implementation.
-     */
-    protected String getName() {
-
-        return (name);
-
-    }
-
-
-    /**
-     * Return the password associated with the given principal's user name.
-     */
-    protected String getPassword(String username) {
-
-        return (null);
-
-    }
-
-
-    /**
-     * Return the Principal associated with the given user name.
-     */
-    protected Principal getPrincipal(String username) {
-
-        return (null);
-
-    }
-
-
-
+    
     /**
      * Return a PreparedStatement configured to perform the SELECT required
      * to retrieve user roles for the specified username.
@@ -520,7 +608,6 @@ public class DataSourceRealm
 
     }
 
-
     // ------------------------------------------------------ Lifecycle Methods
 
 
