@@ -44,7 +44,7 @@ import org.apache.catalina.util.StringManager;
 * @author Craig R. McClanahan
 * @author Carson McDonald
 * @author Ignacio Ortega
-* @version $Revision: 1.8 $ $Date: 2004/06/23 13:51:37 $
+* @version $Revision: 1.14 $ $Date: 2005/03/08 23:03:01 $
 */
 
 public class JDBCRealm
@@ -354,7 +354,7 @@ public class JDBCRealm
             } catch (SQLException e) {
 
                 // Log the problem for posterity
-                container.getLogger().error(sm.getString("jdbcRealm.exception"), e);
+                containerLog.error(sm.getString("jdbcRealm.exception"), e);
 
                 // Close the connection so that it gets reopened next time
                 if (dbConnection != null)
@@ -385,81 +385,44 @@ public class JDBCRealm
      * @param username Username of the Principal to look up
      * @param credentials Password or other credentials to use in
      *  authenticating this username
-     *
-     * @exception SQLException if a database error occurs
      */
     public synchronized Principal authenticate(Connection dbConnection,
                                                String username,
-                                               String credentials)
-        throws SQLException {
+                                               String credentials) {
+
+        // No user - can't possibly authenticate
+        if (username == null) {
+            return (null);
+        }
 
         // Look up the user's credentials
-        String dbCredentials = null;
-        PreparedStatement stmt = null;
-        ResultSet rs = null;
+        String dbCredentials = getPassword(username);
 
-        try {
-            stmt = credentials(dbConnection, username);
-            rs = stmt.executeQuery();
-
-            if (rs.next()) {
-                dbCredentials = rs.getString(1);
-            }
-            rs.close();
-            rs = null;
-            if (dbCredentials == null) {
-                return (null);
-            }
-
-            dbCredentials = dbCredentials.trim();
-
-
-            // Validate the user's credentials
-            boolean validated = false;
-            if (hasMessageDigest()) {
-                // Hex hashes should be compared case-insensitive
-                validated = (digest(credentials).equalsIgnoreCase(dbCredentials));
-            } else {
-                validated = (digest(credentials).equals(dbCredentials));
-            }
-
-            if (validated) {
-                if (container.getLogger().isTraceEnabled())
-                    container.getLogger().trace(sm.getString("jdbcRealm.authenticateSuccess",
-                                     username));
-            } else {
-                if (container.getLogger().isTraceEnabled())
-                    container.getLogger().trace(sm.getString("jdbcRealm.authenticateFailure",
-                                     username));
-                return (null);
-            }
-
-            // Accumulate the user's roles
-            ArrayList roleList = new ArrayList();
-            stmt = roles(dbConnection, username);
-            rs = stmt.executeQuery();
-            while (rs.next()) {
-                String role = rs.getString(1);
-                if (null!=role) {
-                    roleList.add(role.trim());
-                }
-            }
-            rs.close();
-            rs = null;
-
-            // Create and return a suitable Principal for this user
-            return (new GenericPrincipal(this, username, credentials, roleList));
-        } finally {
-            if (rs!=null) {
-                try {
-                    rs.close();
-                } catch(SQLException e) {
-                    container.getLogger().warn(sm.getString("jdbcRealm.abnormalCloseResultSet"));
-                }
-            }
-            dbConnection.commit();
+        // Validate the user's credentials
+        boolean validated = false;
+        if (hasMessageDigest()) {
+            // Hex hashes should be compared case-insensitive
+            validated = (digest(credentials).equalsIgnoreCase(dbCredentials));
+        } else {
+            validated = (digest(credentials).equals(dbCredentials));
         }
 
+        if (validated) {
+            if (containerLog.isTraceEnabled())
+                containerLog.trace(sm.getString("jdbcRealm.authenticateSuccess",
+                                                username));
+        } else {
+            if (containerLog.isTraceEnabled())
+                containerLog.trace(sm.getString("jdbcRealm.authenticateFailure",
+                                                username));
+            return (null);
+        }
+
+        ArrayList roles = getRoles(username);
+        
+        // Create and return a suitable Principal for this user
+        return (new GenericPrincipal(this, username, credentials, roles));
+
     }
 
 
@@ -495,7 +458,7 @@ public class JDBCRealm
         try {
             dbConnection.close();
         } catch (SQLException e) {
-            container.getLogger().warn(sm.getString("jdbcRealm.close"), e); // Just log it here
+            containerLog.warn(sm.getString("jdbcRealm.close"), e); // Just log it here
         } finally {
            this.dbConnection = null;
         }
@@ -553,8 +516,68 @@ public class JDBCRealm
      */
     protected String getPassword(String username) {
 
-        return (null);
+        // Look up the user's credentials
+        String dbCredentials = null;
+        PreparedStatement stmt = null;
+        ResultSet rs = null;
 
+        // Number of tries is the numebr of attempts to connect to the database
+        // during this login attempt (if we need to open the database)
+        // This needs rewritten wuth better pooling support, the existing code
+        // needs signature changes since the Prepared statements needs cached
+        // with the connections.
+        // The code below will try twice if there is a SQLException so the
+        // connection may try to be opened again. On normal conditions (including
+        // invalid login - the above is only used once.
+        int numberOfTries = 2;
+        while (numberOfTries>0) {
+            try {
+                
+                // Ensure that we have an open database connection
+                open();
+                
+                try {
+                    stmt = credentials(dbConnection, username);
+                    rs = stmt.executeQuery();
+                    
+                    if (rs.next()) {
+                        dbCredentials = rs.getString(1);
+                    }
+                    rs.close();
+                    rs = null;
+                    if (dbCredentials == null) {
+                        return (null);
+                    }
+                    
+                    dbCredentials = dbCredentials.trim();
+                    return dbCredentials;
+                    
+                } finally {
+                    if (rs!=null) {
+                        try {
+                            rs.close();
+                        } catch(SQLException e) {
+                            containerLog.warn(sm.getString("jdbcRealm.abnormalCloseResultSet"));
+                        }
+                    }
+                    dbConnection.commit();
+                }
+                
+            } catch (SQLException e) {
+                
+                // Log the problem for posterity
+                containerLog.error(sm.getString("jdbcRealm.exception"), e);
+                
+                // Close the connection so that it gets reopened next time
+                if (dbConnection != null)
+                    close(dbConnection);
+                
+            }
+            
+            numberOfTries--;
+        }
+        
+        return (null);
     }
 
 
@@ -563,11 +586,83 @@ public class JDBCRealm
      */
     protected Principal getPrincipal(String username) {
 
-        return (null);
+        return (new GenericPrincipal(this,
+                                     username,
+                                     getPassword(username),
+                                     getRoles(username)));
 
     }
 
 
+    /**
+     * Return the roles associated with the gven user name.
+     */
+    protected ArrayList getRoles(String username) {
+        
+        PreparedStatement stmt = null;
+        ResultSet rs = null;
+
+        // Number of tries is the numebr of attempts to connect to the database
+        // during this login attempt (if we need to open the database)
+        // This needs rewritten wuth better pooling support, the existing code
+        // needs signature changes since the Prepared statements needs cached
+        // with the connections.
+        // The code below will try twice if there is a SQLException so the
+        // connection may try to be opened again. On normal conditions (including
+        // invalid login - the above is only used once.
+        int numberOfTries = 2;
+        while (numberOfTries>0) {
+            try {
+                
+                // Ensure that we have an open database connection
+                open();
+                
+                try {
+                    // Accumulate the user's roles
+                    ArrayList roleList = new ArrayList();
+                    stmt = roles(dbConnection, username);
+                    rs = stmt.executeQuery();
+                    while (rs.next()) {
+                        String role = rs.getString(1);
+                        if (null!=role) {
+                            roleList.add(role.trim());
+                        }
+                    }
+                    rs.close();
+                    rs = null;
+                    
+                    return (roleList);
+                    
+                } finally {
+                    if (rs!=null) {
+                        try {
+                            rs.close();
+                        } catch(SQLException e) {
+                            containerLog.warn(sm.getString("jdbcRealm.abnormalCloseResultSet"));
+                        }
+                    }
+                    dbConnection.commit();
+                }
+                
+            } catch (SQLException e) {
+                
+                // Log the problem for posterity
+                containerLog.error(sm.getString("jdbcRealm.exception"), e);
+                
+                // Close the connection so that it gets reopened next time
+                if (dbConnection != null)
+                    close(dbConnection);
+                
+            }
+            
+            numberOfTries--;
+        }
+        
+        return (null);
+        
+    }
+    
+    
     /**
      * Open (if necessary) and return a database connection for use by
      * this Realm.
@@ -662,7 +757,7 @@ public class JDBCRealm
         try {
             open();
         } catch (SQLException e) {
-            container.getLogger().error(sm.getString("jdbcRealm.open"), e);
+            containerLog.error(sm.getString("jdbcRealm.open"), e);
         }
 
         // Perform normal superclass initialization
