@@ -14,20 +14,19 @@
  * limitations under the License.
  */
 
-
-
-
 package org.apache.catalina.realm;
 
-
+import java.io.IOException;
 import java.security.Principal;
 import java.text.MessageFormat;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Hashtable;
 import java.util.List;
 
 import javax.naming.Context;
 import javax.naming.CommunicationException;
+import javax.naming.CompositeName;
 import javax.naming.InvalidNameException;
 import javax.naming.NameNotFoundException;
 import javax.naming.NamingEnumeration;
@@ -43,7 +42,8 @@ import javax.naming.directory.SearchControls;
 import javax.naming.directory.SearchResult;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.util.Base64;
-
+import org.apache.tomcat.util.buf.ByteChunk;
+import org.apache.tomcat.util.buf.CharChunk;
 
 /**
  * <p>Implementation of <strong>Realm</strong> that works with a directory
@@ -143,13 +143,13 @@ import org.apache.catalina.util.Base64;
  * <p><strong>WARNING</strong> - There is a reported bug against the Netscape
  * provider code (com.netscape.jndi.ldap.LdapContextFactory) with respect to
  * successfully authenticated a non-existing user. The
- * report is here: http://nagoya.apache.org/bugzilla/show_bug.cgi?id=11210 .
+ * report is here: http://issues.apache.org/bugzilla/show_bug.cgi?id=11210 .
  * With luck, Netscape has updated their provider code and this is not an
  * issue. </p>
  *
  * @author John Holman
  * @author Craig R. McClanahan
- * @version $Revision: 1.15 $ $Date: 2004/10/10 20:43:49 $
+ * @version $Revision: 1.23 $ $Date: 2005/03/25 08:29:32 $
  */
 
 public class JNDIRealm extends RealmBase {
@@ -809,16 +809,8 @@ public class JNDIRealm extends RealmBase {
 
             } catch (CommunicationException e) {
 
-
-                // If contains the work closed. Then assume socket is closed.
-                // If message is null, assume the worst and allow the
-                // connection to be closed.
-                if (e.getMessage()!=null &&
-                    e.getMessage().indexOf("closed") < 0)
-                    throw(e);
-
                 // log the exception so we know it's there.
-                container.getLogger().error(sm.getString("jndiRealm.exception"), e);
+                containerLog.warn(sm.getString("jndiRealm.exception"), e);
 
                 // close the connection so we know it will be reopened.
                 if (context != null)
@@ -842,7 +834,7 @@ public class JNDIRealm extends RealmBase {
         } catch (NamingException e) {
 
             // Log the problem for posterity
-            container.getLogger().error(sm.getString("jndiRealm.exception"), e);
+            containerLog.error(sm.getString("jndiRealm.exception"), e);
 
             // Close the connection so that it gets reopened next time
             if (context != null)
@@ -901,7 +893,7 @@ public class JNDIRealm extends RealmBase {
                         }
                     } catch (InvalidNameException ine) {
                         // Log the problem for posterity
-                        container.getLogger().warn(sm.getString("jndiRealm.exception"), ine);
+                        containerLog.warn(sm.getString("jndiRealm.exception"), ine);
                         // ignore; this is probably due to a name not fitting
                         // the search path format exactly, as in a fully-
                         // qualified name being munged into a search path
@@ -1073,7 +1065,8 @@ public class JNDIRealm extends RealmBase {
 
         // Check no further entries were found
         if (results.hasMore()) {
-            container.getLogger().info("username " + username + " has multiple entries");
+            if(containerLog.isInfoEnabled())
+                containerLog.info("username " + username + " has multiple entries");
             return (null);
         }
 
@@ -1081,13 +1074,16 @@ public class JNDIRealm extends RealmBase {
         NameParser parser = context.getNameParser("");
         Name contextName = parser.parse(context.getNameInNamespace());
         Name baseName = parser.parse(userBase);
-        Name entryName = parser.parse(result.getName());
+
+        // Bugzilla 32267
+        Name entryName = parser.parse(new CompositeName(result.getName()).get(0));
+
         Name name = contextName.addAll(baseName);
         name = name.addAll(entryName);
         String dn = name.toString();
 
-        if (container.getLogger().isTraceEnabled())
-            container.getLogger().trace("  entry found for " + username + " with dn " + dn);
+        if (containerLog.isTraceEnabled())
+            containerLog.trace("  entry found for " + username + " with dn " + dn);
 
         // Get the entry's attributes
         Attributes attrs = result.getAttributes();
@@ -1136,12 +1132,12 @@ public class JNDIRealm extends RealmBase {
              validated = compareCredentials(context, user, credentials);
          }
 
-         if (container.getLogger().isTraceEnabled()) {
+         if (containerLog.isTraceEnabled()) {
              if (validated) {
-                 container.getLogger().trace(sm.getString("jndiRealm.authenticateSuccess",
+                 containerLog.trace(sm.getString("jndiRealm.authenticateSuccess",
                                   user.username));
              } else {
-                 container.getLogger().trace(sm.getString("jndiRealm.authenticateFailure",
+                 containerLog.trace(sm.getString("jndiRealm.authenticateFailure",
                                   user.username));
              }
          }
@@ -1173,8 +1169,8 @@ public class JNDIRealm extends RealmBase {
             return (false);
 
         // Validate the credentials specified by the user
-        if (container.getLogger().isTraceEnabled())
-            container.getLogger().trace("  validating credentials");
+        if (containerLog.isTraceEnabled())
+            containerLog.trace("  validating credentials");
 
         boolean validated = false;
         if (hasMessageDigest()) {
@@ -1191,6 +1187,44 @@ public class JNDIRealm extends RealmBase {
                         new String(Base64.encode(md.digest()));
                     validated = password.equals(digestedPassword);
                 }
+            } else if (password.startsWith("{SSHA}")) {
+                // Bugzilla 32938
+                /* sync since super.digest() does this same thing */
+                synchronized (this) {
+                    password = password.substring(6);
+
+                    md.reset();
+                    md.update(credentials.getBytes());
+
+                    // Decode stored password.
+                    ByteChunk pwbc = new ByteChunk(password.length());
+                    try {
+                        pwbc.append(password.getBytes(), 0, password.length());
+                    } catch (IOException e) {
+                        // Should never happen
+                        containerLog.error("Could not append password bytes to chunk: ", e);
+                    }
+
+                    CharChunk decoded = new CharChunk();
+                    Base64.decode(pwbc, decoded);
+                    char[] pwarray = decoded.getBuffer();
+
+                    // Split decoded password into hash and salt.
+                    final int saltpos = 20;
+                    byte[] hash = new byte[saltpos];
+                    for (int i=0; i< hash.length; i++) {
+                        hash[i] = (byte) pwarray[i];
+                    }
+
+                    byte[] salt = new byte[pwarray.length - saltpos];
+                    for (int i=0; i< salt.length; i++)
+                        salt[i] = (byte)pwarray[i+saltpos];
+
+                    md.update(salt);
+                    byte[] dp = md.digest();
+
+                    validated = Arrays.equals(dp, hash);
+                } // End synchronized(this) block
             } else {
                 // Hex hashes should be compared case-insensitive
                 validated = (digest(credentials).equalsIgnoreCase(password));
@@ -1226,8 +1260,8 @@ public class JNDIRealm extends RealmBase {
              return (false);
 
          // Validate the credentials specified by the user
-         if (container.getLogger().isTraceEnabled()) {
-             container.getLogger().trace("  validating credentials by binding as the user");
+         if (containerLog.isTraceEnabled()) {
+             containerLog.trace("  validating credentials by binding as the user");
         }
 
         // Set up security environment to bind as the user
@@ -1237,15 +1271,15 @@ public class JNDIRealm extends RealmBase {
         // Elicit an LDAP bind operation
         boolean validated = false;
         try {
-            if (container.getLogger().isTraceEnabled()) {
-                container.getLogger().trace("  binding as "  + dn);
+            if (containerLog.isTraceEnabled()) {
+                containerLog.trace("  binding as "  + dn);
             }
             attr = context.getAttributes("", null);
             validated = true;
         }
         catch (AuthenticationException e) {
-            if (container.getLogger().isTraceEnabled()) {
-                container.getLogger().trace("  bind attempt failed");
+            if (containerLog.isTraceEnabled()) {
+                containerLog.trace("  bind attempt failed");
             }
         }
 
@@ -1292,8 +1326,8 @@ public class JNDIRealm extends RealmBase {
         if (dn == null || username == null)
             return (null);
 
-        if (container.getLogger().isTraceEnabled())
-            container.getLogger().trace("  getRoles(" + dn + ")");
+        if (containerLog.isTraceEnabled())
+            containerLog.trace("  getRoles(" + dn + ")");
 
         // Start with roles retrieved from the user entry
         ArrayList list = user.roles;
@@ -1328,13 +1362,13 @@ public class JNDIRealm extends RealmBase {
         }
 
 
-        if (container.getLogger().isTraceEnabled()) {
+        if (containerLog.isTraceEnabled()) {
             if (list != null) {
-                container.getLogger().trace("  Returning " + list.size() + " roles");
+                containerLog.trace("  Returning " + list.size() + " roles");
                 for (int i=0; i<list.size(); i++)
-                    container.getLogger().trace(  "  Found role " + list.get(i));
+                    containerLog.trace(  "  Found role " + list.get(i));
             } else {
-                container.getLogger().trace("  getRoles about to return null ");
+                containerLog.trace("  getRoles about to return null ");
             }
         }
 
@@ -1353,8 +1387,8 @@ public class JNDIRealm extends RealmBase {
     private String getAttributeValue(String attrId, Attributes attrs)
         throws NamingException {
 
-        if (container.getLogger().isTraceEnabled())
-            container.getLogger().trace("  retrieving attribute " + attrId);
+        if (containerLog.isTraceEnabled())
+            containerLog.trace("  retrieving attribute " + attrId);
 
         if (attrId == null || attrs == null)
             return null;
@@ -1390,8 +1424,8 @@ public class JNDIRealm extends RealmBase {
                                          ArrayList values)
         throws NamingException{
 
-        if (container.getLogger().isTraceEnabled())
-            container.getLogger().trace("  retrieving values for attribute " + attrId);
+        if (containerLog.isTraceEnabled())
+            containerLog.trace("  retrieving values for attribute " + attrId);
         if (attrId == null || attrs == null)
             return values;
         if (values == null)
@@ -1421,11 +1455,11 @@ public class JNDIRealm extends RealmBase {
 
         // Close our opened connection
         try {
-            if (container.getLogger().isDebugEnabled())
-                container.getLogger().debug("Closing directory context");
+            if (containerLog.isDebugEnabled())
+                containerLog.debug("Closing directory context");
             context.close();
         } catch (NamingException e) {
-            container.getLogger().error(sm.getString("jndiRealm.close"), e);
+            containerLog.error(sm.getString("jndiRealm.close"), e);
         }
         this.context = null;
 
@@ -1485,7 +1519,7 @@ public class JNDIRealm extends RealmBase {
             connectionAttempt = 1;
 
             // log the first exception.
-            container.getLogger().warn(sm.getString("jndiRealm.exception"), e);
+            containerLog.warn(sm.getString("jndiRealm.exception"), e);
 
             // Try connecting to the alternate url.
             context = new InitialDirContext(getDirectoryContextEnvironment());
@@ -1512,10 +1546,10 @@ public class JNDIRealm extends RealmBase {
         Hashtable env = new Hashtable();
 
         // Configure our directory context environment.
-        if (container.getLogger().isDebugEnabled() && connectionAttempt == 0)
-            container.getLogger().debug("Connecting to URL " + connectionURL);
-        else if (container.getLogger().isDebugEnabled() && connectionAttempt > 0)
-            container.getLogger().debug("Connecting to URL " + alternateURL);
+        if (containerLog.isDebugEnabled() && connectionAttempt == 0)
+            containerLog.debug("Connecting to URL " + connectionURL);
+        else if (containerLog.isDebugEnabled() && connectionAttempt > 0)
+            containerLog.debug("Connecting to URL " + alternateURL);
         env.put(Context.INITIAL_CONTEXT_FACTORY, contextFactory);
         if (connectionName != null)
             env.put(Context.SECURITY_PRINCIPAL, connectionName);
