@@ -60,7 +60,7 @@ import org.apache.commons.modeler.Registry;
  * location) are identical to those currently supported by Tomcat 3.X.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 1.43 $ $Date: 2004/10/27 16:38:41 $
+ * @version $Revision: 1.50 $ $Date: 2005/03/25 08:29:34 $
  */
 
 public abstract class RealmBase
@@ -77,6 +77,12 @@ public abstract class RealmBase
     protected Container container = null;
 
 
+    /**
+     * Container log
+     */
+    protected Log containerLog = null;
+
+
     /**
      * Digest algorithm used in storing passwords in a non-plaintext format.
      * Valid values are those accepted for the algorithm name by the
@@ -169,6 +175,7 @@ public abstract class RealmBase
 
         Container oldContainer = this.container;
         this.container = container;
+        this.containerLog = container.getLogger();
         support.firePropertyChange("container", oldContainer, this.container);
 
     }
@@ -283,17 +290,15 @@ public abstract class RealmBase
             validated = serverCredentials.equals(credentials);
         }
         if(! validated ) {
-            if (container.getLogger().isTraceEnabled()) {
-                container.getLogger().
-                    trace(sm.getString("realmBase.authenticateFailure",
-                                 username));
+            if (containerLog.isTraceEnabled()) {
+                containerLog.trace(sm.getString("realmBase.authenticateFailure",
+                                                username));
             }
             return null;
         }
-        if (container.getLogger().isTraceEnabled()) {
-            container.getLogger().
-                trace(sm.getString("realmBase.authenticateSuccess",
-                             username));
+        if (containerLog.isTraceEnabled()) {
+            containerLog.trace(sm.getString("realmBase.authenticateSuccess",
+                                            username));
         }
 
         return getPrincipal(username);
@@ -351,8 +356,11 @@ public abstract class RealmBase
             }
         }
 
-        String serverDigest =
-            md5Encoder.encode(md5Helper.digest(valueBytes));
+        String serverDigest = null;
+        // Bugzilla 32137
+        synchronized(md5Helper) {
+            serverDigest = md5Encoder.encode(md5Helper.digest(valueBytes));
+        }
 
         if (log.isDebugEnabled()) {
             log.debug("Digest : " + clientDigest + " Username:" + username 
@@ -694,7 +702,7 @@ public abstract class RealmBase
         LoginConfig config = context.getLoginConfig();
         if ((config != null) &&
             (Constants.FORM_METHOD.equals(config.getAuthMethod()))) {
-            String requestURI = request.getDecodedRequestURI();
+            String requestURI = request.getRequestPathMB().toString();
             String loginPage = context.getPath() + config.getLoginPage();
             if (loginPage.equals(requestURI)) {
                 if (log.isDebugEnabled())
@@ -733,10 +741,12 @@ public abstract class RealmBase
                     response.sendError
                         (HttpServletResponse.SC_FORBIDDEN,
                          sm.getString("realmBase.forbidden"));
-                    if( log.isDebugEnabled() ) log.debug("No roles ");
+                    if( log.isDebugEnabled() )
+                        log.debug("No roles ");
                     return (false); // No listed roles means no access at all
                 } else {
-                    log.debug("Passing all access");
+                    if(log.isDebugEnabled())
+                        log.debug("Passing all access");
                     return (true);
                 }
             } else if (principal == null) {
@@ -785,7 +795,8 @@ public abstract class RealmBase
 
         GenericPrincipal gp = (GenericPrincipal) principal;
         if (!(gp.getRealm() == this)) {
-            log.debug("Different realm " + this + " " + gp.getRealm());//    return (false);
+            if(log.isDebugEnabled())
+                log.debug("Different realm " + this + " " + gp.getRealm());//    return (false);
         }
         boolean result = gp.hasRole(role);
         if (log.isDebugEnabled()) {
@@ -948,7 +959,8 @@ public abstract class RealmBase
 
         // Validate and update our current component state
         if (started) {
-            log.info(sm.getString("realmBase.alreadyStarted"));
+            if(log.isInfoEnabled())
+                log.info(sm.getString("realmBase.alreadyStarted"));
             return;
         }
         if( !initialized ) {
@@ -984,7 +996,8 @@ public abstract class RealmBase
 
         // Validate and update our current component state
         if (!started) {
-            log.info(sm.getString("realmBase.notStarted"));
+            if(log.isInfoEnabled())
+                log.info(sm.getString("realmBase.notStarted"));
             return;
         }
         lifecycle.fireLifecycleEvent(STOP_EVENT, null);
@@ -1003,7 +1016,8 @@ public abstract class RealmBase
         if ( oname!=null ) {   
             try {   
                 Registry.getRegistry(null, null).unregisterComponent(oname); 
-                log.debug( "unregistering realm " + oname );   
+                if(log.isDebugEnabled())
+                    log.debug( "unregistering realm " + oname );   
             } catch( Exception ex ) {   
                 log.error( "Can't unregister realm " + oname, ex);   
             }      
@@ -1092,8 +1106,11 @@ public abstract class RealmBase
             }
         }
 
-        byte[] digest =
-            md5Helper.digest(valueBytes);
+        byte[] digest = null;
+        // Bugzilla 32137
+        synchronized(md5Helper) {
+            digest = md5Helper.digest(valueBytes);
+        }
 
         return md5Encoder.encode(digest);
     }
@@ -1129,8 +1146,10 @@ public abstract class RealmBase
      * @param credentials Password or other credentials to use in
      *  authenticating this username
      * @param algorithm Algorithm used to do the digest
+     * @param encoding Character encoding of the string to digest
      */
-    public final static String Digest(String credentials, String algorithm) {
+    public final static String Digest(String credentials, String algorithm,
+                                      String encoding) {
 
         try {
             // Obtain a new message digest with "digest" encryption
@@ -1139,12 +1158,16 @@ public abstract class RealmBase
 
             // encode the credentials
             // Should use the digestEncoding, but that's not a static field
-            md.update(credentials.getBytes());
+            if (encoding == null) {
+                md.update(credentials.getBytes());
+            } else {
+                md.update(credentials.getBytes(encoding));                
+            }
 
             // Digest the credentials and return as hexadecimal
             return (HexUtils.convert(md.digest()));
         } catch(Exception ex) {
-            ex.printStackTrace();
+            log.error(ex);
             return credentials;
         }
 
@@ -1158,14 +1181,22 @@ public abstract class RealmBase
      */
     public static void main(String args[]) {
 
-        if(args.length > 2 && args[0].equalsIgnoreCase("-a")) {
-            for(int i=2; i < args.length ; i++){
+        String encoding = null;
+        int firstCredentialArg = 2;
+        
+        if (args.length > 4 && args[2].equalsIgnoreCase("-e")) {
+            encoding = args[3];
+            firstCredentialArg = 4;
+        }
+        
+        if(args.length > firstCredentialArg && args[0].equalsIgnoreCase("-a")) {
+            for(int i=firstCredentialArg; i < args.length ; i++){
                 System.out.print(args[i]+":");
-                System.out.println(Digest(args[i], args[1]));
+                System.out.println(Digest(args[i], args[1], encoding));
             }
         } else {
             System.out.println
-                ("Usage: RealmBase -a <algorithm> <credentials>");
+                ("Usage: RealmBase -a <algorithm> [-e <encoding>] <credentials>");
         }
 
     }
@@ -1243,11 +1274,12 @@ public abstract class RealmBase
                             host + path);
                 }
                 if( mserver.isRegistered(parent ))  {
-                    log.debug("Register with " + parent);
+                    if(log.isDebugEnabled())
+                        log.debug("Register with " + parent);
                     mserver.setAttribute(parent, new Attribute("realm", this));
                 }
             } catch (Exception e) {
-                log.info("Parent not available yet: " + parent);  
+                log.error("Parent not available yet: " + parent);  
             }
         }
         
@@ -1257,7 +1289,8 @@ public abstract class RealmBase
                 ContainerBase cb=(ContainerBase)container;
                 oname=new ObjectName(cb.getDomain()+":type=Realm" + cb.getContainerSuffix());
                 Registry.getRegistry(null, null).registerComponent(this, oname, null );
-                log.debug("Register Realm "+oname);
+                if(log.isDebugEnabled())
+                    log.debug("Register Realm "+oname);
             } catch (Throwable e) {
                 log.error( "Can't register " + oname, e);
             }
