@@ -69,7 +69,7 @@ import org.apache.naming.resources.ResourceAttributes;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 1.29 $ $Date: 2004/10/14 23:58:38 $
+ * @version $Revision: 1.31 $ $Date: 2004/12/18 12:04:40 $
  */
 
 public class DefaultServlet
@@ -139,6 +139,12 @@ public class DefaultServlet
     protected ProxyDirContext resources = null;
 
 
+    /**
+     * File encoding to be used when reading static files. If none is specified
+     * the platform default is used.
+     */
+    protected String fileEncoding = null;
+    
     // ----------------------------------------------------- Static Initializer
 
 
@@ -228,6 +234,12 @@ public class DefaultServlet
         } catch (Throwable t) {
             ;
         }
+        try {
+            value = getServletConfig().getInitParameter("fileEncoding");
+            fileEncoding = value;
+        } catch (Throwable t) {
+            ;
+        }
 
         globalXsltFile = getServletConfig().getInitParameter("globalXsltFile");
         localXsltFile = getServletConfig().getInitParameter("localXsltFile");
@@ -657,7 +669,16 @@ public class DefaultServlet
                                             Globals.INCLUDE_REQUEST_URI_ATTR);
             if (requestUri == null) {
                 requestUri = request.getRequestURI();
+            } else {
+                // We're included, and the response.sendError() below is going
+                // to be ignored by the resource that is including us.
+                // Therefore, the only way we can let the including resource
+                // know is by including warning message in response
+                response.getWriter().write(
+                    sm.getString("defaultServlet.missingResource",
+                    requestUri));
             }
+
             response.sendError(HttpServletResponse.SC_NOT_FOUND,
                                requestUri);
             return;
@@ -1702,8 +1723,14 @@ public class DefaultServlet
         } else {
             resourceInputStream = is;
         }
-        // FIXME : i18n ?
-        Reader reader = new InputStreamReader(resourceInputStream);
+
+        Reader reader;
+        if (fileEncoding == null) {
+            reader = new InputStreamReader(resourceInputStream);
+        } else {
+            reader = new InputStreamReader(resourceInputStream,
+                                           fileEncoding);
+        }
 
         // Copy the input stream to the output stream
         exception = copyRange(reader, writer);
@@ -1774,7 +1801,15 @@ public class DefaultServlet
         IOException exception = null;
 
         InputStream resourceInputStream = cacheEntry.resource.streamContent();
-        Reader reader = new InputStreamReader(resourceInputStream);
+
+        Reader reader;
+        if (fileEncoding == null) {
+            reader = new InputStreamReader(resourceInputStream);
+        } else {
+            reader = new InputStreamReader(resourceInputStream,
+                                           fileEncoding);
+        }
+
         exception = copyRange(reader, writer, range.start, range.end);
 
         // Clean up the input stream
@@ -1868,7 +1903,14 @@ public class DefaultServlet
         while ( (exception == null) && (ranges.hasMoreElements()) ) {
 
             InputStream resourceInputStream = cacheEntry.resource.streamContent();
-            Reader reader = new InputStreamReader(resourceInputStream);
+            
+            Reader reader;
+            if (fileEncoding == null) {
+                reader = new InputStreamReader(resourceInputStream);
+            } else {
+                reader = new InputStreamReader(resourceInputStream,
+                                               fileEncoding);
+            }
 
             Range currentRange = (Range) ranges.nextElement();
 
