@@ -44,7 +44,7 @@ import java.util.Properties;
  * saved are still subject to being expired based on inactivity.
  *
  * @author Bip Thelin
- * @version $Revision: 1.10 $, $Date: 2004/06/23 13:51:36 $
+ * @version $Revision: 1.13 $, $Date: 2005/01/07 15:09:37 $
  */
 
 public class JDBCStore
@@ -440,9 +440,6 @@ public class JDBCStore
      * @exception IOException if an input/output error occurred
      */
     public String[] keys() throws IOException {
-        String keysSql =
-                "SELECT " + sessionIdCol + " FROM " + sessionTable +
-                " WHERE " + sessionAppCol + " = ?";
         ResultSet rst = null;
         String keys[] = null;
         synchronized (this) {
@@ -455,8 +452,11 @@ public class JDBCStore
                 }
                 try {
                     if (preparedKeysSql == null) {
+                        String keysSql = "SELECT " + sessionIdCol + " FROM "
+                                + sessionTable + " WHERE " + sessionAppCol
+                                + " = ?";
                         preparedKeysSql = _conn.prepareStatement(keysSql);
-                    }
+					}
 
                     preparedKeysSql.setString(1, getName());
                     rst = preparedKeysSql.executeQuery();
@@ -467,6 +467,8 @@ public class JDBCStore
                         }
                     }
                     keys = (String[]) tmpkeys.toArray(new String[tmpkeys.size()]);
+                    // Break out after the finally block
+                    numberOfTries = 0;
                 } catch (SQLException e) {
                     manager.getContainer().getLogger().error(sm.getString(getStoreName() + ".SQLException", e));
                     keys = new String[0];
@@ -500,9 +502,6 @@ public class JDBCStore
      */
     public int getSize() throws IOException {
         int size = 0;
-        String sizeSql =
-                "SELECT COUNT(" + sessionIdCol + ") FROM " + sessionTable +
-                " WHERE " + sessionAppCol + " = ?";
         ResultSet rst = null;
 
         synchronized (this) {
@@ -516,14 +515,19 @@ public class JDBCStore
 
                 try {
                     if (preparedSizeSql == null) {
+                        String sizeSql = "SELECT COUNT(" + sessionIdCol
+                                + ") FROM " + sessionTable + " WHERE "
+                                + sessionAppCol + " = ?";
                         preparedSizeSql = _conn.prepareStatement(sizeSql);
-                    }
+					}
 
                     preparedSizeSql.setString(1, getName());
                     rst = preparedSizeSql.executeQuery();
                     if (rst.next()) {
                         size = rst.getInt(1);
                     }
+                    // Break out after the finally block
+                    numberOfTries = 0;
                 } catch (SQLException e) {
                     manager.getContainer().getLogger().error(sm.getString(getStoreName() + ".SQLException", e));
                     if (dbConnection != null)
@@ -562,11 +566,7 @@ public class JDBCStore
         ObjectInputStream ois = null;
         BufferedInputStream bis = null;
         Container container = manager.getContainer();
-        String loadSql =
-                "SELECT " + sessionIdCol + ", " + sessionDataCol + " FROM " +
-                sessionTable + " WHERE " + sessionIdCol + " = ? AND " +
-                sessionAppCol + " = ?";
-
+ 
         synchronized (this) {
             int numberOfTries = 2;
             while (numberOfTries > 0) {
@@ -577,6 +577,10 @@ public class JDBCStore
 
                 try {
                     if (preparedLoadSql == null) {
+                        String loadSql = "SELECT " + sessionIdCol + ", "
+                                + sessionDataCol + " FROM " + sessionTable
+                                + " WHERE " + sessionIdCol + " = ? AND "
+                                + sessionAppCol + " = ?";
                         preparedLoadSql = _conn.prepareStatement(loadSql);
                     }
 
@@ -607,10 +611,11 @@ public class JDBCStore
                         _session = (StandardSession) manager.createEmptySession();
                         _session.readObjectData(ois);
                         _session.setManager(manager);
-
-                    } else if (manager.getContainer().getLogger().isDebugEnabled()) {
+                      } else if (manager.getContainer().getLogger().isDebugEnabled()) {
                         manager.getContainer().getLogger().debug(getStoreName() + ": No persisted data object found");
                     }
+                    // Break out after the finally block
+                    numberOfTries = 0;
                 } catch (SQLException e) {
                     manager.getContainer().getLogger().error(sm.getString(getStoreName() + ".SQLException", e));
                     if (dbConnection != null)
@@ -649,9 +654,6 @@ public class JDBCStore
      * @exception IOException if an input/output error occurs
      */
     public void remove(String id) throws IOException {
-        String removeSql =
-                "DELETE FROM " + sessionTable + " WHERE " + sessionIdCol +
-                " = ?  AND " + sessionAppCol + " = ?";
 
         synchronized (this) {
             int numberOfTries = 2;
@@ -664,12 +666,17 @@ public class JDBCStore
 
                 try {
                     if (preparedRemoveSql == null) {
+                        String removeSql = "DELETE FROM " + sessionTable
+                                + " WHERE " + sessionIdCol + " = ?  AND "
+                                + sessionAppCol + " = ?";
                         preparedRemoveSql = _conn.prepareStatement(removeSql);
                     }
 
                     preparedRemoveSql.setString(1, id);
                     preparedRemoveSql.setString(2, getName());
                     preparedRemoveSql.execute();
+                    // Break out after the finally block
+                    numberOfTries = 0;
                 } catch (SQLException e) {
                     manager.getContainer().getLogger().error(sm.getString(getStoreName() + ".SQLException", e));
                     if (dbConnection != null)
@@ -692,8 +699,6 @@ public class JDBCStore
      * @exception IOException if an input/output error occurs
      */
     public void clear() throws IOException {
-        String clearSql =
-                "DELETE FROM " + sessionTable + " WHERE " + sessionAppCol + " = ?";
 
         synchronized (this) {
             int numberOfTries = 2;
@@ -705,11 +710,15 @@ public class JDBCStore
 
                 try {
                     if (preparedClearSql == null) {
+                        String clearSql = "DELETE FROM " + sessionTable
+                             + " WHERE " + sessionAppCol + " = ?";
                         preparedClearSql = _conn.prepareStatement(clearSql);
                     }
 
                     preparedClearSql.setString(1, getName());
                     preparedClearSql.execute();
+                    // Break out after the finally block
+                    numberOfTries = 0;
                 } catch (SQLException e) {
                     manager.getContainer().getLogger().error(sm.getString(getStoreName() + ".SQLException", e));
                     if (dbConnection != null)
@@ -729,13 +738,6 @@ public class JDBCStore
      * @exception IOException if an input/output error occurs
      */
     public void save(Session session) throws IOException {
-        String saveSql =
-                "INSERT INTO " + sessionTable + " (" + sessionIdCol + ", " +
-                sessionAppCol + ", " +
-                sessionDataCol + ", " +
-                sessionValidCol + ", " +
-                sessionMaxInactiveCol + ", " +
-                sessionLastAccessedCol + ") VALUES (?, ?, ?, ?, ?, ?)";
         ObjectOutputStream oos = null;
         ByteArrayOutputStream bos = null;
         ByteArrayInputStream bis = null;
@@ -767,8 +769,14 @@ public class JDBCStore
                     in = new BufferedInputStream(bis, size);
 
                     if (preparedSaveSql == null) {
-                        preparedSaveSql = _conn.prepareStatement(saveSql);
-                    }
+                        String saveSql = "INSERT INTO " + sessionTable + " ("
+                           + sessionIdCol + ", " + sessionAppCol + ", "
+                           + sessionDataCol + ", " + sessionValidCol
+                           + ", " + sessionMaxInactiveCol + ", "
+                           + sessionLastAccessedCol
+                           + ") VALUES (?, ?, ?, ?, ?, ?)";
+                       preparedSaveSql = _conn.prepareStatement(saveSql);
+					}
 
                     preparedSaveSql.setString(1, session.getId());
                     preparedSaveSql.setString(2, getName());
@@ -777,6 +785,8 @@ public class JDBCStore
                     preparedSaveSql.setInt(5, session.getMaxInactiveInterval());
                     preparedSaveSql.setLong(6, session.getLastAccessedTime());
                     preparedSaveSql.execute();
+                    // Break out after the finally block
+                    numberOfTries = 0;
                 } catch (SQLException e) {
                     manager.getContainer().getLogger().error(sm.getString(getStoreName() + ".SQLException", e));
                     if (dbConnection != null)
@@ -911,9 +921,8 @@ public class JDBCStore
         } catch (Throwable f) {
             ;
         }
-        this.preparedClearSql = null;
-
-        try {
+         
+		try {
             preparedRemoveSql.close();
         } catch (Throwable f) {
             ;
