@@ -29,6 +29,7 @@ import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.PrivilegedAction;
 import java.util.Date;
+import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Random;
@@ -55,7 +56,7 @@ import org.apache.commons.modeler.Registry;
  * be subclassed to create more sophisticated Manager implementations.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 1.36 $ $Date: 2004/09/24 14:55:13 $
+ * @version $Revision: 1.40 $ $Date: 2005/02/15 21:10:35 $
  */
 
 public abstract class ManagerBase implements Manager, MBeanRegistration {
@@ -180,6 +181,25 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
     protected int duplicates=0;
 
     protected boolean initialized=false;
+    
+    /**
+     * Processing time during session expiration.
+     */
+    protected long processingTime = 0;
+
+    /**
+     * Iteration count for background processing.
+     */
+    private int count = 0;
+
+
+    /**
+     * Frequency of the session expiration, and related manager operations.
+     * Manager operations will be done once for the specified amount of
+     * backgrondProcess calls (ie, the lower the amount, the most often the
+     * checks will occur).
+     */
+    protected int processExpiresFrequency = 6;
 
     /**
      * The string manager for this package.
@@ -257,8 +277,6 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
         Container oldContainer = this.container;
         this.container = container;
         support.firePropertyChange("container", oldContainer, this.container);
-        // TODO: find a good scheme for the log names
-        //log=LogFactory.getLog("tomcat.manager." + container.getName());
     }
 
 
@@ -515,9 +533,11 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
                         this.random = new java.util.Random();
                         this.random.setSeed(seed);
                     }
-                    long t2=System.currentTimeMillis();
-                    if( (t2-t1) > 100 )
-                        log.debug(sm.getString("managerBase.seeding", randomClass) + " " + (t2-t1));
+                    if(log.isDebugEnabled()) {
+                        long t2=System.currentTimeMillis();
+                        if( (t2-t1) > 100 )
+                            log.debug(sm.getString("managerBase.seeding", randomClass) + " " + (t2-t1));
+                    }
                 }
             }
         }
@@ -571,6 +591,42 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
         this.expiredSessions = expiredSessions;
     }
 
+    public long getProcessingTime() {
+        return processingTime;
+    }
+
+
+    public void setProcessingTime(long processingTime) {
+        this.processingTime = processingTime;
+    }
+    
+    /**
+     * Return the frequency of manager checks.
+     */
+    public int getProcessExpiresFrequency() {
+
+        return (this.processExpiresFrequency);
+
+    }
+
+    /**
+     * Set the manager checks frequency.
+     *
+     * @param processExpiresFrequency the new manager checks frequency
+     */
+    public void setProcessExpiresFrequency(int processExpiresFrequency) {
+
+        if (processExpiresFrequency <= 0) {
+            return;
+        }
+
+        int oldProcessExpiresFrequency = this.processExpiresFrequency;
+        this.processExpiresFrequency = processExpiresFrequency;
+        support.firePropertyChange("processExpiresFrequency",
+                                   new Integer(oldProcessExpiresFrequency),
+                                   new Integer(this.processExpiresFrequency));
+
+    }
 
     // --------------------------------------------------------- Public Methods
 
@@ -579,8 +635,34 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
      * Implements the Manager interface, direct call to processExpires
      */
     public void backgroundProcess() {
+        count = (count + 1) % processExpiresFrequency;
+        if (count == 0)
+            processExpires();
     }
 
+    /**
+     * Invalidate all sessions that have expired.
+     */
+    public void processExpires() {
+
+        long timeNow = System.currentTimeMillis();
+        Session sessions[] = findSessions();
+        int expireHere = 0 ;
+        
+        if(log.isDebugEnabled())
+            log.debug("Start expire sessions " + getName() + " at " + timeNow + " sessioncount " + sessions.length);
+        for (int i = 0; i < sessions.length; i++) {
+            if (!sessions[i].isValid()) {
+                expiredSessions++;
+                expireHere++;
+            }
+        }
+        long timeEnd = System.currentTimeMillis();
+        if(log.isDebugEnabled())
+             log.debug("End expire sessions " + getName() + " processingTime " + (timeEnd - timeNow) + " expired sessions: " + expireHere);
+        processingTime += ( timeEnd - timeNow );
+
+    }
 
     public void destroy() {
         if( oname != null )
@@ -598,6 +680,7 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
                 StandardContext ctx=(StandardContext)this.getContainer();
                 Engine eng=(Engine)ctx.getParent().getParent();
                 domain=ctx.getEngineName();
+                distributable = ctx.getDistributable();
                 StandardHost hst=(StandardHost)ctx.getParent();
                 String path = ctx.getPath();
                 if (path.equals("")) {
@@ -610,7 +693,8 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
                 log.error("Error registering ",e);
             }
         }
-        log.debug("Registering " + oname );
+        if(log.isDebugEnabled())
+            log.debug("Registering " + oname );
                
     }
 
@@ -648,12 +732,31 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
      * id will be assigned by this method, and available via the getId()
      * method of the returned session.  If a new session cannot be created
      * for any reason, return <code>null</code>.
-     *
+     * 
+     * @exception IllegalStateException if a new session cannot be
+     *  instantiated for any reason
+     * @deprecated
+     */
+    public Session createSession() {
+        return createSession(null);
+    }
+    
+    
+    /**
+     * Construct and return a new session object, based on the default
+     * settings specified by this Manager's properties.  The session
+     * id specified will be used as the session id.  
+     * If a new session cannot be created for any reason, return 
+     * <code>null</code>.
+     * 
+     * @param sessionId The session id which should be used to create the
+     *  new session; if <code>null</code>, a new session id will be
+     *  generated
      * @exception IllegalStateException if a new session cannot be
      *  instantiated for any reason
      */
-    public Session createSession() {
-
+    public Session createSession(String sessionId) {
+        
         // Recycle or create a Session instance
         Session session = createEmptySession();
 
@@ -662,15 +765,33 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
         session.setValid(true);
         session.setCreationTime(System.currentTimeMillis());
         session.setMaxInactiveInterval(this.maxInactiveInterval);
-        String sessionId = generateSessionId();
+        if (sessionId == null) {
+            sessionId = generateSessionId();
+            // FIXME: Code to be used in case route replacement is needed
+            /*
+        } else {
+            String jvmRoute = getJvmRoute();
+            if (getJvmRoute() != null) {
+                String requestJvmRoute = null;
+                int index = sessionId.indexOf(".");
+                if (index > 0) {
+                    requestJvmRoute = sessionId
+                            .substring(index + 1, sessionId.length());
+                }
+                if (requestJvmRoute != null && !requestJvmRoute.equals(jvmRoute)) {
+                    sessionId = sessionId.substring(0, index) + "." + jvmRoute;
+                }
+            }
+            */
+        }
         session.setId(sessionId);
         sessionCounter++;
 
         return (session);
 
     }
-
-
+    
+    
     /**
      * Get a session from the recycled ones or create a new empty one.
      * The PersistentManager manager does not need to create session data
@@ -768,7 +889,8 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
                 if( len==bytes.length ) {
                     return;
                 }
-                log.debug("Got " + len + " " + bytes.length );
+                if(log.isDebugEnabled())
+                    log.debug("Got " + len + " " + bytes.length );
             } catch( Exception ex ) {
             }
             devRandomSource=null;
@@ -993,7 +1115,8 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
     public String getSessionAttribute( String sessionId, String key ) {
         Session s=(Session)sessions.get(sessionId);
         if( s==null ) {
-            log.info("Session not found " + sessionId);
+            if(log.isInfoEnabled())
+                log.info("Session not found " + sessionId);
             return null;
         }
         Object o=s.getSession().getAttribute(key);
@@ -1002,10 +1125,47 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
     }
 
 
+    /**
+     * Returns information about the session with the given session id.
+     * 
+     * <p>The session information is organized as a HashMap, mapping 
+     * session attribute names to the String representation of their values.
+     *
+     * @param sessionId Session id
+     * 
+     * @return HashMap mapping session attribute names to the String
+     * representation of their values, or null if no session with the
+     * specified id exists, or if the session does not have any attributes
+     */
+    public HashMap getSession(String sessionId) {
+        Session s = (Session) sessions.get(sessionId);
+        if (s == null) {
+            if (log.isInfoEnabled()) {
+                log.info("Session not found " + sessionId);
+            }
+            return null;
+        }
+
+        Enumeration ee = s.getSession().getAttributeNames();
+        if (ee == null || !ee.hasMoreElements()) {
+            return null;
+        }
+
+        HashMap map = new HashMap();
+        while (ee.hasMoreElements()) {
+            String attrName = (String) ee.nextElement();
+            map.put(attrName, getSessionAttribute(sessionId, attrName));
+        }
+
+        return map;
+    }
+
+
     public void expireSession( String sessionId ) {
         Session s=(Session)sessions.get(sessionId);
         if( s==null ) {
-            log.info("Session not found " + sessionId);
+            if(log.isInfoEnabled())
+                log.info("Session not found " + sessionId);
             return;
         }
         s.expire();
