@@ -34,7 +34,7 @@ import org.apache.catalina.util.LifecycleSupport;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
-
+import org.apache.catalina.security.SecurityUtil;
 /**
  * Extends the <b>ManagerBase</b> class to implement most of the
  * functionality required by a Manager which supports any kind of
@@ -46,7 +46,7 @@ import org.apache.commons.logging.LogFactory;
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Revision: 1.22 $ $Date: 2004/10/23 00:16:08 $
+ * @version $Revision: 1.25 $ $Date: 2004/11/22 16:35:18 $
  */
 
 public abstract class PersistentManagerBase
@@ -130,7 +130,7 @@ public abstract class PersistentManagerBase
     /**
      * The descriptive information about this implementation.
      */
-    private static final String info = "PersistentManagerBase/1.0";
+    private static final String info = "PersistentManagerBase/1.1";
 
 
     /**
@@ -206,32 +206,15 @@ public abstract class PersistentManagerBase
 
     // ------------------------------------------------------------- Properties
 
-
-    /**
-     * Perform the background processes for this Manager
-     */
-    public void backgroundProcess() {
-
-        long timeNow = System.currentTimeMillis();
-
-        this.processExpires();
-        this.processPersistenceChecks();
-        if ((this.getStore() != null)
-            && (this.getStore() instanceof StoreBase)) {
-            ((StoreBase) this.getStore()).processExpires();
-        }
-
-        long timeEnd = System.currentTimeMillis();
-        processingTime += ( timeEnd - timeNow );
-
-    }
+    
+  
 
 
     /**
-     * Indicates how many seconds old a session can get, after its last
-     * use in a request, before it should be backed up to the store. -1
-     * means sessions are not backed up.
-     */
+	 * Indicates how many seconds old a session can get, after its last use in a
+	 * request, before it should be backed up to the store. -1 means sessions
+	 * are not backed up.
+	 */
     public int getMaxIdleBackup() {
 
         return maxIdleBackup;
@@ -248,12 +231,12 @@ public abstract class PersistentManagerBase
      * means sessions are not backed up.
      * <p>
      * Note that this is not a hard limit: sessions are checked
-     * against this age limit periodically according to <b>checkInterval</b>.
+     * against this age limit periodically according to <b>processExpiresFrequency</b>.
      * This value should be considered to indicate when a session is
      * ripe for backing up.
      * <p>
      * So it is possible that a session may be idle for maxIdleBackup +
-     * checkInterval seconds, plus the time it takes to handle other
+     * processExpiresFrequency * engine.backgroundProcessorDelay seconds, plus the time it takes to handle other
      * session expiration, swapping, etc. tasks.
      *
      * @param backup The number of seconds after their last accessed
@@ -331,12 +314,13 @@ public abstract class PersistentManagerBase
 
 
     /**
-     * Set the Container with which this Manager has been associated.  If
-     * it is a Context (the usual case), listen for changes to the session
-     * timeout property.
-     *
-     * @param container The associated Container
-     */
+	 * Set the Container with which this Manager has been associated. If it is a
+	 * Context (the usual case), listen for changes to the session timeout
+	 * property.
+	 * 
+	 * @param container
+	 *            The associated Container
+	 */
     public void setContainer(Container container) {
 
         // De-register from the old Container (if any)
@@ -427,17 +411,6 @@ public abstract class PersistentManagerBase
         this.rejectedSessions = rejectedSessions;
     }
 
-
-    public long getProcessingTime() {
-        return processingTime;
-    }
-
-
-    public void setProcessingTime(long processingTime) {
-        this.processingTime = processingTime;
-    }
-
-
     /**
      * Return the descriptive short name of this Manager implementation.
      */
@@ -539,7 +512,7 @@ public abstract class PersistentManagerBase
             return;
 
         try {     
-            if (System.getSecurityManager() != null){
+            if (SecurityUtil.isPackageProtectionEnabled()){
                 try{
                     AccessController.doPrivileged(new PrivilegedStoreClear());
                 }catch(PrivilegedActionException ex){
@@ -559,22 +532,37 @@ public abstract class PersistentManagerBase
 
 
     /**
-     * Invalidate all sessions that have expired.
+     * Implements the Manager interface, direct call to processExpires and processPersistenceChecks
      */
-    public void processExpires() {
-
+	public void processExpires() {
+		
+        long timeNow = System.currentTimeMillis();
         Session sessions[] = findSessions();
-
+        int expireHere = 0 ;
+        if(log.isDebugEnabled())
+             log.debug("Start expire sessions " + getName() + " at " + timeNow + " sessioncount " + sessions.length);
         for (int i = 0; i < sessions.length; i++) {
-            sessions[i].isValid();
+            if (!sessions[i].isValid()) {
+                expiredSessions++;
+                expireHere++;
+            }
         }
-    }
+        processPersistenceChecks();
+        if ((getStore() != null) && (getStore() instanceof StoreBase)) {
+            ((StoreBase) getStore()).processExpires();
+        }
+        
+        long timeEnd = System.currentTimeMillis();
+        if(log.isDebugEnabled())
+             log.debug("End expire sessions " + getName() + " processingTime " + (timeEnd - timeNow) + " expired sessions: " + expireHere);
+        processingTime += (timeEnd - timeNow);
+ 		
+	}
 
 
     /**
-     * Called by the background thread after active sessions have
-     * been checked for expiration, to allow sessions to be
-     * swapped out, backed up, etc.
+     * Called by the background thread after active sessions have been checked
+     * for expiration, to allow sessions to be swapped out, backed up, etc.
      */
     public void processPersistenceChecks() {
 
@@ -640,9 +628,10 @@ public abstract class PersistentManagerBase
 
         String[] ids = null;
         try {
-            if (System.getSecurityManager() != null){
+            if (SecurityUtil.isPackageProtectionEnabled()){
                 try{
-                    ids = (String[])AccessController.doPrivileged(new PrivilegedStoreKeys());
+                    ids = (String[])
+                        AccessController.doPrivileged(new PrivilegedStoreKeys());
                 }catch(PrivilegedActionException ex){
                     Exception exception = ex.getException();
                     log.error("Exception in the Store during load: "
@@ -698,7 +687,7 @@ public abstract class PersistentManagerBase
      */    
     protected void removeSession(String id){
         try {
-            if (System.getSecurityManager() != null){
+            if (SecurityUtil.isPackageProtectionEnabled()){
                 try{
                     AccessController.doPrivileged(new PrivilegedStoreRemove(id));
                 }catch(PrivilegedActionException ex){
@@ -766,9 +755,10 @@ public abstract class PersistentManagerBase
 
         Session session = null;
         try {
-            if (System.getSecurityManager() != null){
+            if (SecurityUtil.isPackageProtectionEnabled()){
                 try{
-                    session = (Session) AccessController.doPrivileged(new PrivilegedStoreLoad(id));
+                    session = (Session) 
+                      AccessController.doPrivileged(new PrivilegedStoreLoad(id));
                 }catch(PrivilegedActionException ex){
                     Exception exception = ex.getException();
                     log.error("Exception in the Store during swapIn: "
@@ -847,7 +837,7 @@ public abstract class PersistentManagerBase
         }
 
         try {
-            if (System.getSecurityManager() != null){
+            if (SecurityUtil.isPackageProtectionEnabled()){
                 try{
                     AccessController.doPrivileged(new PrivilegedStoreSave(session));
                 }catch(PrivilegedActionException ex){
@@ -1137,5 +1127,4 @@ public abstract class PersistentManagerBase
 
     }
 
-
 }
