@@ -45,7 +45,7 @@ import org.apache.catalina.Session;
 import org.apache.catalina.util.CustomObjectInputStream;
 import org.apache.catalina.util.LifecycleSupport;
 
-
+import org.apache.catalina.security.SecurityUtil;
 /**
  * Standard implementation of the <b>Manager</b> interface that provides
  * simple session persistence across restarts of this component (such as
@@ -58,7 +58,7 @@ import org.apache.catalina.util.LifecycleSupport;
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Revision: 1.25 $ $Date: 2004/10/05 17:12:52 $
+ * @version $Revision: 1.28 $ $Date: 2005/02/07 21:56:32 $
  */
 
 public class StandardManager
@@ -107,21 +107,6 @@ public class StandardManager
     protected LifecycleSupport lifecycle = new LifecycleSupport(this);
 
 
-    /**
-     * Iteration count for background processing.
-     */
-    private int count = 0;
-
-
-    /**
-     * Frequency of the session expiration, and related manager operations.
-     * Manager operations will be done once for the specified amount of
-     * backgrondProcess calls (ie, the lower the amount, the most often the
-     * checks will occur).
-     */
-    protected int processExpiresFrequency = 6;
-
-
     /**
      * The maximum number of active Sessions allowed, or -1 for no limit.
      */
@@ -229,46 +214,6 @@ public class StandardManager
     }
 
 
-    public long getProcessingTime() {
-        return processingTime;
-    }
-
-
-    public void setProcessingTime(long processingTime) {
-        this.processingTime = processingTime;
-    }
-
-
-    /**
-     * Return the frequency of manager checks.
-     */
-    public int getProcessExpiresFrequency() {
-
-        return (this.processExpiresFrequency);
-
-    }
-
-
-    /**
-     * Set the manager checks frequency.
-     *
-     * @param processExpiresFrequency the new manager checks frequency
-     */
-    public void setProcessExpiresFrequency(int processExpiresFrequency) {
-
-        if (processExpiresFrequency <= 0) {
-            return;
-        }
-
-        int oldProcessExpiresFrequency = this.processExpiresFrequency;
-        this.processExpiresFrequency = processExpiresFrequency;
-        support.firePropertyChange("processExpiresFrequency",
-                                   new Integer(oldProcessExpiresFrequency),
-                                   new Integer(this.processExpiresFrequency));
-
-    }
-
-
     /**
      * Set the maximum number of actives Sessions allowed, or -1 for
      * no limit.
@@ -333,7 +278,7 @@ public class StandardManager
      * @exception IllegalStateException if a new session cannot be
      *  instantiated for any reason
      */
-    public Session createSession() {
+    public Session createSession(String sessionId) {
 
         if ((maxActiveSessions >= 0) &&
             (sessions.size() >= maxActiveSessions)) {
@@ -342,7 +287,7 @@ public class StandardManager
                 (sm.getString("standardManager.createSession.ise"));
         }
 
-        return (super.createSession());
+        return (super.createSession(sessionId));
 
     }
 
@@ -357,7 +302,7 @@ public class StandardManager
      * @exception IOException if an input/output error occurs
      */
     public void load() throws ClassNotFoundException, IOException {
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             try{
                 AccessController.doPrivileged( new PrivilegedDoLoad() );
             } catch (PrivilegedActionException ex){
@@ -501,7 +446,7 @@ public class StandardManager
      * @exception IOException if an input/output error occurs
      */
     public void unload() throws IOException {
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             try{
                 AccessController.doPrivileged( new PrivilegedDoUnload() );
             } catch (PrivilegedActionException ex){
@@ -799,32 +744,4 @@ public class StandardManager
         return (file);
 
     }
-
-
-    /**
-     * Invalidate all sessions that have expired.
-     */
-    public void processExpires() {
-
-        long timeNow = System.currentTimeMillis();
-        Session sessions[] = findSessions();
-
-        for (int i = 0; i < sessions.length; i++) {
-            sessions[i].isValid();
-        }
-        long timeEnd = System.currentTimeMillis();
-        processingTime += ( timeEnd - timeNow );
-
-    }
-
-    /**
-     * Implements the Manager interface, direct call to processExpires
-     */
-    public void backgroundProcess() {
-        count = (count + 1) % processExpiresFrequency;
-        if (count == 0)
-            processExpires();
-    }
-
-
 }
