@@ -52,7 +52,7 @@ import org.apache.catalina.SessionListener;
 import org.apache.catalina.util.Enumerator;
 import org.apache.catalina.util.StringManager;
 
-
+import org.apache.catalina.security.SecurityUtil;
 /**
  * Standard implementation of the <b>Session</b> interface.  This object is
  * serializable, so that it can be stored in persistent storage or transferred
@@ -71,7 +71,7 @@ import org.apache.catalina.util.StringManager;
  * @author Craig R. McClanahan
  * @author Sean Legassick
  * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens</a>
- * @version $Revision: 1.48 $ $Date: 2004/09/07 20:57:02 $
+ * @version $Revision: 1.55 $ $Date: 2005/03/25 16:49:28 $
  */
 
 public class StandardSession
@@ -321,6 +321,14 @@ public class StandardSession
      */
     public String getId() {
 
+        // Comment out for now, will revisit for 5.5.10
+        /*
+         if ( !isValid() ) {
+             throw new IllegalStateException
+                 (sm.getString("standardSession.getId.ise"));
+         }
+	 */
+
         return (this.id);
 
     }
@@ -408,11 +416,12 @@ public class StandardSession
      * a value associated with the session, do not affect the access time.
      */
     public long getLastAccessedTime() {
-        if ( !isValid() ) {
-            throw new IllegalStateException
-                (sm.getString("standardSession.getLastAccessedTime.ise"));
-        }
-        return (this.lastAccessedTime);
+
+         if ( !isValid() ) {
+             throw new IllegalStateException
+                 (sm.getString("standardSession.getLastAccessedTime.ise"));
+         }
+         return (this.lastAccessedTime);
 
     }
 
@@ -518,7 +527,7 @@ public class StandardSession
     public HttpSession getSession() {
 
         if (facade == null){
-            if (System.getSecurityManager() != null){
+            if (SecurityUtil.isPackageProtectionEnabled()){
                 final StandardSession fsession = this;
                 facade = (StandardSessionFacade)AccessController.doPrivileged(new PrivilegedAction(){
                     public Object run(){
