@@ -20,6 +20,7 @@ package org.apache.catalina.startup;
 
 import java.io.File;
 import java.io.FileInputStream;
+import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.URL;
@@ -32,6 +33,7 @@ import org.apache.catalina.Authenticator;
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
 import org.apache.catalina.Engine;
+import org.apache.catalina.Globals;
 import org.apache.catalina.Host;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
@@ -62,7 +64,7 @@ import org.xml.sax.SAXParseException;
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Revision: 1.59 $ $Date: 2004/10/02 09:22:18 $
+ * @version $Revision: 1.64 $ $Date: 2005/03/16 11:37:43 $
  */
 
 public final class ContextConfig
@@ -168,6 +170,10 @@ public final class ContextConfig
     protected static long deploymentCount = 0L;
     
     
+    private static final LoginConfig DUMMY_LOGIN_CONFIG =
+                                new LoginConfig("NONE", null, null, null);
+
+
     // ------------------------------------------------------------- Properties
 
 
@@ -249,7 +255,19 @@ public final class ContextConfig
             start();
         } else if (event.getType().equals(StandardContext.BEFORE_START_EVENT)) {
             beforeStart();
+        } else if (event.getType().equals(StandardContext.AFTER_START_EVENT)) {
+            // Restore docBase for management tools
+            if (originalDocBase != null) {
+                String docBase = context.getDocBase();
+                context.setDocBase(originalDocBase);
+                originalDocBase = docBase;
+            }
         } else if (event.getType().equals(Lifecycle.STOP_EVENT)) {
+            if (originalDocBase != null) {
+                String docBase = context.getDocBase();
+                context.setDocBase(originalDocBase);
+                originalDocBase = docBase;
+            }
             stop();
         } else if (event.getType().equals(Lifecycle.INIT_EVENT)) {
             init();
@@ -268,12 +286,27 @@ public final class ContextConfig
      */
     protected void applicationWebConfig() {
 
+        String altDDName = null;
+
         // Open the application web.xml file, if it exists
         InputStream stream = null;
         ServletContext servletContext = context.getServletContext();
-        if (servletContext != null)
-            stream = servletContext.getResourceAsStream
-                (Constants.ApplicationWebXml);
+        if (servletContext != null) {
+            altDDName = (String)servletContext.getAttribute(
+                                                        Globals.ALT_DD_ATTR);
+            if (altDDName != null) {
+                try {
+                    stream = new FileInputStream(altDDName);
+                } catch (FileNotFoundException e) {
+                    log.error(sm.getString("contextConfig.altDDNotFound",
+                                           altDDName));
+                }
+            }
+            else {
+                stream = servletContext.getResourceAsStream
+                    (Constants.ApplicationWebXml);
+            }
+        }
         if (stream == null) {
             log.info(sm.getString("contextConfig.applicationMissing") + " " + context);
             return;
@@ -289,8 +322,12 @@ public final class ContextConfig
         // Process the application web.xml file
         synchronized (webDigester) {
             try {
-                url =
-                    servletContext.getResource(Constants.ApplicationWebXml);
+                if (altDDName != null) {
+                    url = new File(altDDName).toURL();
+                } else {
+                    url = servletContext.getResource(
+                                                Constants.ApplicationWebXml);
+                }
                 if( url!=null ) {
                     InputSource is = new InputSource(url.toExternalForm());
                     is.setByteStream(stream);
@@ -370,7 +407,7 @@ public final class ContextConfig
             return;
         LoginConfig loginConfig = context.getLoginConfig();
         if (loginConfig == null) {
-            loginConfig = new LoginConfig("NONE", null, null, null);
+            loginConfig = DUMMY_LOGIN_CONFIG;
             context.setLoginConfig(loginConfig);
         }
 
@@ -581,19 +618,29 @@ public final class ContextConfig
         stream = null;
         source = null;
 
-        file = new File(getConfigBase(), Constants.HostWebXml);
+        String resourceName = getHostConfigPath(Constants.HostWebXml);
+        file = new File(getConfigBase(), resourceName);
         
         try {
-           if (file.exists()) {
+            if ( ! file.exists() ) {
+                // Use getResource and getResourceAsStream
+                stream = getClass().getClassLoader()
+                    .getResourceAsStream(resourceName);
+                if( stream != null ) {
+                    source = new InputSource
+                            (getClass().getClassLoader()
+                            .getResource(resourceName).toString());
+                }
+            } else {
                 source =
                     new InputSource("file://" + file.getAbsolutePath());
                 stream = new FileInputStream(file);
             }
         } catch (Exception e) {
             log.error(sm.getString("contextConfig.defaultMissing") 
-                      + " " + file , e);
+                      + " " + resourceName + " " + file , e);
         }
-        
+
         if (stream != null) {
             processDefaultWebConfig(webDigester, stream, source);
             webRuleSet.recycle();
@@ -607,6 +654,11 @@ public final class ContextConfig
      */
     protected void processDefaultWebConfig(Digester digester, InputStream stream, 
             InputSource source) {
+
+        if (log.isDebugEnabled())
+            log.debug("Processing context [" + context.getName() 
+                    + "] web configuration resource " + source.getSystemId());
+
         // Process the default web.xml file
         synchronized (digester) {
             try {
@@ -659,11 +711,11 @@ public final class ContextConfig
         if( defaultContextXml==null ) getDefaultContextXml();
 
         if (!context.getOverride()) {
-            processContextConfig(new File(getBaseDir(), defaultContextXml));
-            processContextConfig(new File(getConfigBase(), Constants.HostContextXml));
+            processContextConfig(new File(getBaseDir()), defaultContextXml);
+            processContextConfig(getConfigBase(), getHostConfigPath(Constants.HostContextXml));
         }
         if (context.getConfigFile() != null)
-            processContextConfig(new File(context.getConfigFile()));
+            processContextConfig(new File(context.getConfigFile()), null);
         
     }
 
@@ -671,28 +723,45 @@ public final class ContextConfig
     /**
      * Process a context.xml.
      */
-    protected void processContextConfig(File file) {
+    protected void processContextConfig(File baseDir, String resourceName) {
         
         if (log.isDebugEnabled())
-            log.debug("Processing context [" + context.getName() + "] configuration file " + file);
-        
-        // Add as watched resource so that cascade reload occurs if a default
-        // config file is modified/added/removed
-        context.addWatchedResource(file.getAbsolutePath());
+            log.debug("Processing context [" + context.getName() 
+                    + "] configuration file " + baseDir + " " + resourceName);
 
         InputSource source = null;
         InputStream stream = null;
+
+        File file = baseDir;
+        if (resourceName != null) {
+            file = new File(baseDir, resourceName);
+        }
+        
         try {
-            if (file.exists()) {
-                stream = new FileInputStream(file);
+            if ( !file.exists() ) {
+                if (resourceName != null) {
+                    // Use getResource and getResourceAsStream
+                    stream = getClass().getClassLoader()
+                        .getResourceAsStream(resourceName);
+                    if( stream != null ) {
+                        source = new InputSource
+                            (getClass().getClassLoader()
+                            .getResource(resourceName).toString());
+                    }
+                }
+            } else {
                 source =
                     new InputSource("file://" + file.getAbsolutePath());
-            } else if (log.isDebugEnabled()) {
-                log.debug("Context [" + context.getName() + "] configuration file " + file + " not found");
+                stream = new FileInputStream(file);
+                // Add as watched resource so that cascade reload occurs if a default
+                // config file is modified/added/removed
+                context.addWatchedResource(file.getAbsolutePath());
             }
         } catch (Exception e) {
-            log.error(sm.getString("contextConfig.defaultMissing") + file);
+            log.error(sm.getString("contextConfig.defaultMissing") 
+                      + " " + resourceName + " " + file , e);
         }
+        
         if (source == null)
             return;
         if (contextDigester == null){
@@ -710,6 +779,9 @@ public final class ContextConfig
                 if (parseException != null) {
                     ok = false;
                 }
+                if (log.isDebugEnabled())
+                    log.debug("Successfully processed context [" + context.getName() 
+                            + "] configuration file " + baseDir + " " + resourceName);
             } catch (SAXParseException e) {
                 log.error(sm.getString("contextConfig.defaultParse"), e);
                 log.error(sm.getString("contextConfig.defaultPosition",
@@ -843,7 +915,11 @@ public final class ContextConfig
             }
             File docBaseFile = new File(docBase);
             if (!docBaseFile.isAbsolute()) {
-                docBaseFile = new File(appBase, docBase);
+                File file = new File(appBase);
+                if (!file.isAbsolute()) {
+                    file = new File(System.getProperty("catalina.base"), appBase);
+                }
+                docBaseFile = new File(file, docBase);
             }
             
             String path = context.getPath();
@@ -1222,7 +1298,14 @@ public final class ContextConfig
             new File(System.getProperty("catalina.base"), "conf");
         if (!configBase.exists()) {
             return null;
+        } else {
+            return configBase;
         }
+    }  
+
+    
+    protected String getHostConfigPath(String resourceName) {
+        StringBuffer result = new StringBuffer();
         Container container = context;
         Container host = null;
         Container engine = null;
@@ -1234,13 +1317,13 @@ public final class ContextConfig
             container = container.getParent();
         }
         if (engine != null) {
-            configBase = new File(configBase, engine.getName());
+            result.append(engine.getName()).append('/');
         }
         if (host != null) {
-            configBase = new File(configBase, host.getName());
+            result.append(host.getName()).append('/');
         }
-        configBase.mkdirs();
-        return configBase;
+        result.append(resourceName);
+        return result.toString();
     }
 
 
