@@ -92,17 +92,11 @@ import org.apache.tomcat.util.IntrospectionUtils;
  * To initiate a normal shutdown, call the <code>stop()</code> method of
  * this object.
  * <p>
- * <strong>IMPLEMENTATION NOTE</strong>:  The <code>main()</code> method of
- * this class is a simple example that exercizes the features of dynamically
- * starting and stopping various components.  You can execute this by executing
- * the following steps (on a Unix platform):
- * <pre>
- *   cd $CATALINA_HOME
- *   ./bin/catalina.sh embedded
- * </pre>
+ * @see org.apache.catalina.startup.Catalina#main For a complete example
+ * of how Tomcat is set up and launched as an Embedded application.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 1.25 $ $Date: 2004/10/23 16:54:23 $
+ * @version $Revision: 1.27 $ $Date: 2004/12/08 14:34:56 $
  */
 
 public class Embedded  extends StandardService implements Lifecycle {
@@ -394,6 +388,8 @@ public class Embedded  extends StandardService implements Lifecycle {
                 connector = new Connector("org.apache.jk.server.JkCoyoteHandler");
             } else if (protocol.equals("memory")) {
                 connector = new Connector("org.apache.coyote.memory.MemoryProtocolHandler");
+            } else if (protocol.equals("http")) {
+                connector = new Connector();
             } else if (protocol.equals("https")) {
                 connector = new Connector();
                 connector.setScheme("https");
