@@ -25,6 +25,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.LinkedHashMap;
 import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
 
@@ -50,7 +51,7 @@ import org.apache.tomcat.util.digester.Digester;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 1.48 $ $Date: 2004/10/05 17:12:49 $
+ * @version $Revision: 1.59 $ $Date: 2005/03/14 11:52:04 $
  */
 
 public class HostConfig
@@ -553,9 +554,10 @@ public class HostConfig
             return;
 
         // Assume this is a configuration descriptor and deploy it
-        log.debug(sm.getString("hostConfig.deployDescriptor", file));
+        if(log.isDebugEnabled())
+            log.debug(sm.getString("hostConfig.deployDescriptor", file));
+        Context context = null;
         try {
-            Context context = null;
             synchronized (digester) {
                 try {
                     context = (Context) digester.parse(contextXml);
@@ -571,9 +573,6 @@ public class HostConfig
             }
             context.setConfigFile(contextXml.getAbsolutePath());
             context.setPath(contextPath);
-            // Add the context XML to the list of watched files
-            deployedApp.reloadResources.put
-                (contextXml.getAbsolutePath(), new Long(contextXml.lastModified()));
             // Add the associated docBase to the redeployed list if it's a WAR
             boolean isWar = false;
             if (context.getDocBase() != null) {
@@ -588,52 +587,58 @@ public class HostConfig
                 }
             }
             host.addChild(context);
+            // Get paths for WAR and expanded WAR in appBase
+            String name = null;
+            String path = context.getPath();
+            if (path.equals("")) {
+                name = "ROOT";
+            } else {
+                if (path.startsWith("/")) {
+                    name = path.substring(1);
+                } else {
+                    name = path;
+                }
+            }
+            File expandedDocBase = new File(name);
+            File warDocBase = new File(name + ".war");
+            if (!expandedDocBase.isAbsolute()) {
+                expandedDocBase = new File(appBase(), name);
+                warDocBase = new File(appBase(), name + ".war");
+            }
             // Add the eventual unpacked WAR and all the resources which will be
             // watched inside it
             if (isWar && unpackWARs) {
-                String name = null;
-                String path = context.getPath();
-                if (path.equals("")) {
-                    name = "ROOT";
-                } else {
-                    if (path.startsWith("/")) {
-                        name = path.substring(1);
-                    } else {
-                        name = path;
-                    }
-                }
-                File docBase = new File(name);
-                if (!docBase.isAbsolute()) {
-                    docBase = new File(appBase(), name);
-                }
-                deployedApp.redeployResources.put(docBase.getAbsolutePath(),
-                        new Long(docBase.lastModified()));
-                addWatchedResources(deployedApp, docBase.getAbsolutePath(), context);
+                deployedApp.redeployResources.put(expandedDocBase.getAbsolutePath(),
+                        new Long(expandedDocBase.lastModified()));
+                deployedApp.redeployResources.put
+                    (contextXml.getAbsolutePath(), new Long(contextXml.lastModified()));
+                addWatchedResources(deployedApp, expandedDocBase.getAbsolutePath(), context);
             } else {
-                if (context.getDocBase() != null) {
-                    File docBase = new File(context.getDocBase());
-                    if (!docBase.isAbsolute()) {
-                        docBase = new File(appBase(), context.getDocBase());
-                    }
-                    try {
-                        docBase = docBase.getCanonicalFile();
-                        if (!docBase.getAbsolutePath().startsWith(appBase().getAbsolutePath())) {
-                            deployedApp.redeployResources.put
-                                (contextXml.getAbsolutePath(), new Long(contextXml.lastModified()));
-                            deployedApp.reloadResources.remove(contextXml.getAbsolutePath());
-                        }
-                    } catch (IOException e) {
-                        // Ignore
-                    }
+                // Find an existing matching war and expanded folder
+                if (warDocBase.exists()) {
+                    deployedApp.redeployResources.put(warDocBase.getAbsolutePath(),
+                            new Long(warDocBase.lastModified()));
                 }
-                addWatchedResources(deployedApp, null, context);
+                if (expandedDocBase.exists()) {
+                    deployedApp.redeployResources.put(expandedDocBase.getAbsolutePath(),
+                            new Long(expandedDocBase.lastModified()));
+                    addWatchedResources(deployedApp, 
+                            expandedDocBase.getAbsolutePath(), context);
+                } else {
+                    addWatchedResources(deployedApp, null, context);
+                }
+                // Add the context XML to the list of files which should trigger a redeployment
+                deployedApp.redeployResources.put
+                    (contextXml.getAbsolutePath(), new Long(contextXml.lastModified()));
             }
         } catch (Throwable t) {
             log.error(sm.getString("hostConfig.deployDescriptor.error",
                                    file), t);
         }
 
-        deployed.put(contextPath, deployedApp);
+        if (context != null && host.findChild(context.getName()) != null) {
+            deployed.put(contextPath, deployedApp);
+        }
     }
 
 
@@ -758,7 +763,13 @@ public class HostConfig
         DeployedApplication deployedApp = new DeployedApplication(contextPath);
         
         // Deploy the application in this WAR file
-        log.info(sm.getString("hostConfig.deployJar", file));
+        if(log.isInfoEnabled()) 
+            log.info(sm.getString("hostConfig.deployJar", file));
+
+        // Populate redeploy resources with the WAR file
+        deployedApp.redeployResources.put
+            (dir.getAbsolutePath(), new Long(dir.lastModified()));
+
         try {
             Context context = (Context) Class.forName(contextClass).newInstance();
             if (context instanceof Lifecycle) {
@@ -771,7 +782,7 @@ public class HostConfig
             context.setDocBase(file);
             if (xml.exists()) {
                 context.setConfigFile(xml.getAbsolutePath());
-                deployedApp.reloadResources.put
+                deployedApp.redeployResources.put
                     (xml.getAbsolutePath(), new Long(xml.lastModified()));
             }
             host.addChild(context);
@@ -803,9 +814,6 @@ public class HostConfig
             log.error(sm.getString("hostConfig.deployJar.error", file), t);
         }
         
-        // Populate redeploy resources with the WAR file
-        deployedApp.redeployResources.put
-            (dir.getAbsolutePath(), new Long(dir.lastModified()));
         deployed.put(contextPath, deployedApp);
     }
 
@@ -879,13 +887,17 @@ public class HostConfig
             }
             context.setPath(contextPath);
             context.setDocBase(file);
+            File configFile = new File(dir, Constants.ApplicationContextXml);
             if (deployXML) {
-                context.setConfigFile
-                    ((new File(dir, Constants.ApplicationContextXml)).getAbsolutePath());
+                context.setConfigFile(configFile.getAbsolutePath());
             }
             host.addChild(context);
             deployedApp.redeployResources.put(dir.getAbsolutePath(),
                     new Long(dir.lastModified()));
+            if (deployXML) {
+                deployedApp.redeployResources.put(configFile.getAbsolutePath(),
+                        new Long(configFile.lastModified()));
+            }
             addWatchedResources(deployedApp, dir.getAbsolutePath(), context);
         } catch (Throwable t) {
             log.error(sm.getString("hostConfig.deployDir.error", file), t);
@@ -907,7 +919,9 @@ public class HostConfig
 
     /**
      * Add watched resources to the specified Context.
-     * @param app
+     * @param app HostConfig deployed app
+     * @param docBase web app docBase
+     * @param context web application context
      */
     protected void addWatchedResources(DeployedApplication app, String docBase, Context context) {
         // FIXME: Feature idea. Add support for patterns (ex: WEB-INF/*, WEB-INF/*.xml), where
@@ -947,7 +961,9 @@ public class HostConfig
             if (resource.exists()) {
                 long lastModified = ((Long) app.redeployResources.get(resources[i])).longValue();
                 if ((!resource.isDirectory()) && resource.lastModified() > lastModified) {
-                    // Redeploy application
+                    // Undeploy application
+                    if (log.isInfoEnabled())
+                        log.info(sm.getString("hostConfig.undeploy", app.name));
                     ContainerBase context = (ContainerBase) host.findChild(app.name);
                     host.removeChild(context);
                     try {
@@ -957,28 +973,32 @@ public class HostConfig
                                  ("hostConfig.context.destroy", app.name), e);
                     }
                     // Delete other redeploy resources
-                    for (int j = 0; j < resources.length; j++) {
-                        if (j != i) {
-                            try {
-                                File current = new File(resources[j]);
-                                current = current.getCanonicalFile();
-                                if ((current.getAbsolutePath().startsWith(appBase().getAbsolutePath()))
+                    for (int j = i + 1; j < resources.length; j++) {
+                        try {
+                            File current = new File(resources[j]);
+                            current = current.getCanonicalFile();
+                            if ((current.getAbsolutePath().startsWith(appBase().getAbsolutePath()))
                                     || (current.getAbsolutePath().startsWith(configBase().getAbsolutePath()))) {
-                                    if (log.isDebugEnabled())
-                                        log.debug("Delete " + current);
-                                    ExpandWar.delete(current);
-                                }
-                            } catch (IOException e) {
-                                log.warn(sm.getString
-                                        ("hostConfig.canonicalizing", app.name), e);
+                                if (log.isDebugEnabled())
+                                    log.debug("Delete " + current);
+                                ExpandWar.delete(current);
                             }
+                        } catch (IOException e) {
+                            log.warn(sm.getString
+                                    ("hostConfig.canonicalizing", app.name), e);
                         }
                     }
                     deployed.remove(app.name);
                     return;
                 }
             } else {
+                long lastModified = ((Long) app.redeployResources.get(resources[i])).longValue();
+                if (lastModified == 0L) {
+                    continue;
+                }
                 // Undeploy application
+                if (log.isInfoEnabled())
+                    log.info(sm.getString("hostConfig.undeploy", app.name));
                 ContainerBase context = (ContainerBase) host.findChild(app.name);
                 host.removeChild(context);
                 try {
@@ -988,7 +1008,7 @@ public class HostConfig
                              ("hostConfig.context.destroy", app.name), e);
                 }
                 // Delete all redeploy resources
-                for (int j = 0; j < resources.length; j++) {
+                for (int j = i + 1; j < resources.length; j++) {
                     try {
                         File current = new File(resources[j]);
                         current = current.getCanonicalFile();
@@ -1034,6 +1054,8 @@ public class HostConfig
             if ((!resource.exists() && lastModified != 0L) 
                 || (resource.lastModified() != lastModified)) {
                 // Reload application
+                if(log.isInfoEnabled())
+                    log.info(sm.getString("hostConfig.reload", app.name));
                 Container context = host.findChild(app.name);
                 try {
                     ((Lifecycle) context).stop();
@@ -1073,10 +1095,11 @@ public class HostConfig
             Registry.getRegistry(null, null).registerComponent
                 (this, oname, this.getClass().getName());
         } catch (Exception e) {
-            log.error(sm.getString("hostConfig.jmx.register"), e);
+            log.error(sm.getString("hostConfig.jmx.register", oname), e);
         }
-        
-        deployApps();
+
+        if (host.getDeployOnStartup())
+            deployApps();
         
     }
 
@@ -1095,7 +1118,7 @@ public class HostConfig
             try {
                 Registry.getRegistry(null, null).unregisterComponent(oname);
             } catch (Exception e) {
-                log.error(sm.getString("hostConfig.jmx.unregister"), e);
+                log.error(sm.getString("hostConfig.jmx.unregister", oname), e);
             }
         }
         oname = null;
@@ -1247,7 +1270,7 @@ public class HostConfig
     	 * contain resources like the context.xml file, a compressed WAR path.
          * The value is the last modification time.
     	 */
-    	public HashMap redeployResources = new HashMap();
+    	public LinkedHashMap redeployResources = new LinkedHashMap();
 
     	/**
     	 * Any modification of the specified (static) resources will cause a 
