@@ -1,12 +1,12 @@
 /*
  * Copyright 1999-2001,2004 The Apache Software Foundation.
- * 
+ *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -103,10 +103,10 @@ import org.apache.catalina.util.StringManager;
  *
  * @author Craig R. McClanahan
  * @author Jason Brittain
- * @version $Revision: 1.13 $ $Date: 2004/08/29 16:46:14 $
+ * @version $Revision: 1.14 $ $Date: 2004/12/23 23:58:07 $
  */
 
-public final class AccessLogValve
+public class AccessLogValve
     extends ValveBase
     implements Lifecycle {
 
@@ -964,14 +964,14 @@ public final class AccessLogValve
      */
     private Date getDate() {
         if(currentDate == null) {
-	    currentDate = new Date();
+        currentDate = new Date();
         } else {
           // Only create a new Date once per second, max.
           long systime = System.currentTimeMillis();
           if ((systime - currentDate.getTime()) > 1000) {
               currentDate = new Date(systime);
           }
-	}
+    }
 
         return currentDate;
     }
