@@ -1,12 +1,12 @@
 /*
  * Copyright 1999-2001,2004 The Apache Software Foundation.
- * 
+ *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -129,10 +129,10 @@ import org.apache.commons.logging.LogFactory;
  *
  *
  * @author Tim Funk
- * @version $Revision: 1.11 $ $Date: 2004/08/29 16:46:14 $
+ * @version $Revision: 1.12 $ $Date: 2004/12/23 23:58:07 $
  */
 
-public final class ExtendedAccessLogValve
+public class ExtendedAccessLogValve
     extends ValveBase
     implements Lifecycle {
 
