@@ -17,12 +17,9 @@
 package org.apache.catalina.cluster.tcp;
 
 /**
- * <p>Title: </p>
- * <p>Description: </p>
- * <p>Copyright: Copyright (c) 2002</p>
- * <p>Company: </p>
- * @author not attributable
+ * @author Peter Rossbach
  * @version 1.0
+ * @since 5.5.7
  */
 
 public interface IDataSender
@@ -36,4 +33,8 @@ public interface IDataSender
     public void setSuspect(boolean suspect);
     public boolean getSuspect();
     public void setAckTimeout(long timeout);
+    public long getAckTimeout();
+    public boolean isWaitForAck();
+    public void setWaitForAck(boolean isWaitForAck);
+
 }
