@@ -22,14 +22,13 @@ import org.apache.struts.action.ActionError;
 import org.apache.struts.action.ActionErrors;
 import org.apache.struts.action.ActionForm;
 import org.apache.struts.action.ActionMapping;
-import java.net.InetAddress;
 import java.util.List;
 
 /**
  * Form bean for the context page.
  *
  * @author Manveen Kaur
- * @version $Revision: 1.7 $ $Date: 2004/07/29 06:31:59 $
+ * @version $Revision: 1.8 $ $Date: 2005/01/14 23:55:41 $
  */
 
 public final class ContextForm extends ActionForm {
@@ -86,6 +85,11 @@ public final class ContextForm extends ActionForm {
      */
     private String override = "false";
     
+    /**
+     * The text for privileged boolean.
+     */
+    private String privileged = "false";
+    
     /**
      * The text for the context path for this context.
      */
@@ -362,6 +366,26 @@ public final class ContextForm extends ActionForm {
     }
     
     
+    /**
+     * Return the privileged boolean value.
+     */
+    
+    public String getPrivileged() {
+        
+        return this.privileged;
+        
+    }
+    
+    /**
+     * Set the privileged value.
+     */
+    public void setPrivileged(String privileged) {
+        
+        this.privileged = privileged;
+        
+    }
+    
+    
     /**
      * Return the context path.
      */
