@@ -18,17 +18,11 @@ package org.apache.webapp.admin.context;
 
 
 import java.net.URLEncoder;
-import java.util.Iterator;
 import java.util.Locale;
 import java.io.IOException;
 import javax.management.Attribute;
 import javax.management.MBeanServer;
-import javax.management.MBeanServerFactory;
-import javax.management.QueryExp;
-import javax.management.Query;
-import javax.management.ObjectInstance;
 import javax.management.ObjectName;
-import javax.management.JMException;
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
@@ -53,7 +47,7 @@ import org.apache.webapp.admin.TreeControlNode;
  * <em>Edit Context</em> transactions.
  *
  * @author Manveen Kaur
- * @version $Revision: 1.17 $ $Date: 2004/10/18 06:37:53 $
+ * @version $Revision: 1.18 $ $Date: 2005/01/14 23:55:41 $
  */
 
 public final class SaveContextAction extends Action {
@@ -292,6 +286,16 @@ public final class SaveContextAction extends Action {
             mBServer.setAttribute(coname,
                                   new Attribute("override", new Boolean(override)));
 
+            attribute = "privileged";
+            String privileged = "false";
+            try {
+                privileged = cform.getPrivileged();
+            } catch (Throwable t) {
+                privileged = "false";
+            }
+            mBServer.setAttribute(coname,
+                                  new Attribute("privileged", new Boolean(privileged)));
+
             attribute = "reloadable";
             String reloadable = "false";
             try {
