@@ -32,6 +32,7 @@ import javax.servlet.http.HttpServletResponse;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
+import org.apache.catalina.util.RequestUtil;
 import org.apache.catalina.util.ServerInfo;
 import org.apache.commons.fileupload.FileItem;
 import org.apache.commons.fileupload.DiskFileUpload;
@@ -55,7 +56,7 @@ import org.apache.commons.fileupload.DiskFileUpload;
 * @author Bip Thelin
 * @author Malcolm Edgar
 * @author Glenn L. Nielsen
-* @version $Revision: 1.17 $, $Date: 2004/10/14 17:25:32 $
+* @version $Revision: 1.18 $, $Date: 2005/01/05 01:03:22 $
 * @see ManagerServlet
 */
 
@@ -105,7 +106,8 @@ public final class HTMLManagerServlet extends ManagerServlet {
             message = stop(path);
         } else {
             message =
-                sm.getString("managerServlet.unknownCommand", command);
+                sm.getString("managerServlet.unknownCommand",
+                             RequestUtil.filter(command));
         }
 
         list(request, response, message);
