@@ -32,7 +32,6 @@ import javax.naming.Binding;
 import javax.naming.InitialContext;
 import javax.naming.NamingEnumeration;
 import javax.naming.NamingException;
-import javax.naming.directory.DirContext;
 import javax.servlet.ServletException;
 import javax.servlet.ServletInputStream;
 import javax.servlet.UnavailableException;
@@ -53,11 +52,10 @@ import org.apache.catalina.Session;
 import org.apache.catalina.UserDatabase;
 import org.apache.catalina.Wrapper;
 import org.apache.catalina.core.StandardServer;
+import org.apache.catalina.util.RequestUtil;
 import org.apache.catalina.util.ServerInfo;
 import org.apache.catalina.util.StringManager;
 import org.apache.commons.modeler.Registry;
-import org.apache.naming.resources.ProxyDirContext;
-import org.apache.naming.resources.WARDirContext;
 
 
 /**
@@ -152,7 +150,7 @@ import org.apache.naming.resources.WARDirContext;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 1.22 $ $Date: 2004/09/23 07:03:27 $
+ * @version $Revision: 1.27 $ $Date: 2005/01/05 01:03:22 $
  */
 
 public class ManagerServlet
@@ -355,7 +353,7 @@ public class ManagerServlet
             reload(writer, path);
         } else if (command.equals("/remove")) {
             // Deprecated
-            remove(writer, path);
+            undeploy(writer, path);
         } else if (command.equals("/resources")) {
             resources(writer, type);
         } else if (command.equals("/roles")) {
@@ -648,7 +646,7 @@ public class ManagerServlet
         }
         
         context = (Context) host.findChild(path);
-        if (context != null) {
+        if (context != null && context.getConfigured()) {
             writer.println(sm.getString("managerServlet.deployed", displayPath));
         } else {
             // Something failed
@@ -714,7 +712,7 @@ public class ManagerServlet
         }
         
         context = (Context) host.findChild(path);
-        if (context != null) {
+        if (context != null && context.getConfigured()) {
             writer.println(sm.getString("managerServlet.deployed", displayPath));
         } else {
             // Something failed
@@ -737,6 +735,9 @@ public class ManagerServlet
     protected void deploy(PrintWriter writer, String config,
             String path, String war, boolean update) {
         
+        if (config != null && config.length() == 0) {
+            config = null;
+        }
         if (war != null && war.length() == 0) {
             war = null;
         }
@@ -762,7 +763,7 @@ public class ManagerServlet
         
         if (path == null || path.length() == 0 || !path.startsWith("/")) {
             writer.println(sm.getString("managerServlet.invalidPath",
-                    path));
+                                        RequestUtil.filter(path)));
             return;
         }
         String displayPath = path;
@@ -815,8 +816,13 @@ public class ManagerServlet
                     removeServiced(path);
                 }
             }
-            writer.println(sm.getString("managerServlet.deployed",
-                    displayPath));
+            context = (Context) host.findChild(path);
+            if (context != null && context.getConfigured()) {
+                writer.println(sm.getString("managerServlet.deployed", displayPath));
+            } else {
+                // Something failed
+                writer.println(sm.getString("managerServlet.deployFailed", displayPath));
+            }
         } catch (Throwable t) {
             log("ManagerServlet.install[" + displayPath + "]", t);
             writer.println(sm.getString("managerServlet.exception",
@@ -876,7 +882,8 @@ public class ManagerServlet
             log("restart: Reloading web application at '" + path + "'");
 
         if ((path == null) || (!path.startsWith("/") && path.equals(""))) {
-            writer.println(sm.getString("managerServlet.invalidPath", path));
+            writer.println(sm.getString("managerServlet.invalidPath",
+                                        RequestUtil.filter(path)));
             return;
         }
         String displayPath = path;
@@ -887,16 +894,8 @@ public class ManagerServlet
             Context context = (Context) host.findChild(path);
             if (context == null) {
                 writer.println(sm.getString
-                               ("managerServlet.noContext", displayPath));
-                return;
-            }
-            DirContext resources = context.getResources();
-            if (resources instanceof ProxyDirContext) {
-                resources = ((ProxyDirContext) resources).getDirContext();
-            }
-            if (resources instanceof WARDirContext) {
-                writer.println(sm.getString
-                               ("managerServlet.noReload", displayPath));
+                               ("managerServlet.noContext",
+                                   RequestUtil.filter(displayPath)));
                 return;
             }
             // It isn't possible for the manager to reload itself
@@ -916,48 +915,6 @@ public class ManagerServlet
     }
 
 
-    /**
-     * Remove the web application at the specified context path.
-     *
-     * @param writer Writer to render to
-     * @param path Context path of the application to be removed
-     * @deprecated Replaced by undeploy
-     */
-    protected void remove(PrintWriter writer, String path) {
-
-        if (debug >= 1)
-            log("remove: Removing web application at '" + path + "'");
-
-        if ((path == null) || (!path.startsWith("/") && path.equals(""))) {
-            writer.println(sm.getString("managerServlet.invalidPath", path));
-            return;
-        }
-        String displayPath = path;
-        if( path.equals("/") )
-            path = "";
-
-        try {
-            Context context = (Context) host.findChild(path);
-            if (context == null) {
-                writer.println(sm.getString("managerServlet.noContext", displayPath));
-                return;
-            }
-            // It isn't possible for the manager to remove itself
-            if (context.getPath().equals(this.context.getPath())) {
-                writer.println(sm.getString("managerServlet.noSelf"));
-                return;
-            }
-            host.removeChild(context);
-            writer.println(sm.getString("managerServlet.undeployed", displayPath));
-        } catch (Throwable t) {
-            log("ManagerServlet.remove[" + displayPath + "]", t);
-            writer.println(sm.getString("managerServlet.exception",
-                                        t.toString()));
-        }
-
-    }
-
-
     /**
      * Render a list of available global JNDI resources.
      *
@@ -1134,7 +1091,8 @@ public class ManagerServlet
             log("sessions: Session information for web application at '" + path + "'");
 
         if ((path == null) || (!path.startsWith("/") && path.equals(""))) {
-            writer.println(sm.getString("managerServlet.invalidPath", path));
+            writer.println(sm.getString("managerServlet.invalidPath",
+                                        RequestUtil.filter(path)));
             return;
         }
         String displayPath = path;
@@ -1143,7 +1101,8 @@ public class ManagerServlet
         try {
             Context context = (Context) host.findChild(path);
             if (context == null) {
-                writer.println(sm.getString("managerServlet.noContext", displayPath));
+                writer.println(sm.getString("managerServlet.noContext",
+                                            RequestUtil.filter(displayPath)));
                 return;
             }
             writer.println(sm.getString("managerServlet.sessions", displayPath));
@@ -1198,7 +1157,8 @@ public class ManagerServlet
             log("start: Starting web application at '" + path + "'");
 
         if ((path == null) || (!path.startsWith("/") && path.equals(""))) {
-            writer.println(sm.getString("managerServlet.invalidPath", path));
+            writer.println(sm.getString("managerServlet.invalidPath",
+                                        RequestUtil.filter(path)));
             return;
         }
         String displayPath = path;
@@ -1209,7 +1169,7 @@ public class ManagerServlet
             Context context = (Context) host.findChild(path);
             if (context == null) {
                 writer.println(sm.getString("managerServlet.noContext", 
-                                            displayPath));
+                                            RequestUtil.filter(displayPath)));
                 return;
             }
             ((Lifecycle) context).start();
@@ -1243,7 +1203,8 @@ public class ManagerServlet
             log("stop: Stopping web application at '" + path + "'");
 
         if ((path == null) || (!path.startsWith("/") && path.equals(""))) {
-            writer.println(sm.getString("managerServlet.invalidPath", path));
+            writer.println(sm.getString("managerServlet.invalidPath",
+                                        RequestUtil.filter(path)));
             return;
         }
         String displayPath = path;
@@ -1254,7 +1215,7 @@ public class ManagerServlet
             Context context = (Context) host.findChild(path);
             if (context == null) {
                 writer.println(sm.getString("managerServlet.noContext", 
-                                            displayPath));
+                                            RequestUtil.filter(displayPath)));
                 return;
             }
             // It isn't possible for the manager to stop itself
@@ -1285,7 +1246,8 @@ public class ManagerServlet
             log("undeploy: Undeploying web application at '" + path + "'");
 
         if ((path == null) || (!path.startsWith("/") && path.equals(""))) {
-            writer.println(sm.getString("managerServlet.invalidPath", path));
+            writer.println(sm.getString("managerServlet.invalidPath",
+                                        RequestUtil.filter(path)));
             return;
         }
         String displayPath = path;
@@ -1298,7 +1260,7 @@ public class ManagerServlet
             Context context = (Context) host.findChild(path);
             if (context == null) {
                 writer.println(sm.getString("managerServlet.noContext",
-                                            displayPath));
+                                            RequestUtil.filter(displayPath)));
                 return;
             }
 
@@ -1314,11 +1276,15 @@ public class ManagerServlet
                 }
             }
 
-            // Stop the context first to be nicer
             if (!isServiced(path)) {
                 addServiced(path);
                 try {
+                    // Try to stop the context first to be nicer
                     ((Lifecycle) context).stop();
+                } catch (Throwable t) {
+                    // Ignore
+                }
+                try {
                     File war = new File(getAppBase(), getDocBase(path) + ".war");
                     File dir = new File(getAppBase(), getDocBase(path));
                     File xml = new File(configBase, getConfigFile(path) + ".xml");
