@@ -49,6 +49,7 @@ import org.apache.tomcat.util.buf.HexUtils;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.http.FastHttpDateFormat;
 import org.apache.tomcat.util.http.MimeHeaders;
+import org.apache.tomcat.util.net.PoolTcpEndpoint;
 import org.apache.tomcat.util.net.SSLSupport;
 import org.apache.tomcat.util.threads.ThreadPool;
 import org.apache.tomcat.util.threads.ThreadWithAttributes;
@@ -297,6 +298,12 @@ public class Http11Processor implements Processor, ActionHook {
     protected ThreadPool threadPool;
 
 
+    /**
+     * Associated endpoint.
+     */
+    protected PoolTcpEndpoint endpoint;
+
+
     /**
      * Allow a customized the server header for the tin-foil hat folks.
      */
@@ -356,6 +363,12 @@ public class Http11Processor implements Processor, ActionHook {
         this.threadPool = threadPool;
     }
 
+    
+    public void setEndpoint(PoolTcpEndpoint endpoint) {
+        this.endpoint = endpoint;
+    }
+
+
     /**
      * Add user-agent for which gzip compression didn't works
      * The user agent String given will be exactly matched
@@ -764,15 +777,33 @@ public class Http11Processor implements Processor, ActionHook {
 
         int keepAliveLeft = maxKeepAliveRequests;
         int soTimeout = socket.getSoTimeout();
+        int oldSoTimeout = soTimeout;
 
-        float threadRatio =
-            (float) threadPool.getCurrentThreadsBusy()
-            / (float) threadPool.getMaxThreads();
-        if ((threadRatio > 0.33) && (threadRatio <= 0.66)) {
+        int threadRatio = 0;
+        if (threadPool.getCurrentThreadsBusy() > 0) {
+            threadRatio = (threadPool.getCurrentThreadsBusy() * 100)
+                / threadPool.getMaxThreads();
+        } else {
+            threadRatio = (endpoint.getCurrentThreadsBusy() * 100)
+                / endpoint.getMaxThreads();
+        }
+        if ((threadRatio > 33) && (threadRatio <= 66)) {
             soTimeout = soTimeout / 2;
-        } else if (threadRatio > 0.66) {
+        } else if ((threadRatio > 66) && (threadRatio <= 90)) {
             soTimeout = soTimeout / 3;
             keepAliveLeft = 1;
+        } else if (threadRatio > 90) {
+            soTimeout = soTimeout / 20;
+            keepAliveLeft = 1;
+        }
+        
+        if (soTimeout != oldSoTimeout) {
+            try {
+                socket.setSoTimeout(soTimeout);
+            } catch (Throwable t) {
+                log.debug("Error setting timeout", t);
+                error = true;
+            }
         }
 
         boolean keptAlive = false;
@@ -1222,14 +1253,6 @@ public class Http11Processor implements Processor, ActionHook {
         // Input filter setup
         InputFilter[] inputFilters = inputBuffer.getFilters();
 
-        // Parse content-length header
-        long contentLength = request.getContentLengthLong();
-        if (contentLength >= 0) {
-            inputBuffer.addActiveFilter
-                (inputFilters[Constants.IDENTITY_FILTER]);
-            contentDelimitation = true;
-        }
-
         // Parse transfer-encoding header
         MessageBytes transferEncodingValueMB = null;
         if (http11)
@@ -1262,6 +1285,14 @@ public class Http11Processor implements Processor, ActionHook {
             }
         }
 
+        // Parse content-length header
+        long contentLength = request.getContentLengthLong();
+        if (contentLength >= 0 && !contentDelimitation) {
+            inputBuffer.addActiveFilter
+                (inputFilters[Constants.IDENTITY_FILTER]);
+            contentDelimitation = true;
+        }
+
         MessageBytes valueMB = headers.getValue("host");
 
         // Check host header
@@ -1413,7 +1444,7 @@ public class Http11Processor implements Processor, ActionHook {
         }
 
         // Check if suffisant len to trig the compression
-        int contentLength = response.getContentLength();
+        long contentLength = response.getContentLengthLong();
         if ((contentLength == -1)
             || (contentLength > compressionMinSize)) {
             // Check for compatible MIME-TYPE
@@ -1489,9 +1520,9 @@ public class Http11Processor implements Processor, ActionHook {
             }
         }
 
-        int contentLength = response.getContentLength();
+        long contentLength = response.getContentLengthLong();
         if (contentLength != -1) {
-            headers.setValue("Content-Length").setInt(contentLength);
+            headers.setValue("Content-Length").setLong(contentLength);
             outputBuffer.addActiveFilter
                 (outputFilters[Constants.IDENTITY_FILTER]);
             contentDelimitation = true;
@@ -1542,7 +1573,7 @@ public class Http11Processor implements Processor, ActionHook {
         keepAlive = keepAlive && !statusDropsConnection(statusCode);
         if (!keepAlive) {
             headers.addValue(Constants.CONNECTION).setString(Constants.CLOSE);
-        } else if (!http11) {
+        } else if (!http11 && !error) {
             headers.addValue(Constants.CONNECTION).setString(Constants.KEEPALIVE);
         }
 
