@@ -59,7 +59,7 @@ import org.apache.tomcat.util.threads.ThreadPoolRunnable;
 */
 
 /**
- *  Jk2 can use multiple protocols/transports.
+ *  Jk can use multiple protocols/transports.
  *  Various container adapters should load this object ( as a bean ),
  *  set configurations and use it. Note that the connector will handle
  *  all incoming protocols - it's not specific to ajp1x. The protocol
@@ -70,7 +70,7 @@ import org.apache.tomcat.util.threads.ThreadPoolRunnable;
 /** Accept ( and send ) TCP messages.
  *
  * @author Costin Manolache
- * @jmx:mbean name="jk2:service=ChannelSocket"
+ * @jmx:mbean name="jk:service=ChannelSocket"
  *            description="Accept socket connections"
  * @jmx:notification name="org.apache.coyote.INVOKE
  * @jmx:notification-handler name="org.apache.jk.JK_SEND_PACKET
@@ -335,7 +335,8 @@ public class ChannelSocket extends JkHandler
         // Find a port.
         if (startPort == 0) {
             port = 0;
-            log.info("JK2: ajp13 disabling channelSocket");
+            if(log.isInfoEnabled())
+                log.info("JK: ajp13 disabling channelSocket");
             running = true;
             return;
         }
@@ -351,7 +352,8 @@ public class ChannelSocket extends JkHandler
                 port=i;
                 break;
             } catch( IOException ex ) {
-                log.info("Port busy " + i + " " + ex.toString());
+                if(log.isInfoEnabled())
+                    log.info("Port busy " + i + " " + ex.toString());
                 continue;
             }
         }
@@ -360,7 +362,8 @@ public class ChannelSocket extends JkHandler
             log.error("Can't find free port " + startPort + " " + maxPort );
             return;
         }
-        log.info("JK2: ajp13 listening on " + getAddress() + ":" + port );
+        if(log.isInfoEnabled())
+            log.info("JK: ajp13 listening on " + getAddress() + ":" + port );
 
         // If this is not the base port and we are the 'main' channleSocket and
         // SHM didn't already set the localId - we'll set the instance id
@@ -433,7 +436,7 @@ public class ChannelSocket extends JkHandler
                      getChannelName()+",name=JkRequest" +count);
                 ep.setNote(JMXRequestNote, roname);
                         
-                Registry.getRegistry().registerComponent( rp, roname, null);
+                Registry.getRegistry(null, null).registerComponent( rp, roname, null);
             } catch( Exception ex ) {
                 log.warn("Error registering request");
             }
@@ -480,10 +483,10 @@ public class ChannelSocket extends JkHandler
             sSocket.close(); // XXX?
             
             if( tpOName != null )  {
-                Registry.getRegistry().unregisterComponent(tpOName);
+                Registry.getRegistry(null, null).unregisterComponent(tpOName);
             }
             if( rgOName != null ) {
-                Registry.getRegistry().unregisterComponent(rgOName);
+                Registry.getRegistry(null, null).unregisterComponent(rgOName);
             }
         } catch(Exception e) {
             log.info("Error shutting down the channel " + port + " " +
@@ -703,7 +706,9 @@ public class ChannelSocket extends JkHandler
                 Request req = (Request)ep.getRequest();
                 if( req != null ) {
                     ObjectName roname = (ObjectName)ep.getNote(JMXRequestNote);
-                    Registry.getRegistry().unregisterComponent(roname);
+                    if( roname != null ) {
+                        Registry.getRegistry(null, null).unregisterComponent(roname);
+                    }
                     req.getRequestProcessor().setGlobalProcessor(null);
                 }
             } catch( Exception ee) {
@@ -864,5 +869,6 @@ class SocketConnection implements ThreadPoolRunnable {
     
     public void runIt(Object perTh[]) {
         wajp.processConnection(ep);
+        ep = null;
     }
 }
