@@ -22,11 +22,8 @@ import java.io.IOException;
 import java.io.CharConversionException;
 import java.net.InetAddress;
 import java.util.Properties;
-import javax.management.ObjectName;
 
-import org.apache.commons.modeler.Registry;
 import org.apache.coyote.Request;
-import org.apache.coyote.RequestGroupInfo;
 import org.apache.coyote.RequestInfo;
 import org.apache.coyote.Response;
 import org.apache.coyote.Constants;
@@ -189,6 +186,11 @@ public class HandlerRequest extends JkHandler
      */
     public static final int HOSTBUFFER = 10;
 
+    /**
+     * Thread lock.
+     */
+    private static Object lock = new Object();
+
     HandlerDispatch dispatch;
     String ajpidDir="conf";
     
@@ -316,10 +318,10 @@ public class HandlerRequest extends JkHandler
             }
 
             FileOutputStream stopF=new FileOutputStream( sf );
-            props.save( stopF, "Automatically generated, don't edit" );
+            props.store( stopF, "Automatically generated, don't edit" );
         } catch( IOException ex ) {
-            log.debug( "Can't create stop file: "+sf );
-            ex.printStackTrace();
+            if(log.isDebugEnabled())
+                log.debug( "Can't create stop file: "+sf,ex );
         }
     }
     
@@ -346,7 +348,7 @@ public class HandlerRequest extends JkHandler
 
         MessageBytes tmpMB=(MessageBytes)ep.getNote( tmpBufNote );
         if( tmpMB==null ) {
-            tmpMB=new MessageBytes();
+            tmpMB= MessageBytes.newInstance();
             ep.setNote( tmpBufNote, tmpMB);
         }
         if( log.isDebugEnabled() )
@@ -418,7 +420,8 @@ public class HandlerRequest extends JkHandler
             checkRequest(ep);
             next.invoke( msg, ep );
 
-            log.info("Exiting");
+            if(log.isInfoEnabled())
+                log.info("Exiting");
             System.exit(0);
             
             return OK;
@@ -435,7 +438,8 @@ public class HandlerRequest extends JkHandler
             return OK;
 
         default:
-            log.info("Unknown message " + type);
+            if(log.isInfoEnabled())
+                log.info("Unknown message " + type);
         }
 
         return OK;
@@ -451,7 +455,9 @@ public class HandlerRequest extends JkHandler
             req.setResponse(res);
             ep.setRequest( req );
             if( registerRequests ) {
-                ep.getSource().registerRequest(req, ep, count++);
+                synchronized(lock) {
+                    ep.getSource().registerRequest(req, ep, count++);
+                }
             }
         }
         return req;
@@ -597,7 +603,7 @@ public class HandlerRequest extends JkHandler
                 // Transform the string into certificate.
                 MessageBytes tmpMB2 = (MessageBytes)req.getNote(WorkerEnv.SSL_CERT_NOTE);
                 if(tmpMB2 == null) {
-                    tmpMB2 = new MessageBytes();
+                    tmpMB2 = MessageBytes.newInstance();
                     req.setNote(WorkerEnv.SSL_CERT_NOTE, tmpMB2);
                 }
                 // SSL certificate extraction is costy, moved to JkCoyoteHandler
@@ -668,8 +674,6 @@ public class HandlerRequest extends JkHandler
                 hId = -1;
                 msg.getBytes( tmpMB );
                 ByteChunk bc=tmpMB.getByteChunk();
-                //hName=tmpMB.toString();
-                //                vMB=headers.addValue( hName );
                 vMB=headers.addValue( bc.getBuffer(),
                                       bc.getStart(), bc.getLength() );
             }
@@ -677,12 +681,11 @@ public class HandlerRequest extends JkHandler
             msg.getBytes(vMB);
 
             if (hId == SC_REQ_CONTENT_LENGTH ||
-                tmpMB.equalsIgnoreCase("Content-Length")) {
+                (hId == -1 && tmpMB.equalsIgnoreCase("Content-Length"))) {
                 // just read the content-length header, so set it
-                int contentLength = (vMB == null) ? -1 : vMB.getInt();
-                req.setContentLength(contentLength);
+                req.setContentLength( vMB.getInt() );
             } else if (hId == SC_REQ_CONTENT_TYPE ||
-                       tmpMB.equalsIgnoreCase("Content-Type")) {
+                (hId == -1 && tmpMB.equalsIgnoreCase("Content-Type"))) {
                 // just read the content-type header, so set it
                 ByteChunk bchunk = vMB.getByteChunk();
                 req.contentType().setBytes(bchunk.getBytes(),
