@@ -17,7 +17,7 @@
 /***************************************************************************
  * Description: Next generation bi-directional protocol handler.           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 1.20 $                                          *
+ * Version:     $Revision: 1.26 $                                          *
  ***************************************************************************/
 
 
@@ -34,11 +34,14 @@
 
 void ajp14_compute_md5(jk_login_service_t *s, jk_logger_t *l)
 {
+    JK_TRACE_ENTER(l);
     jk_md5((const unsigned char *)s->entropy,
            (const unsigned char *)s->secret_key, s->computed_key);
 
-    jk_log(l, JK_LOG_DEBUG, "Into ajp14_compute_md5 (%s/%s) -> (%s)\n",
-           s->entropy, s->secret_key, s->computed_key);
+    if (JK_IS_DEBUG_LEVEL(l))
+        jk_log(l, JK_LOG_DEBUG, "(%s/%s) -> (%s)",
+               s->entropy, s->secret_key, s->computed_key);
+    JK_TRACE_EXIT(l);
 }
 
 
@@ -54,32 +57,35 @@ void ajp14_compute_md5(jk_login_service_t *s, jk_logger_t *l)
 int ajp14_marshal_login_init_into_msgb(jk_msg_buf_t *msg,
                                        jk_login_service_t *s, jk_logger_t *l)
 {
-    jk_log(l, JK_LOG_DEBUG, "Into ajp14_marshal_login_init_into_msgb\n");
-
+    JK_TRACE_ENTER(l);
     /* To be on the safe side */
     jk_b_reset(msg);
 
     /*
      * LOGIN
      */
-    if (jk_b_append_byte(msg, AJP14_LOGINIT_CMD))
+    if (jk_b_append_byte(msg, AJP14_LOGINIT_CMD)) {
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
-
+    }
     /*
      * NEGOCIATION FLAGS
      */
-    if (jk_b_append_long(msg, s->negociation))
+    if (jk_b_append_long(msg, s->negociation)) {
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
-
+    }
     /*
      * WEB-SERVER NAME
      */
     if (jk_b_append_string(msg, s->web_server_name)) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_marshal_login_init_into_msgb - Error appending the web_server_name string\n");
+               "failed appending the web_server_name string");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
@@ -96,14 +102,18 @@ int ajp14_marshal_login_init_into_msgb(jk_msg_buf_t *msg,
 int ajp14_unmarshal_login_seed(jk_msg_buf_t *msg,
                                jk_login_service_t *s, jk_logger_t *l)
 {
+    JK_TRACE_ENTER(l);
+
     if (jk_b_get_bytes
         (msg, (unsigned char *)s->entropy, AJP14_ENTROPY_SEED_LEN) < 0) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_unmarshal_login_seed - can't get seed\n");
+               "can't get seed");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
     s->entropy[AJP14_ENTROPY_SEED_LEN] = 0;     /* Just to have a CString */
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
@@ -119,7 +129,7 @@ int ajp14_unmarshal_login_seed(jk_msg_buf_t *msg,
 int ajp14_marshal_login_comp_into_msgb(jk_msg_buf_t *msg,
                                        jk_login_service_t *s, jk_logger_t *l)
 {
-    jk_log(l, JK_LOG_DEBUG, "Into ajp14_marshal_login_comp_into_msgb\n");
+    JK_TRACE_ENTER(l);
 
     /* To be on the safe side */
     jk_b_reset(msg);
@@ -127,9 +137,10 @@ int ajp14_marshal_login_comp_into_msgb(jk_msg_buf_t *msg,
     /*
      * LOGIN
      */
-    if (jk_b_append_byte(msg, AJP14_LOGCOMP_CMD))
+    if (jk_b_append_byte(msg, AJP14_LOGCOMP_CMD)) {
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
-
+    }
     /*
      * COMPUTED-SEED
      */
@@ -137,10 +148,12 @@ int ajp14_marshal_login_comp_into_msgb(jk_msg_buf_t *msg,
         (msg, (const unsigned char *)s->computed_key,
          AJP14_COMPUTED_KEY_LEN)) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_marshal_login_comp_into_msgb - Error appending the COMPUTED MD5 bytes\n");
+               "failed appending the COMPUTED MD5 bytes");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
@@ -160,11 +173,14 @@ int ajp14_unmarshal_log_ok(jk_msg_buf_t *msg,
     unsigned long nego;
     char *sname;
 
+    JK_TRACE_ENTER(l);
+
     nego = jk_b_get_long(msg);
 
     if (nego == 0xFFFFFFFF) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_unmarshal_log_ok - can't get negociated data\n");
+               "can't get negociated data");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
@@ -172,7 +188,8 @@ int ajp14_unmarshal_log_ok(jk_msg_buf_t *msg,
 
     if (!sname) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_unmarshal_log_ok - can't get servlet engine name\n");
+               "can't get servlet engine name");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
@@ -183,10 +200,12 @@ int ajp14_unmarshal_log_ok(jk_msg_buf_t *msg,
 
     if (!s->servlet_engine_name) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_unmarshal_log_ok - can't malloc servlet engine name\n");
+               "can't malloc servlet engine name");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
@@ -204,18 +223,20 @@ int ajp14_unmarshal_log_nok(jk_msg_buf_t *msg, jk_logger_t *l)
 {
     unsigned long status;
 
-    jk_log(l, JK_LOG_DEBUG, "Into ajp14_unmarshal_log_nok\n");
+    JK_TRACE_ENTER(l);
 
     status = jk_b_get_long(msg);
 
     if (status == 0xFFFFFFFF) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_unmarshal_log_nok - can't get failure code\n");
+               "can't get failure code");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
     jk_log(l, JK_LOG_INFO, "Can't Log with servlet engine - code %08lx",
            status);
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
@@ -232,7 +253,8 @@ int ajp14_unmarshal_log_nok(jk_msg_buf_t *msg, jk_logger_t *l)
 int ajp14_marshal_shutdown_into_msgb(jk_msg_buf_t *msg,
                                      jk_login_service_t *s, jk_logger_t *l)
 {
-    jk_log(l, JK_LOG_DEBUG, "Into ajp14_marshal_shutdown_into_msgb\n");
+
+    JK_TRACE_ENTER(l);
 
     /* To be on the safe side */
     jk_b_reset(msg);
@@ -240,9 +262,10 @@ int ajp14_marshal_shutdown_into_msgb(jk_msg_buf_t *msg,
     /*
      * SHUTDOWN CMD
      */
-    if (jk_b_append_byte(msg, AJP14_SHUTDOWN_CMD))
+    if (jk_b_append_byte(msg, AJP14_SHUTDOWN_CMD)) {
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
-
+    }
     /*
      * COMPUTED-SEED
      */
@@ -250,10 +273,12 @@ int ajp14_marshal_shutdown_into_msgb(jk_msg_buf_t *msg,
         (msg, (const unsigned char *)s->computed_key,
          AJP14_COMPUTED_KEY_LEN)) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_marshal_shutdown_into_msgb - Error appending the COMPUTED MD5 bytes\n");
+               "failed appending the COMPUTED MD5 bytes");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
@@ -269,18 +294,19 @@ int ajp14_unmarshal_shutdown_nok(jk_msg_buf_t *msg, jk_logger_t *l)
 {
     unsigned long status;
 
-    jk_log(l, JK_LOG_DEBUG, "Into ajp14_unmarshal_shutdown_nok\n");
-
+    JK_TRACE_ENTER(l);
     status = jk_b_get_long(msg);
 
     if (status == 0xFFFFFFFF) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_unmarshal_shutdown_nok - can't get failure code\n");
+               "can't get failure code");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
     jk_log(l, JK_LOG_INFO, "Can't shutdown servlet engine - code %08lx",
            status);
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
@@ -296,7 +322,7 @@ int ajp14_unmarshal_shutdown_nok(jk_msg_buf_t *msg, jk_logger_t *l)
 int ajp14_marshal_unknown_packet_into_msgb(jk_msg_buf_t *msg,
                                            jk_msg_buf_t *unk, jk_logger_t *l)
 {
-    jk_log(l, JK_LOG_DEBUG, "Into ajp14_marshal_unknown_packet_into_msgb\n");
+    JK_TRACE_ENTER(l);
 
     /* To be on the safe side */
     jk_b_reset(msg);
@@ -304,25 +330,29 @@ int ajp14_marshal_unknown_packet_into_msgb(jk_msg_buf_t *msg,
     /*
      * UNKNOWN PACKET CMD
      */
-    if (jk_b_append_byte(msg, AJP14_UNKNOW_PACKET_CMD))
+    if (jk_b_append_byte(msg, AJP14_UNKNOW_PACKET_CMD)) {
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
-
+    }
     /*
      * UNHANDLED MESSAGE SIZE
      */
-    if (jk_b_append_int(msg, (unsigned short)jk_b_get_len(unk)))
+    if (jk_b_append_int(msg, (unsigned short)jk_b_get_len(unk))) {
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
-
+    }
     /*
      * UNHANDLED MESSAGE (Question : Did we have to send all the message or only part of)
      *                                       (           ie: only 1k max                                                                )
      */
     if (jk_b_append_bytes(msg, jk_b_get_buff(unk), jk_b_get_len(unk))) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_marshal_unknown_packet_into_msgb - Error appending the UNHANDLED MESSAGE\n");
+               "failed appending the UNHANDLED MESSAGE");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
@@ -338,7 +368,7 @@ int ajp14_marshal_unknown_packet_into_msgb(jk_msg_buf_t *msg,
 int ajp14_marshal_context_query_into_msgb(jk_msg_buf_t *msg,
                                           char *virtual, jk_logger_t *l)
 {
-    jk_log(l, JK_LOG_DEBUG, "Into ajp14_marshal_context_query_into_msgb\n");
+    JK_TRACE_ENTER(l);
 
     /* To be on the safe side */
     jk_b_reset(msg);
@@ -346,18 +376,21 @@ int ajp14_marshal_context_query_into_msgb(jk_msg_buf_t *msg,
     /*
      * CONTEXT QUERY CMD
      */
-    if (jk_b_append_byte(msg, AJP14_CONTEXT_QRY_CMD))
+    if (jk_b_append_byte(msg, AJP14_CONTEXT_QRY_CMD)) {
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
-
+    }
     /*
      * VIRTUAL HOST CSTRING
      */
     if (jk_b_append_string(msg, virtual)) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_marshal_context_query_into_msgb - Error appending the virtual host string\n");
+               "failed appending the virtual host string");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
@@ -382,23 +415,26 @@ int ajp14_unmarshal_context_info(jk_msg_buf_t *msg,
 
     vname = (char *)jk_b_get_string(msg);
 
+    JK_TRACE_ENTER(l);
     jk_log(l, JK_LOG_DEBUG,
-           "ajp14_unmarshal_context_info - get virtual %s for virtual %s\n",
-           vname, c->virtual);
+           "get virtual %s for virtual %s",
+           vname, c->virt);
 
     if (!vname) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_unmarshal_context_info - can't get virtual hostname\n");
+               "can't get virtual hostname");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
     /* Check if we get the correct virtual host */
-    if (c->virtual != NULL && vname != NULL && strcmp(c->virtual, vname)) {
+    if (c->virt != NULL && vname != NULL && strcmp(c->virt, vname)) {
         /* set the virtual name, better to add to a virtual list ? */
 
         if (context_set_virtual(c, vname) == JK_FALSE) {
             jk_log(l, JK_LOG_ERROR,
-                   "Error ajp14_unmarshal_context_info - can't malloc virtual hostname\n");
+                   "can't malloc virtual hostname");
+            JK_TRACE_EXIT(l);
             return JK_FALSE;
         }
     }
@@ -409,12 +445,13 @@ int ajp14_unmarshal_context_info(jk_msg_buf_t *msg,
 
         if (!cname) {
             jk_log(l, JK_LOG_ERROR,
-                   "Error ajp14_unmarshal_context_info - can't get context\n");
+                   "can't get context");
+            JK_TRACE_EXIT(l);
             return JK_FALSE;
         }
 
         jk_log(l, JK_LOG_DEBUG,
-               "ajp14_unmarshal_context_info - get context %s for virtual %s\n",
+               "get context %s for virtual %s",
                cname, vname);
 
         /* grab all contexts up to empty one which indicate end of contexts */
@@ -425,8 +462,9 @@ int ajp14_unmarshal_context_info(jk_msg_buf_t *msg,
 
         if (context_add_base(c, cname) == JK_FALSE) {
             jk_log(l, JK_LOG_ERROR,
-                   "Error ajp14_unmarshal_context_info - can't add/set context %s\n",
+                   "can't add/set context %s",
                    cname);
+            JK_TRACE_EXIT(l);
             return JK_FALSE;
         }
 
@@ -436,7 +474,8 @@ int ajp14_unmarshal_context_info(jk_msg_buf_t *msg,
 
             if (!uri) {
                 jk_log(l, JK_LOG_ERROR,
-                       "Error ajp14_unmarshal_context_info - can't get URI\n");
+                       "can't get URI");
+                JK_TRACE_EXIT(l);
                 return JK_FALSE;
             }
 
@@ -446,18 +485,20 @@ int ajp14_unmarshal_context_info(jk_msg_buf_t *msg,
             }
 
             jk_log(l, JK_LOG_INFO,
-                   "Got URI (%s) for virtualhost %s and context %s\n", uri,
+                   "Got URI (%s) for virtualhost %s and context %s", uri,
                    vname, cname);
 
             if (context_add_uri(c, cname, uri) == JK_FALSE) {
                 jk_log(l, JK_LOG_ERROR,
-                       "Error ajp14_unmarshal_context_info - can't add/set uri (%s) for context %s\n",
+                       "can't add/set uri (%s) for context %s",
                        uri, cname);
+                JK_TRACE_EXIT(l);
                 return JK_FALSE;
             }
         }
     }
 
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
@@ -481,7 +522,7 @@ int ajp14_marshal_context_state_into_msgb(jk_msg_buf_t *msg,
     jk_context_item_t *ci;
     int i;
 
-    jk_log(l, JK_LOG_DEBUG, "Into ajp14_marshal_context_state_into_msgb\n");
+    JK_TRACE_ENTER(l);
 
     /* To be on the safe side */
     jk_b_reset(msg);
@@ -489,15 +530,17 @@ int ajp14_marshal_context_state_into_msgb(jk_msg_buf_t *msg,
     /*
      * CONTEXT STATE CMD
      */
-    if (jk_b_append_byte(msg, AJP14_CONTEXT_STATE_CMD))
+    if (jk_b_append_byte(msg, AJP14_CONTEXT_STATE_CMD)) {
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
-
+    }
     /*
      * VIRTUAL HOST CSTRING
      */
-    if (jk_b_append_string(msg, c->virtual)) {
+    if (jk_b_append_string(msg, c->virt)) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_marshal_context_state_into_msgb - Error appending the virtual host string\n");
+               "failed appending the virtual host string");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
@@ -507,8 +550,9 @@ int ajp14_marshal_context_state_into_msgb(jk_msg_buf_t *msg,
 
         if (!ci) {
             jk_log(l, JK_LOG_ERROR,
-                   "Warning ajp14_marshal_context_state_into_msgb - unknown context %s\n",
+                   "unknown context %s",
                    cname);
+            JK_TRACE_EXIT(l);
             return JK_FALSE;
         }
 
@@ -518,8 +562,9 @@ int ajp14_marshal_context_state_into_msgb(jk_msg_buf_t *msg,
 
         if (jk_b_append_string(msg, cname)) {
             jk_log(l, JK_LOG_ERROR,
-                   "Error ajp14_marshal_context_state_into_msgb - Error appending the context string %s\n",
+                   "failed appending the context string %s",
                    cname);
+            JK_TRACE_EXIT(l);
             return JK_FALSE;
         }
     }
@@ -532,7 +577,9 @@ int ajp14_marshal_context_state_into_msgb(jk_msg_buf_t *msg,
              */
             if (jk_b_append_string(msg, c->contexts[i]->cbase)) {
                 jk_log(l, JK_LOG_ERROR,
-                       "Error ajp14_marshal_context_state_into_msgb - Error appending the context string\n");
+                       "failed appending the context string %s",
+                       c->contexts[i]->cbase);
+                JK_TRACE_EXIT(l);
                 return JK_FALSE;
             }
         }
@@ -542,10 +589,12 @@ int ajp14_marshal_context_state_into_msgb(jk_msg_buf_t *msg,
 
     if (jk_b_append_string(msg, "")) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_marshal_context_state_into_msgb - Error appending end of contexts\n");
+               "failed appending end of contexts");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
@@ -568,20 +617,23 @@ int ajp14_unmarshal_context_state_reply(jk_msg_buf_t *msg,
     char *cname;
     jk_context_item_t *ci;
 
+    JK_TRACE_ENTER(l);
     /* get virtual name */
     vname = (char *)jk_b_get_string(msg);
 
     if (!vname) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_unmarshal_context_state_reply - can't get virtual hostname\n");
+               "can't get virtual hostname");
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
     /* Check if we speak about the correct virtual */
-    if (strcmp(c->virtual, vname)) {
+    if (strcmp(c->virt, vname)) {
         jk_log(l, JK_LOG_ERROR,
-               "Error ajp14_unmarshal_context_state_reply - incorrect virtual %s instead of %s\n",
-               vname, c->virtual);
+               "incorrect virtual %s instead of %s",
+               vname, c->virt);
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
@@ -592,7 +644,8 @@ int ajp14_unmarshal_context_state_reply(jk_msg_buf_t *msg,
 
         if (!cname) {
             jk_log(l, JK_LOG_ERROR,
-                   "Error ajp14_unmarshal_context_state_reply - can't get context\n");
+                   "can't get context");
+            JK_TRACE_EXIT(l);
             return JK_FALSE;
         }
 
@@ -603,18 +656,21 @@ int ajp14_unmarshal_context_state_reply(jk_msg_buf_t *msg,
 
         if (!ci) {
             jk_log(l, JK_LOG_ERROR,
-                   "Error ajp14_unmarshal_context_state_reply - unknow context %s for virtual %s\n",
+                   "unknow context %s for virtual %s",
                    cname, vname);
+            JK_TRACE_EXIT(l);
             return JK_FALSE;
         }
 
         ci->status = jk_b_get_int(msg);
 
-        jk_log(l, JK_LOG_DEBUG,
-               "ajp14_unmarshal_context_state_reply - updated context %s to state %d\n",
-               cname, ci->status);
+        if (JK_IS_DEBUG_LEVEL(l))
+            jk_log(l, JK_LOG_DEBUG,
+                   "updated context %s to state %d",
+                   cname, ci->status);
     }
 
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
@@ -630,5 +686,9 @@ int ajp14_unmarshal_context_state_reply(jk_msg_buf_t *msg,
 int ajp14_unmarshal_context_update_cmd(jk_msg_buf_t *msg,
                                        jk_context_t *c, jk_logger_t *l)
 {
-    return (ajp14_unmarshal_context_state_reply(msg, c, l));
+    int rc;
+    JK_TRACE_ENTER(l);
+    rc = ajp14_unmarshal_context_state_reply(msg, c, l);
+    JK_TRACE_EXIT(l);
+    return rc;
 }
