@@ -17,7 +17,7 @@
 /***************************************************************************
  * Description: load balance worker header file                            *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 1.7 $                                           *
+ * Version:     $Revision: 1.14 $                                           *
  ***************************************************************************/
 
 #ifndef JK_LB_WORKER_H
@@ -25,19 +25,55 @@
 
 #include "jk_logger.h"
 #include "jk_service.h"
+#include "jk_mt.h"
+#include "jk_shm.h"
 
 #ifdef __cplusplus
 extern "C"
 {
 #endif                          /* __cplusplus */
 
-#define JK_LB_WORKER_NAME ("lb")
+#define JK_LB_WORKER_NAME     ("lb")
+#define JK_LB_WORKER_TYPE     (5)
+#define JK_LB_DEF_DOMAIN_NAME ("unknown")
 
-    int JK_METHOD lb_worker_factory(jk_worker_t **w,
-                                    const char *name, jk_logger_t *l);
+#define JK_LB_BYREQUESTS      (0)
+#define JK_LB_BYTRAFFIC       (1)
+#define JK_LB_METHOD_REQUESTS ("request")
+#define JK_LB_METHODTRAFFIC   ("traffic")
+#define WAIT_BEFORE_RECOVER   (60)
+
+struct worker_record
+{
+    jk_worker_t     *w;
+    /* Shared memory worker data */
+    jk_shm_worker_t  *s;
+    /* Current jvmRoute. Can be name or domain */
+    const char       *r;
+};
+typedef struct worker_record worker_record_t;
+
+struct lb_worker
+{
+    worker_record_t *lb_workers;
+    unsigned int num_of_workers;
+    int          lbmethod;
+
+    jk_pool_t p;
+    jk_pool_atom_t buf[TINY_POOL_SIZE];
+
+    jk_worker_t worker;
+    JK_CRIT_SEC cs; 
+
+    /* Shared memory worker data */
+    jk_shm_worker_t  *s;
+};
+typedef struct lb_worker lb_worker_t;
+
+int JK_METHOD lb_worker_factory(jk_worker_t **w,
+                                const char *name, jk_logger_t *l);
 
 #ifdef __cplusplus
 }
 #endif                          /* __cplusplus */
-
 #endif                          /* JK_LB_WORKER_H */
