@@ -18,12 +18,16 @@
  * Description: Utility functions (mainly configuration)                   *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 1.30 $                                          *
+ * Version:     $Revision: 1.61 $                                          *
  ***************************************************************************/
 
 
 #include "jk_util.h"
 #include "jk_ajp12_worker.h"
+#include "jk_ajp13_worker.h"
+#include "jk_ajp14_worker.h"
+#include "jk_lb_worker.h"
+#include "jk_mt.h"
 
 #define SYSPROPS_OF_WORKER          ("sysprops")
 #define STDERR_OF_WORKER            ("stderr")
@@ -49,19 +53,30 @@
 #define PREPOST_TIMEOUT_OF_WORKER   ("prepost_timeout")
 #define REPLY_TIMEOUT_OF_WORKER     ("reply_timeout")
 #define SOCKET_TIMEOUT_OF_WORKER    ("socket_timeout")
+#define SOCKET_BUFFER_OF_WORKER     ("socket_buffer")
 #define SOCKET_KEEPALIVE_OF_WORKER  ("socket_keepalive")
+#define RECYCLE_TIMEOUT_OF_WORKER   ("recycle_timeout")
 #define LOAD_FACTOR_OF_WORKER       ("lbfactor")
+/* deprecated directive. Use balance_workers instead */
 #define BALANCED_WORKERS            ("balanced_workers")
+#define BALANCE_WORKERS             ("balance_workers")
 #define STICKY_SESSION              ("sticky_session")
-#define LOCAL_WORKER_ONLY_FLAG      ("local_worker_only")
-#define LOCAL_WORKER_FLAG           ("local_worker")
-#define WORKER_AJP12                ("ajp12")
-#define DEFAULT_WORKER_TYPE         JK_AJP12_WORKER_NAME
-#define SECRET_KEY_OF_WORKER        ("secretkey")
+#define STICKY_SESSION_FORCE        ("sticky_session_force")
+#define DOMAIN_OF_WORKER            ("domain")
+#define REDIRECT_OF_WORKER          ("redirect")
+#define MOUNT_OF_WORKER             ("mount")
+#define METHOD_OF_WORKER            ("method")
+#define IS_WORKER_DISABLED          ("disabled")
+#define WORKER_RECOVER_TIME         ("recover_time")
 
-#define DEFAULT_WORKER              JK_AJP12_WORKER_NAME
+
+#define DEFAULT_WORKER_TYPE         JK_AJP13_WORKER_NAME
+#define SECRET_KEY_OF_WORKER        ("secretkey")
+#define RETRIES_OF_WORKER           ("retries")
+
+#define DEFAULT_WORKER              JK_AJP13_WORKER_NAME
 #define WORKER_LIST_PROPERTY_NAME   ("worker.list")
-#define DEFAULT_LB_FACTOR           (1.0)
+#define DEFAULT_LB_FACTOR           (1)
 #define LOG_FORMAT                  ("log_format")
 
 #define TOMCAT32_BRIDGE_NAME        ("tomcat32")
@@ -83,6 +98,17 @@
 #define JK_TIME_FORMAT "[%a %b %d %H:%M:%S %Y] "
 #endif
 
+
+static const char *jk_level_werbs[] = {
+    "[" JK_LOG_TRACE_VERB "] ",
+    "[" JK_LOG_DEBUG_VERB "] ",
+    "[" JK_LOG_INFO_VERB  "]  ",
+    "[" JK_LOG_WARNING_VERB  "]  ",
+    "[" JK_LOG_ERROR_VERB "] ",
+    "[" JK_LOG_EMERG_VERB "] ",
+    NULL
+};
+
 const char *jk_log_fmt = JK_TIME_FORMAT;
 
 static void set_time_str(char *str, int len)
@@ -102,7 +128,7 @@ static void set_time_str(char *str, int len)
  */
 #if !defined(HAVE_VSNPRINTF) && !defined(HAVE_APR)
 static FILE *f = NULL;
-int vsnprintf(char *str, size_t n, const char *fmt, va_list ap)
+static int vsnprintf(char *str, size_t n, const char *fmt, va_list ap)
 {
     int res;
 
@@ -122,7 +148,7 @@ int vsnprintf(char *str, size_t n, const char *fmt, va_list ap)
 }
 #endif
 #if !defined(HAVE_SNPRINTF) && !defined(HAVE_APR)
-int snprintf(char *str, size_t n, const char *fmt, ...)
+static int snprintf(char *str, size_t n, const char *fmt, ...)
 {
     va_list ap;
     int res;
@@ -143,8 +169,10 @@ static int JK_METHOD log_to_file(jk_logger_t *l, int level, const char *what)
         if (sz) {
             file_logger_t *p = l->logger_private;
             fwrite(what, 1, sz, p->logfile);
+            fputc('\n', p->logfile);
             /* [V] Flush the dam' thing! */
-            fflush(p->logfile);
+            if (l->level < JK_LOG_INFO_LEVEL)
+                fflush(p->logfile);
         }
 
         return JK_TRUE;
@@ -155,19 +183,31 @@ static int JK_METHOD log_to_file(jk_logger_t *l, int level, const char *what)
 
 int jk_parse_log_level(const char *level)
 {
+    if (0 == strcasecmp(level, JK_LOG_TRACE_VERB)) {
+        return JK_LOG_TRACE_LEVEL;
+    }
+
+    if (0 == strcasecmp(level, JK_LOG_DEBUG_VERB)) {
+        return JK_LOG_DEBUG_LEVEL;
+    }
+
     if (0 == strcasecmp(level, JK_LOG_INFO_VERB)) {
         return JK_LOG_INFO_LEVEL;
     }
 
+    if (0 == strcasecmp(level, JK_LOG_WARNING_VERB)) {
+        return JK_LOG_WARNING_LEVEL;
+    }
+
     if (0 == strcasecmp(level, JK_LOG_ERROR_VERB)) {
         return JK_LOG_ERROR_LEVEL;
     }
 
     if (0 == strcasecmp(level, JK_LOG_EMERG_VERB)) {
-        return JK_LOG_EMERG_LEVEL;
+        return JK_LOG_ERROR_LEVEL;
     }
 
-    return JK_LOG_DEBUG_LEVEL;
+    return JK_LOG_INFO_LEVEL;
 }
 
 int jk_open_file_logger(jk_logger_t **l, const char *file, int level)
@@ -205,9 +245,11 @@ int jk_close_file_logger(jk_logger_t **l)
 {
     if (l && *l) {
         file_logger_t *p = (*l)->logger_private;
-        fflush(p->logfile);
-        fclose(p->logfile);
-        free(p);
+        if (p) {
+            fflush(p->logfile);
+            fclose(p->logfile);
+            free(p);
+        }
         free(*l);
         *l = NULL;
 
@@ -217,7 +259,8 @@ int jk_close_file_logger(jk_logger_t **l)
 }
 
 int jk_log(jk_logger_t *l,
-           const char *file, int line, int level, const char *fmt, ...)
+           const char *file, int line, const char *funcname, int level,
+           const char *fmt, ...)
 {
     int rc = 0;
     if (!l || !file || !fmt) {
@@ -243,7 +286,6 @@ int jk_log(jk_logger_t *l,
             f++;
         }
 
-#ifdef USE_SPRINTF              /* until we get a snprintf function */
 #ifdef NETWARE
         buf = (char *)malloc(HUGE_BUFFER_SIZE);
         if (NULL == buf)
@@ -251,15 +293,38 @@ int jk_log(jk_logger_t *l,
 #endif
         set_time_str(buf, HUGE_BUFFER_SIZE);
         used = strlen(buf);
-        if (line)
-            used += sprintf(&buf[used], " [%s (%d)]: ", f, line);
+        
+        /* Log [pid:threadid] for debug and trace levels */
+        if (l->level < JK_LOG_INFO_LEVEL) {
+#ifdef USE_SPRINTF              /* until we get a snprintf function */        
+            used += sprintf(&buf[used], "[%04d:%04d] ", getpid(),
+                            jk_gettid());
 #else
-        set_time_str(buf, HUGE_BUFFER_SIZE);
-        used = strlen(buf);
+            used += snprintf(&buf[used], HUGE_BUFFER_SIZE, "[%04d:%04d] ",
+                             getpid(), jk_gettid());
+#endif
+            if (used < 0) {
+                return 0;
+            }
+        }
+        if (line) {
+            strcat(buf, jk_level_werbs[level]);
+            used += 8;
+        
+            if (funcname) {
+                strcat(buf, funcname);
+                strcat(buf, "::");
+                used += strlen(funcname) + 2;
+            }
+        }
+
+#ifdef USE_SPRINTF              /* until we get a snprintf function */        
         if (line)
-            used +=
-                snprintf(&buf[used], HUGE_BUFFER_SIZE, " [%s (%d)]: ", f,
-                         line);
+            used += sprintf(&buf[used], "%s (%d): ", f, line);
+#else
+        if (line)
+            used += snprintf(&buf[used], HUGE_BUFFER_SIZE, "%s (%d): ",
+                             f, line);
 #endif
         if (used < 0) {
             return 0;           /* [V] not sure what to return... */
@@ -281,7 +346,7 @@ int jk_log(jk_logger_t *l,
     return rc;
 }
 
-char *jk_get_worker_type(jk_map_t *m, const char *wname)
+const char *jk_get_worker_type(jk_map_t *m, const char *wname)
 {
     char buf[1024];
 
@@ -291,13 +356,32 @@ char *jk_get_worker_type(jk_map_t *m, const char *wname)
 
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, TYPE_OF_WORKER);
 
-    return map_get_string(m, buf, DEFAULT_WORKER_TYPE);
+    return jk_map_get_string(m, buf, DEFAULT_WORKER_TYPE);
 }
 
-char *jk_get_worker_secret(jk_map_t *m, const char *wname)
+const char *jk_get_worker_domain(jk_map_t *m, const char *wname, const char *def)
+{
+    char buf[1024];
+    if (!m || !wname) {
+        return NULL;
+    }
+    sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, DOMAIN_OF_WORKER);
+    return jk_map_get_string(m, buf, def);
+}
+
+const char *jk_get_worker_redirect(jk_map_t *m, const char *wname, const char *def)
+{
+    char buf[1024];
+    if (!m || !wname) {
+        return NULL;
+    }
+    sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, REDIRECT_OF_WORKER);
+    return jk_map_get_string(m, buf, def);
+}
+
+const char *jk_get_worker_secret(jk_map_t *m, const char *wname)
 {
     char buf[1024];
-    char *secret;
 
     if (!m || !wname) {
         return NULL;
@@ -305,9 +389,7 @@ char *jk_get_worker_secret(jk_map_t *m, const char *wname)
 
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, SECRET_OF_WORKER);
 
-    secret = map_get_string(m, buf, NULL);
-
-    return secret;
+    return jk_map_get_string(m, buf, NULL);
 }
 
 /* [V] I suggest that the following general purpose functions be used.       */
@@ -315,13 +397,13 @@ char *jk_get_worker_secret(jk_map_t *m, const char *wname)
 /*     Functions that can be simulated with these should be "deprecated".    */
 
 int jk_get_worker_str_prop(jk_map_t *m,
-                           const char *wname, const char *pname, char **prop)
+                           const char *wname, const char *pname, const char **prop)
 {
     char buf[1024];
 
     if (m && prop && wname && pname) {
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, pname);
-        *prop = map_get_string(m, buf, NULL);
+        *prop = jk_map_get_string(m, buf, NULL);
         if (*prop) {
             return JK_TRUE;
         }
@@ -337,7 +419,7 @@ int jk_get_worker_int_prop(jk_map_t *m,
     if (m && prop && wname && pname) {
         int i;
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, pname);
-        i = map_get_int(m, buf, -1);
+        i = jk_map_get_int(m, buf, -1);
         if (-1 != i) {
             *prop = i;
             return JK_TRUE;
@@ -346,7 +428,7 @@ int jk_get_worker_int_prop(jk_map_t *m,
     return JK_FALSE;
 }
 
-char *jk_get_worker_host(jk_map_t *m, const char *wname, const char *def)
+const char *jk_get_worker_host(jk_map_t *m, const char *wname, const char *def)
 {
     char buf[1024];
 
@@ -356,7 +438,7 @@ char *jk_get_worker_host(jk_map_t *m, const char *wname, const char *def)
 
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, HOST_OF_WORKER);
 
-    return map_get_string(m, buf, def);
+    return jk_map_get_string(m, buf, def);
 }
 
 int jk_get_worker_port(jk_map_t *m, const char *wname, int def)
@@ -369,7 +451,24 @@ int jk_get_worker_port(jk_map_t *m, const char *wname, int def)
 
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, PORT_OF_WORKER);
 
-    return map_get_int(m, buf, def);
+    return jk_map_get_int(m, buf, def);
+}
+
+static int def_cache_size = -1;
+int jk_get_worker_def_cache_size(int protocol)
+{
+    if (def_cache_size < 1) {
+        if (protocol == AJP14_PROTO)
+            def_cache_size = AJP14_DEF_CACHE_SZ;
+        else 
+            def_cache_size = AJP13_DEF_CACHE_SZ;
+    }
+    return def_cache_size;
+}
+
+void jk_set_worker_def_cache_size(int sz)
+{
+    def_cache_size = sz;
 }
 
 int jk_get_worker_cache_size(jk_map_t *m, const char *wname, int def)
@@ -382,7 +481,7 @@ int jk_get_worker_cache_size(jk_map_t *m, const char *wname, int def)
 
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, CACHE_OF_WORKER);
 
-    return map_get_int(m, buf, def);
+    return jk_map_get_int(m, buf, def);
 }
 
 int jk_get_worker_socket_timeout(jk_map_t *m, const char *wname, int def)
@@ -396,7 +495,38 @@ int jk_get_worker_socket_timeout(jk_map_t *m, const char *wname, int def)
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
             SOCKET_TIMEOUT_OF_WORKER);
 
-    return map_get_int(m, buf, def);
+    return jk_map_get_int(m, buf, def);
+}
+
+int jk_get_worker_recover_timeout(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+
+    if (!m || !wname) {
+        return -1;
+    }
+
+    sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
+            WORKER_RECOVER_TIME);
+
+    return jk_map_get_int(m, buf, def);
+}
+
+int jk_get_worker_socket_buffer(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+    int i;
+    if (!m || !wname) {
+        return -1;
+    }
+
+    sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
+            SOCKET_BUFFER_OF_WORKER);
+
+    i = jk_map_get_int(m, buf, 0);
+    if (i > 0 && i < def)
+        i = def;
+    return i;
 }
 
 int jk_get_worker_socket_keepalive(jk_map_t *m, const char *wname, int def)
@@ -410,7 +540,7 @@ int jk_get_worker_socket_keepalive(jk_map_t *m, const char *wname, int def)
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
             SOCKET_KEEPALIVE_OF_WORKER);
 
-    return map_get_int(m, buf, def);
+    return jk_map_get_bool(m, buf, def);
 }
 
 int jk_get_worker_cache_timeout(jk_map_t *m, const char *wname, int def)
@@ -424,7 +554,7 @@ int jk_get_worker_cache_timeout(jk_map_t *m, const char *wname, int def)
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
             CACHE_TIMEOUT_OF_WORKER);
 
-    return map_get_int(m, buf, def);
+    return jk_map_get_int(m, buf, def);
 }
 
 int jk_get_worker_connect_timeout(jk_map_t *m, const char *wname, int def)
@@ -438,7 +568,7 @@ int jk_get_worker_connect_timeout(jk_map_t *m, const char *wname, int def)
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
             CONNECT_TIMEOUT_OF_WORKER);
 
-    return map_get_int(m, buf, def);
+    return jk_map_get_int(m, buf, def);
 }
 
 int jk_get_worker_prepost_timeout(jk_map_t *m, const char *wname, int def)
@@ -452,7 +582,7 @@ int jk_get_worker_prepost_timeout(jk_map_t *m, const char *wname, int def)
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
             PREPOST_TIMEOUT_OF_WORKER);
 
-    return map_get_int(m, buf, def);
+    return jk_map_get_int(m, buf, def);
 }
 
 int jk_get_worker_reply_timeout(jk_map_t *m, const char *wname, int def)
@@ -466,7 +596,39 @@ int jk_get_worker_reply_timeout(jk_map_t *m, const char *wname, int def)
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
             REPLY_TIMEOUT_OF_WORKER);
 
-    return map_get_int(m, buf, def);
+    return jk_map_get_int(m, buf, def);
+}
+
+int jk_get_worker_recycle_timeout(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+
+    if (!m || !wname) {
+        return -1;
+    }
+
+    sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
+            RECYCLE_TIMEOUT_OF_WORKER);
+
+    return jk_map_get_int(m, buf, def);
+}
+
+int jk_get_worker_retries(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+    int rv;
+    if (!m || !wname) {
+        return -1;
+    }
+
+    sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
+            RETRIES_OF_WORKER);
+
+    rv = jk_map_get_int(m, buf, def);
+    if (rv < 1)
+        rv = 1;
+
+    return rv;        
 }
 
 int jk_get_worker_recovery_opts(jk_map_t *m, const char *wname, int def)
@@ -480,10 +642,10 @@ int jk_get_worker_recovery_opts(jk_map_t *m, const char *wname, int def)
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
             RECOVERY_OPTS_OF_WORKER);
 
-    return map_get_int(m, buf, def);
+    return jk_map_get_int(m, buf, def);
 }
 
-char *jk_get_worker_secret_key(jk_map_t *m, const char *wname)
+const char *jk_get_worker_secret_key(jk_map_t *m, const char *wname)
 {
     char buf[1024];
 
@@ -492,13 +654,13 @@ char *jk_get_worker_secret_key(jk_map_t *m, const char *wname)
     }
 
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, SECRET_KEY_OF_WORKER);
-    return map_get_string(m, buf, NULL);
+    return jk_map_get_string(m, buf, NULL);
 }
 
 int jk_get_worker_list(jk_map_t *m, char ***list, unsigned *num_of_wokers)
 {
     if (m && list && num_of_wokers) {
-        char **ar = map_get_string_list(m,
+        char **ar = jk_map_get_string_list(m,
                                         WORKER_LIST_PROPERTY_NAME,
                                         num_of_wokers,
                                         DEFAULT_WORKER);
@@ -513,12 +675,26 @@ int jk_get_worker_list(jk_map_t *m, char ***list, unsigned *num_of_wokers)
     return JK_FALSE;
 }
 
+int jk_get_is_worker_disabled(jk_map_t *m, const char *wname)
+{
+    int rc = JK_TRUE;
+    char buf[1024];
+    if (m && wname) {
+        int value;
+        sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, IS_WORKER_DISABLED);
+        value = jk_map_get_bool(m, buf, 0);
+        if (!value)
+            rc = JK_FALSE;
+    }
+    return rc;
+}
+
 void jk_set_log_format(const char *logformat)
 {
     jk_log_fmt = (logformat) ? logformat : JK_TIME_FORMAT;
 }
 
-double jk_get_lb_factor(jk_map_t *m, const char *wname)
+int jk_get_lb_factor(jk_map_t *m, const char *wname)
 {
     char buf[1024];
 
@@ -528,7 +704,7 @@ double jk_get_lb_factor(jk_map_t *m, const char *wname)
 
     sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, LOAD_FACTOR_OF_WORKER);
 
-    return map_get_double(m, buf, DEFAULT_LB_FACTOR);
+    return jk_map_get_int(m, buf, DEFAULT_LB_FACTOR);
 }
 
 int jk_get_is_sticky_session(jk_map_t *m, const char *wname)
@@ -538,54 +714,67 @@ int jk_get_is_sticky_session(jk_map_t *m, const char *wname)
     if (m && wname) {
         int value;
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, STICKY_SESSION);
-        value = map_get_int(m, buf, 1);
+        value = jk_map_get_bool(m, buf, 1);
         if (!value)
             rc = JK_FALSE;
     }
     return rc;
 }
 
-int jk_get_is_local_worker(jk_map_t *m, const char *wname)
+int jk_get_is_sticky_session_force(jk_map_t *m, const char *wname)
 {
     int rc = JK_FALSE;
     char buf[1024];
     if (m && wname) {
         int value;
-        sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, LOCAL_WORKER_FLAG);
-        value = map_get_int(m, buf, 0);
+        sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, STICKY_SESSION_FORCE);
+        value = jk_map_get_bool(m, buf, 0);
         if (value)
             rc = JK_TRUE;
     }
     return rc;
 }
 
-int jk_get_local_worker_only_flag(jk_map_t *m, const char *lb_wname)
+int jk_get_lb_method(jk_map_t *m, const char *wname)
 {
-    int rc = JK_FALSE;
     char buf[1024];
-    if (m && lb_wname) {
-        int value;
-        sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, lb_wname,
-                LOCAL_WORKER_ONLY_FLAG);
-        value = map_get_int(m, buf, 0);
-        if (value)
-            rc = JK_TRUE;
+    const char *v;
+    if (!m || !wname) {
+        return DEFAULT_LB_FACTOR;
     }
-    return rc;
+
+    sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, METHOD_OF_WORKER);
+    v = jk_map_get_string(m, buf, NULL);
+    if (!v)
+        return JK_LB_BYREQUESTS;
+    else if  (*v == 't' || *v == 'T' || *v == '1')
+        return JK_LB_BYTRAFFIC;
+    else if  (*v == 'r' || *v == 'R' || *v == '0')
+        return JK_LB_BYREQUESTS;
+    else
+        return JK_LB_BYREQUESTS;
 }
 
 int jk_get_lb_worker_list(jk_map_t *m,
                           const char *lb_wname,
-                          char ***list, unsigned *num_of_wokers)
+                          char ***list, unsigned int *num_of_wokers)
 {
     char buf[1024];
 
     if (m && list && num_of_wokers && lb_wname) {
         char **ar = NULL;
 
+        sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, lb_wname,
+                BALANCE_WORKERS);
+        ar = jk_map_get_string_list(m, buf, num_of_wokers, NULL);
+        if (ar) {
+            *list = ar;
+            return JK_TRUE;
+        }
+        /* Try old balanced_workers directive */
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, lb_wname,
                 BALANCED_WORKERS);
-        ar = map_get_string_list(m, buf, num_of_wokers, NULL);
+        ar = jk_map_get_string_list(m, buf, num_of_wokers, NULL);
         if (ar) {
             *list = ar;
             return JK_TRUE;
@@ -597,6 +786,29 @@ int jk_get_lb_worker_list(jk_map_t *m,
     return JK_FALSE;
 }
 
+int jk_get_worker_mount_list(jk_map_t *m,
+                             const char *wname,
+                             char ***list, unsigned int *num_of_maps)
+{
+    char buf[1024];
+
+    if (m && list && num_of_maps && wname) {
+        char **ar = NULL;
+
+        sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
+                MOUNT_OF_WORKER);
+        ar = jk_map_get_string_list(m, buf, num_of_maps, NULL);
+        if (ar) {
+            *list = ar;
+            return JK_TRUE;
+        }
+        *list = NULL;
+        *num_of_maps = 0;
+    }
+
+    return JK_FALSE;
+}
+
 int jk_get_worker_mx(jk_map_t *m, const char *wname, unsigned *mx)
 {
     char buf[1024];
@@ -605,7 +817,7 @@ int jk_get_worker_mx(jk_map_t *m, const char *wname, unsigned *mx)
         int i;
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, MX_OF_WORKER);
 
-        i = map_get_int(m, buf, -1);
+        i = jk_map_get_int(m, buf, -1);
         if (-1 != i) {
             *mx = (unsigned)i;
             return JK_TRUE;
@@ -623,7 +835,7 @@ int jk_get_worker_ms(jk_map_t *m, const char *wname, unsigned *ms)
         int i;
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, MS_OF_WORKER);
 
-        i = map_get_int(m, buf, -1);
+        i = jk_map_get_int(m, buf, -1);
         if (-1 != i) {
             *ms = (unsigned)i;
             return JK_TRUE;
@@ -633,14 +845,14 @@ int jk_get_worker_ms(jk_map_t *m, const char *wname, unsigned *ms)
     return JK_FALSE;
 }
 
-int jk_get_worker_classpath(jk_map_t *m, const char *wname, char **cp)
+int jk_get_worker_classpath(jk_map_t *m, const char *wname, const char **cp)
 {
     char buf[1024];
 
     if (m && cp && wname) {
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, CP_OF_WORKER);
 
-        *cp = map_get_string(m, buf, NULL);
+        *cp = jk_map_get_string(m, buf, NULL);
         if (*cp) {
             return JK_TRUE;
         }
@@ -652,12 +864,12 @@ int jk_get_worker_classpath(jk_map_t *m, const char *wname, char **cp)
 int jk_get_worker_bridge_type(jk_map_t *m, const char *wname, unsigned *bt)
 {
     char buf[1024];
-    char *type;
+    const char *type;
 
     if (m && bt && wname) {
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, BRIDGE_OF_WORKER);
 
-        type = map_get_string(m, buf, NULL);
+        type = jk_map_get_string(m, buf, NULL);
 
         if (type) {
             if (!strcasecmp(type, TOMCAT32_BRIDGE_NAME))
@@ -678,14 +890,14 @@ int jk_get_worker_bridge_type(jk_map_t *m, const char *wname, unsigned *bt)
     return JK_FALSE;
 }
 
-int jk_get_worker_jvm_path(jk_map_t *m, const char *wname, char **vm_path)
+int jk_get_worker_jvm_path(jk_map_t *m, const char *wname, const char **vm_path)
 {
     char buf[1024];
 
     if (m && vm_path && wname) {
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, JVM_OF_WORKER);
 
-        *vm_path = map_get_string(m, buf, NULL);
+        *vm_path = jk_map_get_string(m, buf, NULL);
         if (*vm_path) {
             return JK_TRUE;
         }
@@ -695,7 +907,7 @@ int jk_get_worker_jvm_path(jk_map_t *m, const char *wname, char **vm_path)
 }
 
 /* [V] This is unused. currently. */
-int jk_get_worker_callback_dll(jk_map_t *m, const char *wname, char **cb_path)
+int jk_get_worker_callback_dll(jk_map_t *m, const char *wname, const char **cb_path)
 {
     char buf[1024];
 
@@ -703,7 +915,7 @@ int jk_get_worker_callback_dll(jk_map_t *m, const char *wname, char **cb_path)
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname,
                 NATIVE_LIB_OF_WORKER);
 
-        *cb_path = map_get_string(m, buf, NULL);
+        *cb_path = jk_map_get_string(m, buf, NULL);
         if (*cb_path) {
             return JK_TRUE;
         }
@@ -712,14 +924,14 @@ int jk_get_worker_callback_dll(jk_map_t *m, const char *wname, char **cb_path)
     return JK_FALSE;
 }
 
-int jk_get_worker_cmd_line(jk_map_t *m, const char *wname, char **cmd_line)
+int jk_get_worker_cmd_line(jk_map_t *m, const char *wname, const char **cmd_line)
 {
     char buf[1024];
 
     if (m && cmd_line && wname) {
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, CMD_LINE_OF_WORKER);
 
-        *cmd_line = map_get_string(m, buf, NULL);
+        *cmd_line = jk_map_get_string(m, buf, NULL);
         if (*cmd_line) {
             return JK_TRUE;
         }
@@ -770,14 +982,14 @@ int jk_is_cmd_line_poperty(const char *prp_name)
     return jk_is_some_property(prp_name, CMD_LINE_OF_WORKER);
 }
 
-int jk_get_worker_stdout(jk_map_t *m, const char *wname, char **stdout_name)
+int jk_get_worker_stdout(jk_map_t *m, const char *wname, const char **stdout_name)
 {
     char buf[1024];
 
     if (m && stdout_name && wname) {
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, STDOUT_OF_WORKER);
 
-        *stdout_name = map_get_string(m, buf, NULL);
+        *stdout_name = jk_map_get_string(m, buf, NULL);
         if (*stdout_name) {
             return JK_TRUE;
         }
@@ -786,14 +998,14 @@ int jk_get_worker_stdout(jk_map_t *m, const char *wname, char **stdout_name)
     return JK_FALSE;
 }
 
-int jk_get_worker_stderr(jk_map_t *m, const char *wname, char **stderr_name)
+int jk_get_worker_stderr(jk_map_t *m, const char *wname, const char **stderr_name)
 {
     char buf[1024];
 
     if (m && stderr_name && wname) {
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, STDERR_OF_WORKER);
 
-        *stderr_name = map_get_string(m, buf, NULL);
+        *stderr_name = jk_map_get_string(m, buf, NULL);
         if (*stderr_name) {
             return JK_TRUE;
         }
@@ -802,14 +1014,14 @@ int jk_get_worker_stderr(jk_map_t *m, const char *wname, char **stderr_name)
     return JK_FALSE;
 }
 
-int jk_get_worker_sysprops(jk_map_t *m, const char *wname, char **sysprops)
+int jk_get_worker_sysprops(jk_map_t *m, const char *wname, const char **sysprops)
 {
     char buf[1024];
 
     if (m && sysprops && wname) {
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, SYSPROPS_OF_WORKER);
 
-        *sysprops = map_get_string(m, buf, NULL);
+        *sysprops = jk_map_get_string(m, buf, NULL);
         if (*sysprops) {
             return JK_TRUE;
         }
@@ -818,14 +1030,14 @@ int jk_get_worker_sysprops(jk_map_t *m, const char *wname, char **sysprops)
     return JK_FALSE;
 }
 
-int jk_get_worker_libpath(jk_map_t *m, const char *wname, char **libpath)
+int jk_get_worker_libpath(jk_map_t *m, const char *wname, const char **libpath)
 {
     char buf[1024];
 
     if (m && libpath && wname) {
         sprintf(buf, "%s.%s.%s", PREFIX_OF_WORKER, wname, LIBPATH_OF_WORKER);
 
-        *libpath = map_get_string(m, buf, NULL);
+        *libpath = jk_map_get_string(m, buf, NULL);
         if (*libpath) {
             return JK_TRUE;
         }
@@ -935,4 +1147,21 @@ void jk_init_ws_service(jk_ws_service_t *s)
     s->attributes_values = NULL;
     s->num_attributes = 0;
     s->jvm_route = NULL;
+    s->retries = JK_RETRIES;
 }
+
+#ifdef _MT_CODE_PTHREAD
+int jk_gettid()
+{
+    pthread_t t = pthread_self();
+#ifdef AS400
+    /* OS400 use 64 bits ThreadId, get only low 32 bits for now */
+    pthread_id_np_t       tid;
+    pthread_getunique_np(&t, &tid);      
+    return ((int)(tid.intId.lo & 0xFFFFFFFF));
+#else
+    int tid = (int)(t & 0xFFFF);
+    return tid;
+#endif /* AS400 */
+}
+#endif
