@@ -19,7 +19,8 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Larry Isaacs <larryi@apache.org>                           *
  * Author:      Ignacio J. Ortega <nacho@apache.org>                       *
- * Version:     $Revision: 1.25 $                                           *
+ * Author:      Mladen Turk <mturk@apache.org>                             *
+ * Version:     $Revision: 1.45 $                                          *
  ***************************************************************************/
 
 // This define is needed to include wincrypt,h, needed to get client certificates
@@ -36,12 +37,13 @@
 #include "jk_service.h"
 #include "jk_worker.h"
 #include "jk_uri_worker_map.h"
+#include "jk_shm.h"
 
 #define VERSION_STRING "Jakarta/ISAPI/" JK_VERSTRING
 
 #define DEFAULT_WORKER_NAME ("ajp13")
 /*
- * We use special headers to pass values from the filter to the 
+ * We use special headers to pass values from the filter to the
  * extension. These values are:
  *
  * 1. The real URI before redirection took place
@@ -71,7 +73,20 @@
 #define BAD_PATH        -2
 #define MAX_SERVERNAME  128
 
-#define JK_TOLOWER(x)   ((char)tolower((unsigned char)(x)))
+#define HTML_ERROR_400          "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"  \
+                                "<HTML><HEAD><TITLE>Bad request!</TITLE></HEAD>"                    \
+                                "<BODY><H1>Bad request!</H1><DL><DD>\n"                             \
+                                "Your browser (or proxy) sent a request that "                      \
+                                "this server could not understand.</DL></DD></BODY></HTML>"
+
+#define HTML_ERROR_404          "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"  \
+                                "<HTML><HEAD><TITLE>Object not found!</TITLE></HEAD>"               \
+                                "<BODY><H1>The requested URL was not found on this server"          \
+                                "</H1><DL><DD>\nIf you entered the URL manually please check your"  \
+                                "spelling and try again.</DL></DD></BODY></HTML>"
+
+
+#define JK_TOLOWER(x)   ((char)tolower((BYTE)(x)))
 
 #define GET_SERVER_VARIABLE_VALUE(name, place)          \
   do {                                                  \
@@ -80,8 +95,7 @@
     if (get_server_value(private_data->lpEcb,           \
                         (name),                         \
                         huge_buf,                       \
-                        huge_buf_sz,                    \
-                        "")) {                          \
+                        huge_buf_sz)) {                 \
         (place) = jk_pool_strdup(&private_data->p,      \
                                  huge_buf);             \
   } } while(0)
@@ -92,8 +106,7 @@
     if (get_server_value(private_data->lpEcb,               \
                         (name),                             \
                         huge_buf,                           \
-                        huge_buf_sz,                        \
-                        "")) {                              \
+                        huge_buf_sz)) {                     \
         (place) = atoi(huge_buf);                           \
         if (0 == (place)) {                                 \
             (place) = def;                                  \
@@ -112,14 +125,14 @@ static jk_uri_worker_map_t *uw_map = NULL;
 static jk_logger_t *logger = NULL;
 static char *SERVER_NAME = "SERVER_NAME";
 static char *SERVER_SOFTWARE = "SERVER_SOFTWARE";
-
+static char *CONTENT_TYPE = "Content-Type:text/html\r\n\r\n";
 
 static char extension_uri[INTERNET_MAX_URL_LENGTH] =
     "/jakarta/isapi_redirect.dll";
 static char log_file[MAX_PATH * 2];
 static int log_level = JK_LOG_EMERG_LEVEL;
 static char worker_file[MAX_PATH * 2];
-static char worker_mount_file[MAX_PATH * 2];
+static char worker_mount_file[MAX_PATH * 2] = {0};
 
 #define URI_SELECT_OPT_PARSED       0
 #define URI_SELECT_OPT_UNPARSED     1
@@ -135,7 +148,7 @@ struct isapi_private_data_t
     jk_pool_t p;
 
     int request_started;
-    unsigned bytes_read_so_far;
+    unsigned int bytes_read_so_far;
     LPEXTENSION_CONTROL_BLOCK lpEcb;
 };
 
@@ -145,12 +158,12 @@ static int JK_METHOD start_response(jk_ws_service_t *s,
                                     const char *reason,
                                     const char *const *header_names,
                                     const char *const *header_values,
-                                    unsigned num_of_headers);
+                                    unsigned int num_of_headers);
 
 static int JK_METHOD read(jk_ws_service_t *s,
-                          void *b, unsigned l, unsigned *a);
+                          void *b, unsigned int l, unsigned int *a);
 
-static int JK_METHOD write(jk_ws_service_t *s, const void *b, unsigned l);
+static int JK_METHOD write(jk_ws_service_t *s, const void *b, unsigned int l);
 
 static int init_ws_service(isapi_private_data_t * private_data,
                            jk_ws_service_t *s, char **worker_name);
@@ -167,7 +180,7 @@ static int get_registry_config_parameter(HKEY hkey,
 
 static int get_server_value(LPEXTENSION_CONTROL_BLOCK lpEcb,
                             char *name,
-                            char *buf, DWORD bufsz, char *def_val);
+                            char *buf, DWORD bufsz);
 
 static int base64_encode_cert_len(int len);
 
@@ -257,7 +270,7 @@ static void getparents(char *name)
             else
                 l = 0;
             n = l;
-            while ((name[n] = name[m])) {
+            while ((name[n] = name[m]) != '\0') {
                 n++;
                 m++;
             }
@@ -287,7 +300,7 @@ static void getparents(char *name)
 
 #define T_OS_ESCAPE_PATH    (4)
 
-static const unsigned char test_char_table[256] = {
+static const BYTE test_char_table[256] = {
      0, 14, 14, 14, 14, 14, 14, 14, 14, 14, 15, 14, 14, 14, 14, 14,
     14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,
     14,  0,  7,  6,  1,  6,  1,  1,  9,  9,  1,  0,  8,  0,  0, 10,
@@ -306,11 +319,11 @@ static const unsigned char test_char_table[256] = {
      6,  6,  6,  6,  6,  6,  6,  6,  6,  6,  6,  6,  6,  6,  6,  6
 };
 
-#define TEST_CHAR(c, f) (test_char_table[(unsigned)(c)] & (f))
+#define TEST_CHAR(c, f) (test_char_table[(unsigned int)(c)] & (f))
 
 static const char c2x_table[] = "0123456789abcdef";
 
-static unsigned char *c2x(unsigned what, unsigned char *where)
+static BYTE *c2x(unsigned int what, BYTE *where)
 {
     *where++ = '%';
     *where++ = c2x_table[what >> 4];
@@ -318,12 +331,67 @@ static unsigned char *c2x(unsigned what, unsigned char *where)
     return where;
 }
 
+static char *status_reason(int status)
+{
+    static struct reasons {
+        int status;
+        char *reason;
+    } *r, reasons[] = {
+        { 100, "Continue" },
+        { 101, "Switching Protocols" },
+        { 200, "OK" },
+        { 201, "Created" },
+        { 202, "Accepted" },
+        { 203, "Non-Authoritative Information" },
+        { 204, "No Content" },
+        { 205, "Reset Content" },
+        { 206, "Partial Content" },
+        { 300, "Multiple Choices" },
+        { 301, "Moved Permanently" },
+        { 302, "Moved Temporarily" },
+        { 303, "See Other" },
+        { 304, "Not Modified" },
+        { 305, "Use Proxy" },
+        { 400, "Bad Request" },
+        { 401, "Unauthorized" },
+        { 402, "Payment Required" },
+        { 403, "Forbidden" },
+        { 404, "Not Found" },
+        { 405, "Method Not Allowed" },
+        { 406, "Not Acceptable" },
+        { 407, "Proxy Authentication Required" },
+        { 408, "Request Timeout" },
+        { 409, "Conflict" },
+        { 410, "Gone" },
+        { 411, "Length Required" },
+        { 412, "Precondition Failed" },
+        { 413, "Request Entity Too Large" },
+        { 414, "Request-URI Too Long" },
+        { 415, "Unsupported Media Type" },
+        { 500, "Internal Server Error" },
+        { 501, "Not Implemented" },
+        { 502, "Bad Gateway" },
+        { 503, "Service Unavailable" },
+        { 504, "Gateway Timeout" },
+        { 505, "HTTP Version Not Supported" },
+        { 000, NULL}
+    };
+
+    r = reasons;
+    while (r->status <= status)
+        if (r->status == status)
+            return r->reason;
+        else
+            r++;
+    return "No Reason";
+}
+
 static int escape_url(const char *path, char *dest, int destsize)
 {
-    const unsigned char *s = (const unsigned char *)path;
-    unsigned char *d = (unsigned char *)dest;
-    unsigned char *e = d + destsize - 1;
-    unsigned char *ee = d + destsize - 3;
+    const BYTE *s = (const BYTE *)path;
+    BYTE *d = (BYTE *)dest;
+    BYTE *e = d + destsize - 1;
+    BYTE *ee = d + destsize - 3;
 
     while (*s) {
         if (TEST_CHAR(*s, T_OS_ESCAPE_PATH)) {
@@ -342,17 +410,33 @@ static int escape_url(const char *path, char *dest, int destsize)
     return JK_TRUE;
 }
 
-static int uri_is_web_inf(char *uri)
+/*
+ * Find the first occurrence of find in s.
+ */
+static char *stristr(const char *s, const char *find)
 {
-    char *c = uri;
-    while (*c) {
-        *c = JK_TOLOWER(*c);
-        c++;
+    char c, sc;
+    size_t len;
+
+    if ((c = tolower((unsigned char)(*find++))) != 0) {
+        len = strlen(find);
+        do {
+            do {
+                if ((sc = tolower((unsigned char)(*s++))) == 0)
+                    return (NULL);
+            } while (sc != c);
+        } while (strnicmp(s, find, len) != 0);
+        s--;
     }
-    if (strstr(uri, "web-inf")) {
+    return ((char *)s);
+}
+
+static int uri_is_web_inf(const char *uri)
+{
+    if (stristr(uri, "web-inf")) {
         return JK_TRUE;
     }
-    if (strstr(uri, "meta-inf")) {
+    if (stristr(uri, "meta-inf")) {
         return JK_TRUE;
     }
 
@@ -362,16 +446,14 @@ static int uri_is_web_inf(char *uri)
 static void write_error_response(PHTTP_FILTER_CONTEXT pfc, char *status,
                                  char *msg)
 {
-    char ctype[30];
-    size_t len = strlen(msg);
-
-    strcpy(ctype, "Content-Type:text/html\r\n\r\n");
+    DWORD len = (DWORD)strlen(msg);
 
     /* reject !!! */
+    pfc->AddResponseHeaders(pfc, CONTENT_TYPE, 0);
     pfc->ServerSupportFunction(pfc,
                                SF_REQ_SEND_RESPONSE_HEADER,
-                               status, (DWORD) &ctype[0], 0);
-    pfc->WriteClient(pfc, msg, (LPDWORD)&len, 0);
+                               status, 0, 0);
+    pfc->WriteClient(pfc, msg, &len, 0);
 }
 
 
@@ -380,16 +462,16 @@ static int JK_METHOD start_response(jk_ws_service_t *s,
                                     const char *reason,
                                     const char *const *header_names,
                                     const char *const *header_values,
-                                    unsigned num_of_headers)
+                                    unsigned int num_of_headers)
 {
     static char crlf[3] = { (char)13, (char)10, '\0' };
 
-    jk_log(logger, JK_LOG_DEBUG, "Into jk_ws_service_t::start_response\n");
-
+    JK_TRACE_ENTER(logger);
     if (status < 100 || status > 1000) {
         jk_log(logger, JK_LOG_ERROR,
-               "jk_ws_service_t::start_response, invalid status %d\n",
+               "invalid status %d",
                status);
+        JK_TRACE_EXIT(logger);
         return JK_FALSE;
     }
 
@@ -406,7 +488,7 @@ static int JK_METHOD start_response(jk_ws_service_t *s,
              * Create the status line
              */
             if (!reason) {
-                reason = "";
+                reason = status_reason(status);
             }
             status_str = (char *)_alloca((6 + strlen(reason)) * sizeof(char));
             sprintf(status_str, "%d %s", status, reason);
@@ -445,26 +527,25 @@ static int JK_METHOD start_response(jk_ws_service_t *s,
                                                  (LPDWORD) &len_of_status,
                                                  (LPDWORD) headers_str)) {
                 jk_log(logger, JK_LOG_ERROR,
-                       "jk_ws_service_t::start_response, ServerSupportFunction failed\n");
+                       "HSE_REQ_SEND_RESPONSE_HEADER failed");
+                JK_TRACE_EXIT(logger);
                 return JK_FALSE;
             }
-
-
         }
+        JK_TRACE_EXIT(logger);
         return JK_TRUE;
 
     }
 
-    jk_log(logger, JK_LOG_ERROR,
-           "jk_ws_service_t::start_response, NULL parameters\n");
-
+    JK_LOG_NULL_PARAMS(logger);
+    JK_TRACE_EXIT(logger);
     return JK_FALSE;
 }
 
 static int JK_METHOD read(jk_ws_service_t *s,
-                          void *b, unsigned l, unsigned *a)
+                          void *b, unsigned int l, unsigned int *a)
 {
-    jk_log(logger, JK_LOG_DEBUG, "Into jk_ws_service_t::read\n");
+    JK_TRACE_ENTER(logger);
 
     if (s && s->ws_private && b && a) {
         isapi_private_data_t *p = s->ws_private;
@@ -481,7 +562,7 @@ static int JK_METHOD read(jk_ws_service_t *s,
             }
             else {
                 /*
-                 * Try to copy what we already have 
+                 * Try to copy what we already have
                  */
                 if (already_read > 0) {
                     memcpy(buf, p->lpEcb->lpbData + p->bytes_read_so_far,
@@ -501,27 +582,30 @@ static int JK_METHOD read(jk_ws_service_t *s,
                 }
                 else {
                     jk_log(logger, JK_LOG_ERROR,
-                           "jk_ws_service_t::read, ReadClient failed\n");
+                           "ReadClient failed with %08x", GetLastError());
+                    JK_TRACE_EXIT(logger);
                     return JK_FALSE;
                 }
             }
         }
+        JK_TRACE_EXIT(logger);
         return JK_TRUE;
     }
 
-    jk_log(logger, JK_LOG_ERROR, "jk_ws_service_t::read, NULL parameters\n");
+    JK_LOG_NULL_PARAMS(logger);
+    JK_TRACE_EXIT(logger);
     return JK_FALSE;
 }
 
-static int JK_METHOD write(jk_ws_service_t *s, const void *b, unsigned l)
+static int JK_METHOD write(jk_ws_service_t *s, const void *b, unsigned int l)
 {
-    jk_log(logger, JK_LOG_DEBUG, "Into jk_ws_service_t::write\n");
+    JK_TRACE_ENTER(logger);
 
     if (s && s->ws_private && b) {
         isapi_private_data_t *p = s->ws_private;
 
         if (l) {
-            unsigned written = 0;
+            unsigned int written = 0;
             char *buf = (char *)b;
 
             if (!p->request_started) {
@@ -533,19 +617,21 @@ static int JK_METHOD write(jk_ws_service_t *s, const void *b, unsigned l)
                 if (!p->lpEcb->WriteClient(p->lpEcb->ConnID,
                                            buf + written, &try_to_write, 0)) {
                     jk_log(logger, JK_LOG_ERROR,
-                           "jk_ws_service_t::write, WriteClient failed\n");
+                           "WriteClient failed with %08x", GetLastError());
+                    JK_TRACE_EXIT(logger);
                     return JK_FALSE;
                 }
                 written += try_to_write;
             }
         }
 
+        JK_TRACE_EXIT(logger);
         return JK_TRUE;
 
     }
 
-    jk_log(logger, JK_LOG_ERROR, "jk_ws_service_t::write, NULL parameters\n");
-
+    JK_LOG_NULL_PARAMS(logger);
+    JK_TRACE_EXIT(logger);
     return JK_FALSE;
 }
 
@@ -601,10 +687,10 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
             GetServerVariable(pfc, SERVER_SOFTWARE, serverSoftware, &dwLen)) {
             iis5 = (atof(serverSoftware + 14) >= 5.0);
             if (iis5) {
-                jk_log(logger, JK_LOG_INFO, "Detected IIS >= 5.0\n");
+                jk_log(logger, JK_LOG_DEBUG, "Detected IIS >= 5.0");
             }
             else {
-                jk_log(logger, JK_LOG_INFO, "Detected IIS < 5.0\n");
+                jk_log(logger, JK_LOG_DEBUG, "Detected IIS < 5.0");
             }
         }
     }
@@ -651,9 +737,8 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                 ((PHTTP_FILTER_PREPROC_HEADERS) pvNotification)->AddHeader;
         }
 
-
-        jk_log(logger, JK_LOG_DEBUG, "HttpFilterProc started\n");
-
+        if (JK_IS_DEBUG_LEVEL(logger))
+            jk_log(logger, JK_LOG_DEBUG, "Filter started");
 
         /*
          * Just in case somebody set these headers in the request!
@@ -665,13 +750,13 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
 
         if (!GetHeader(pfc, "url", (LPVOID) uri, (LPDWORD) & sz)) {
             jk_log(logger, JK_LOG_ERROR,
-                   "HttpFilterProc error while getting the url\n");
+                   "error while getting the url");
             return SF_STATUS_REQ_ERROR;
         }
 
         if (strlen(uri)) {
             int rc;
-            char *worker = 0;
+            const char *worker = NULL;
             query = strchr(uri, '?');
             if (query) {
                 *query++ = '\0';
@@ -680,18 +765,18 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
             rc = unescape_url(uri);
             if (rc == BAD_REQUEST) {
                 jk_log(logger, JK_LOG_ERROR,
-                       "HttpFilterProc [%s] contains one or more invalid escape sequences.\n",
+                       "[%s] contains one or more invalid escape sequences.",
                        uri);
                 write_error_response(pfc, "400 Bad Request",
-                                     "<HTML><BODY><H1>Request contains invalid encoding</H1></BODY></HTML>");
+                                     HTML_ERROR_400);
                 return SF_STATUS_REQ_FINISHED;
             }
             else if (rc == BAD_PATH) {
                 jk_log(logger, JK_LOG_EMERG,
-                       "HttpFilterProc [%s] contains forbidden escape sequences.\n",
+                       "[%s] contains forbidden escape sequences.",
                        uri);
-                write_error_response(pfc, "403 Forbidden",
-                                     "<HTML><BODY><H1>Access is Forbidden</H1></BODY></HTML>");
+                write_error_response(pfc, "404 Not Found",
+                                     HTML_ERROR_404);
                 return SF_STATUS_REQ_FINISHED;
             }
             getparents(uri);
@@ -718,24 +803,44 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
             if (szHost > 0) {
                 strcat(snuri, Host);
                 strcat(snuri, uri);
-                jk_log(logger, JK_LOG_DEBUG,
-                       "In HttpFilterProc Virtual Host redirection of %s\n",
-                       snuri);
+                if (JK_IS_DEBUG_LEVEL(logger))
+                    jk_log(logger, JK_LOG_DEBUG,
+                           "Virtual Host redirection of %s",
+                           snuri);
                 worker = map_uri_to_worker(uw_map, snuri, logger);
             }
             if (!worker) {
-                jk_log(logger, JK_LOG_DEBUG,
-                       "In HttpFilterProc test Default redirection of %s\n",
-                       uri);
+                if (JK_IS_DEBUG_LEVEL(logger))
+                    jk_log(logger, JK_LOG_DEBUG,
+                           "Default redirection of %s",
+                           uri);
                 worker = map_uri_to_worker(uw_map, uri, logger);
             }
+            /*
+             * Check if somebody is feading us with his own TOMCAT data headers.
+             * We reject such postings !
+             */
+            if (JK_IS_DEBUG_LEVEL(logger))
+                jk_log(logger, JK_LOG_DEBUG,
+                       "check if [%s] is points to the web-inf directory",
+                       uri);
+
+            if (uri_is_web_inf(uri)) {
+                jk_log(logger, JK_LOG_EMERG,
+                       "[%s] points to the web-inf or meta-inf directory.\nSomebody try to hack into the site!!!",
+                       uri);
+
+                write_error_response(pfc, "404 Not Found",
+                                     HTML_ERROR_404);
+                return SF_STATUS_REQ_FINISHED;
+            }
 
             if (worker) {
                 char *forwardURI;
 
                 /* This is a servlet, should redirect ... */
                 jk_log(logger, JK_LOG_DEBUG,
-                       "HttpFilterProc [%s] is a servlet url - should redirect to %s\n",
+                       "[%s] is a servlet url - should redirect to %s",
                        uri, worker);
 
                 /* get URI we should forward */
@@ -745,23 +850,25 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                     /* restore terminator for uri portion */
                     if (query)
                         *(query - 1) = '\0';
-                    jk_log(logger, JK_LOG_DEBUG,
-                           "HttpFilterProc fowarding original URI [%s]\n",
-                           uri);
+                    if (JK_IS_DEBUG_LEVEL(logger))
+                        jk_log(logger, JK_LOG_DEBUG,
+                               "fowarding original URI [%s]",
+                               uri);
                     forwardURI = uri;
                 }
                 else if (uri_select_option == URI_SELECT_OPT_ESCAPED) {
                     if (!escape_url(uri, snuri, INTERNET_MAX_URL_LENGTH)) {
                         jk_log(logger, JK_LOG_ERROR,
-                               "HttpFilterProc [%s] re-encoding request exceeds maximum buffer size.\n",
+                               "[%s] re-encoding request exceeds maximum buffer size.",
                                uri);
                         write_error_response(pfc, "400 Bad Request",
-                                             "<HTML><BODY><H1>Request contains too many characters that need to be encoded.</H1></BODY></HTML>");
+                                             HTML_ERROR_400);
                         return SF_STATUS_REQ_FINISHED;
                     }
-                    jk_log(logger, JK_LOG_DEBUG,
-                           "HttpFilterProc fowarding escaped URI [%s]\n",
-                           snuri);
+                    if (JK_IS_DEBUG_LEVEL(logger))
+                        jk_log(logger, JK_LOG_DEBUG,
+                               "fowarding escaped URI [%s]",
+                               snuri);
                     forwardURI = snuri;
                 }
                 else {
@@ -771,10 +878,10 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                 if (!AddHeader(pfc, URI_HEADER_NAME, forwardURI) ||
                     ((query != NULL && strlen(query) > 0)
                      ? !AddHeader(pfc, QUERY_HEADER_NAME, query) : FALSE) ||
-                    !AddHeader(pfc, WORKER_HEADER_NAME, worker) ||
+                    !AddHeader(pfc, WORKER_HEADER_NAME, (LPSTR)worker) ||
                     !SetHeader(pfc, "url", extension_uri)) {
                     jk_log(logger, JK_LOG_ERROR,
-                           "HttpFilterProc error while adding request headers\n");
+                           "error while adding request headers");
                     return SF_STATUS_REQ_ERROR;
                 }
 
@@ -789,33 +896,16 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                     if (!AddHeader
                         (pfc, TOMCAT_TRANSLATE_HEADER_NAME, Translate)) {
                         jk_log(logger, JK_LOG_ERROR,
-                               "HttpFilterProc error while adding Tomcat-Translate headers\n");
+                               "error while adding Tomcat-Translate headers");
                         return SF_STATUS_REQ_ERROR;
                     }
                     SetHeader(pfc, "Translate:", NULL);
                 }
             }
             else {
-                jk_log(logger, JK_LOG_DEBUG,
-                       "HttpFilterProc [%s] is not a servlet url\n", uri);
-            }
-
-            /*
-             * Check if somebody is feading us with his own TOMCAT data headers.
-             * We reject such postings !
-             */
-            jk_log(logger, JK_LOG_DEBUG,
-                   "HttpFilterProc check if [%s] is points to the web-inf directory\n",
-                   uri);
-
-            if (uri_is_web_inf(uri)) {
-                jk_log(logger, JK_LOG_EMERG,
-                       "HttpFilterProc [%s] points to the web-inf or meta-inf directory.\nSomebody try to hack into the site!!!\n",
-                       uri);
-
-                write_error_response(pfc, "403 Forbidden",
-                                     "<HTML><BODY><H1>Access is Forbidden</H1></BODY></HTML>");
-                return SF_STATUS_REQ_FINISHED;
+                if (JK_IS_DEBUG_LEVEL(logger))
+                    jk_log(logger, JK_LOG_DEBUG,
+                           "[%s] is not a servlet url", uri);
             }
         }
     }
@@ -843,7 +933,7 @@ DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpEcb)
 
     lpEcb->dwHttpStatusCode = HTTP_STATUS_SERVER_ERROR;
 
-    jk_log(logger, JK_LOG_DEBUG, "HttpExtensionProc started\n");
+    JK_TRACE_ENTER(logger);
 
     /* Initialise jk */
     if (is_inited && !is_mapread) {
@@ -880,30 +970,34 @@ DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpEcb)
         if (init_ws_service(&private_data, &s, &worker_name)) {
             jk_worker_t *worker = wc_get_worker_for_name(worker_name, logger);
 
-            jk_log(logger, JK_LOG_DEBUG,
-                   "HttpExtensionProc %s a worker for name %s\n",
-                   worker ? "got" : "could not get", worker_name);
+            if (JK_IS_DEBUG_LEVEL(logger))
+                jk_log(logger, JK_LOG_DEBUG,
+                       "%s a worker for name %s",
+                       worker ? "got" : "could not get", worker_name);
 
             if (worker) {
                 jk_endpoint_t *e = NULL;
+                /* Update retries for this worker */
+                s.retries = worker->retries;
                 if (worker->get_endpoint(worker, &e, logger)) {
-                    int recover = JK_FALSE;
-                    if (e->service(e, &s, logger, &recover)) {
+                    int is_error = JK_HTTP_SERVER_ERROR;
+                    if (e->service(e, &s, logger, &is_error)) {
                         rc = HSE_STATUS_SUCCESS;
                         lpEcb->dwHttpStatusCode = HTTP_STATUS_OK;
                         jk_log(logger, JK_LOG_DEBUG,
-                               "HttpExtensionProc service() returned OK\n");
+                               "service() returned OK");
                     }
                     else {
+                        lpEcb->dwHttpStatusCode = is_error;
                         jk_log(logger, JK_LOG_ERROR,
-                               "HttpExtensionProc error, service() failed\n");
+                               "service() failed");
                     }
                     e->done(&e, logger);
                 }
             }
             else {
                 jk_log(logger, JK_LOG_ERROR,
-                       "HttpExtensionProc error, could not get a worker for name %s\n",
+                       "could not get a worker for name %s",
                        worker_name);
             }
         }
@@ -911,9 +1005,10 @@ DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpEcb)
     }
     else {
         jk_log(logger, JK_LOG_ERROR,
-               "HttpExtensionProc error, not initialized\n");
+               "not initialized");
     }
 
+    JK_TRACE_EXIT(logger);
     return rc;
 }
 
@@ -990,62 +1085,42 @@ static int init_jk(char *serverName)
     if (!jk_open_file_logger(&logger, log_file, log_level)) {
         logger = NULL;
     }
+     /* Simulate shared memory
+      * For now use fixed size.
+      */
+     jk_shm_open(NULL, JK_SHM_DEF_SIZE, logger);
+
+     /* 10 is minimum supported on WINXP */
+     jk_set_worker_def_cache_size(10);
+
     /* Logging the initialization type: registry or properties file in virtual dir
      */
-    if (using_ini_file) {
-        jk_log(logger, JK_LOG_DEBUG, "Using ini file %s.\n", ini_file_name);
-    }
-    else {
-        jk_log(logger, JK_LOG_DEBUG, "Using registry.\n");
-    }
-    jk_log(logger, JK_LOG_DEBUG, "Using log file %s.\n", log_file);
-    jk_log(logger, JK_LOG_DEBUG, "Using log level %d.\n", log_level);
-    jk_log(logger, JK_LOG_DEBUG, "Using extension uri %s.\n", extension_uri);
-    jk_log(logger, JK_LOG_DEBUG, "Using worker file %s.\n", worker_file);
-    jk_log(logger, JK_LOG_DEBUG, "Using worker mount file %s.\n",
-           worker_mount_file);
-    jk_log(logger, JK_LOG_DEBUG, "Using uri select %d.\n", uri_select_option);
-
-    if (map_alloc(&map)) {
-        if (map_read_properties(map, worker_mount_file)) {
-            /* remove non-mapping entries (assume they were string substitutions) */
-            jk_map_t *map2;
-            if (map_alloc(&map2)) {
-                int sz, i;
-                void *old;
-
-                sz = map_size(map);
-                for (i = 0; i < sz; i++) {
-                    char *name = map_name_at(map, i);
-                    if ('/' == *name) {
-                        map_put(map2, name, map_value_at(map, i), &old);
-                    }
-                    else {
-                        jk_log(logger, JK_LOG_DEBUG,
-                               "Ignoring worker mount file entry %s=%s.\n",
-                               name, map_value_at(map, i));
-                    }
-                }
-
-                if (uri_worker_map_alloc(&uw_map, map2, logger)) {
-                    rc = JK_TRUE;
-                }
-
-                map_free(&map2);
-            }
+    if (JK_IS_DEBUG_LEVEL(logger)) {
+        if (using_ini_file) {
+            jk_log(logger, JK_LOG_DEBUG, "Using ini file %s.", ini_file_name);
         }
         else {
-            jk_log(logger, JK_LOG_EMERG,
-                   "Unable to read worker mount file %s.\n",
-                   worker_mount_file);
+            jk_log(logger, JK_LOG_DEBUG, "Using registry.");
         }
-        map_free(&map);
-    }
 
+        jk_log(logger, JK_LOG_DEBUG, "Using log file %s.", log_file);
+        jk_log(logger, JK_LOG_DEBUG, "Using log level %d.", log_level);
+        jk_log(logger, JK_LOG_DEBUG, "Using extension uri %s.", extension_uri);
+        jk_log(logger, JK_LOG_DEBUG, "Using worker file %s.", worker_file);
+        jk_log(logger, JK_LOG_DEBUG, "Using worker mount file %s.",
+               worker_mount_file);
+        jk_log(logger, JK_LOG_DEBUG, "Using uri select %d.", uri_select_option);
+    }
+    if (uri_worker_map_alloc(&uw_map, NULL, logger)) {
+        rc = JK_FALSE;
+        uw_map->fname = worker_mount_file;
+        if (worker_mount_file[0])
+            rc = uri_worker_map_load(uw_map, logger);
+    }
     if (rc) {
         rc = JK_FALSE;
-        if (map_alloc(&map)) {
-            if (map_read_properties(map, worker_file)) {
+        if (jk_map_alloc(&map)) {
+            if (jk_map_read_properties(map, worker_file, NULL)) {
                 /* we add the URI->WORKER MAP since workers using AJP14 will feed it */
 
                 worker_env.uri_to_worker = uw_map;
@@ -1057,9 +1132,9 @@ static int init_jk(char *serverName)
             }
             else {
                 jk_log(logger, JK_LOG_EMERG,
-                       "Unable to read worker file %s.\n", worker_file);
+                       "Unable to read worker file %s.", worker_file);
             }
-            map_free(&map);
+            jk_map_free(&map);
         }
     }
 
@@ -1098,51 +1173,51 @@ static int read_registry_init_data(void)
     HKEY hkey;
     long rc;
     int ok = JK_TRUE;
-    char *tmp;
+    const char *tmp;
     jk_map_t *map;
 
-    if (map_alloc(&map)) {
-        if (map_read_properties(map, ini_file_name)) {
+    if (jk_map_alloc(&map)) {
+        if (jk_map_read_properties(map, ini_file_name, NULL)) {
             using_ini_file = JK_TRUE;
         }
     }
     if (using_ini_file) {
-        tmp = map_get_string(map, JK_LOG_FILE_TAG, NULL);
+        tmp = jk_map_get_string(map, JK_LOG_FILE_TAG, NULL);
         if (tmp) {
             strcpy(log_file, tmp);
         }
         else {
             ok = JK_FALSE;
         }
-        tmp = map_get_string(map, JK_LOG_LEVEL_TAG, NULL);
+        tmp = jk_map_get_string(map, JK_LOG_LEVEL_TAG, NULL);
         if (tmp) {
             log_level = jk_parse_log_level(tmp);
         }
         else {
             ok = JK_FALSE;
         }
-        tmp = map_get_string(map, EXTENSION_URI_TAG, NULL);
+        tmp = jk_map_get_string(map, EXTENSION_URI_TAG, NULL);
         if (tmp) {
             strcpy(extension_uri, tmp);
         }
         else {
             ok = JK_FALSE;
         }
-        tmp = map_get_string(map, JK_WORKER_FILE_TAG, NULL);
+        tmp = jk_map_get_string(map, JK_WORKER_FILE_TAG, NULL);
         if (tmp) {
             strcpy(worker_file, tmp);
         }
         else {
             ok = JK_FALSE;
         }
-        tmp = map_get_string(map, JK_MOUNT_FILE_TAG, NULL);
+        tmp = jk_map_get_string(map, JK_MOUNT_FILE_TAG, NULL);
         if (tmp) {
             strcpy(worker_mount_file, tmp);
         }
         else {
             ok = JK_FALSE;
         }
-        tmp = map_get_string(map, URI_SELECT_TAG, NULL);
+        tmp = jk_map_get_string(map, URI_SELECT_TAG, NULL);
         if (tmp) {
             int opt = parse_uri_select(tmp);
             if (opt >= 0) {
@@ -1291,7 +1366,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
     s->headers_names = NULL;
     s->headers_values = NULL;
     s->num_headers = 0;
-
+    s->uw_map = uw_map;
     /*
      * Add SSL IIS environment
      */
@@ -1318,8 +1393,8 @@ static int init_ws_service(isapi_private_data_t * private_data,
             NULL,
             NULL
         };
-        unsigned i;
-        unsigned num_of_vars = 0;
+        unsigned int i;
+        unsigned int num_of_vars = 0;
 
         for (i = 0; i < 9; i++) {
             GET_SERVER_VARIABLE_VALUE(ssl_env_names[i], ssl_env_values[i]);
@@ -1328,7 +1403,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
             }
         }
         if (num_of_vars) {
-            unsigned j;
+            unsigned int j;
 
             s->attributes_names =
                 jk_pool_alloc(&private_data->p, num_of_vars * sizeof(char *));
@@ -1356,7 +1431,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
                                           (LPVOID) & cc, NULL,
                                           NULL) != FALSE) {
                     jk_log(logger, JK_LOG_DEBUG,
-                           "Client Certificate encoding:%d sz:%d flags:%ld\n",
+                           "Client Certificate encoding:%d sz:%d flags:%ld",
                            cc.CertContext.
                            dwCertEncodingType & X509_ASN_ENCODING,
                            cc.CertContext.cbCertEncoded,
@@ -1377,8 +1452,8 @@ static int init_ws_service(isapi_private_data_t * private_data,
 
     huge_buf_sz = sizeof(huge_buf);
     if (get_server_value(private_data->lpEcb,
-                         "ALL_HTTP", huge_buf, huge_buf_sz, "")) {
-        unsigned cnt = 0;
+                         "ALL_HTTP", huge_buf, huge_buf_sz)) {
+        unsigned int cnt = 0;
         char *tmp;
 
         for (tmp = huge_buf; *tmp; tmp++) {
@@ -1389,7 +1464,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
 
         if (cnt) {
             char *headers_buf = jk_pool_strdup(&private_data->p, huge_buf);
-            unsigned i;
+            unsigned int i;
             size_t len_of_http_prefix = strlen("HTTP_");
             BOOL need_content_length_header = (s->content_length == 0);
 
@@ -1490,18 +1565,16 @@ static int init_ws_service(isapi_private_data_t * private_data,
 }
 
 static int get_server_value(LPEXTENSION_CONTROL_BLOCK lpEcb,
-                            char *name, char *buf, DWORD bufsz, char *def_val)
+                            char *name, char *buf, DWORD bufsz)
 {
-    if (!lpEcb->GetServerVariable(lpEcb->ConnID,
-                                  name, buf, (LPDWORD) &bufsz)) {
-        strcpy(buf, def_val);
+    DWORD sz = bufsz;
+    buf[0]   = '\0';
+    if (!lpEcb->GetServerVariable(lpEcb->ConnID, name,
+                                  buf, (LPDWORD) &sz))
         return JK_FALSE;
-    }
-
-    if (bufsz > 0) {
-        buf[bufsz - 1] = '\0';
-    }
 
+    if (sz <= bufsz)
+        buf[sz-1] = '\0';
     return JK_TRUE;
 }
 
