@@ -62,7 +62,7 @@ import org.apache.jasper.xmlparser.TreeNode;
 class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
 
     // Logger
-    private static Log log = LogFactory.getLog(TagLibraryInfoImpl.class);
+    private Log log = LogFactory.getLog(TagLibraryInfoImpl.class);
 
     private Hashtable jarEntries;
     private JspCompilationContext ctxt;
@@ -473,10 +473,14 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
                 name = child.getBody();
             } else if ("path".equals(tname)) {
                 path = child.getBody();
+            } else if ("example".equals(tname)) {
+                // Ignore <example> element: Bugzilla 33538
+            } else if ("tag-extension".equals(tname)) {
+                // Ignore <tag-extension> element: Bugzilla 33538
             } else {
                 if (log.isWarnEnabled()) {
                     log.warn(Localizer.getMessage(
-                            "jsp.warning.unknown.element.in.attribute", tname));
+                            "jsp.warning.unknown.element.in.tagfile", tname));
                 }
             }
         }
