@@ -49,6 +49,7 @@ import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.jasper.Constants;
 import org.apache.jasper.compiler.Localizer;
+import org.apache.jasper.security.SecurityUtil;
 
 /**
  * Implementation of the PageContext class from the JSP spec.
@@ -216,7 +217,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
                     Localizer.getMessage("jsp.error.attribute.null_name"));
         }
 
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             return AccessController.doPrivileged(new PrivilegedAction(){
                 public Object run(){
                     return doGetAttribute(name);
@@ -239,7 +240,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
                     Localizer.getMessage("jsp.error.attribute.null_name"));
         }
 
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             return AccessController.doPrivileged(new PrivilegedAction(){
                 public Object run(){
                     return doGetAttribute(name, scope);
@@ -281,7 +282,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
                     Localizer.getMessage("jsp.error.attribute.null_name"));
         }
 
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             AccessController.doPrivileged(new PrivilegedAction(){
                 public Object run(){
                     doSetAttribute(name, attribute);
@@ -308,7 +309,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
                     Localizer.getMessage("jsp.error.attribute.null_name"));
         }
 
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             AccessController.doPrivileged(new PrivilegedAction(){
                 public Object run(){
                     doSetAttribute(name, o, scope);
@@ -358,7 +359,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
             throw new NullPointerException(
                     Localizer.getMessage("jsp.error.attribute.null_name"));
         }
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             AccessController.doPrivileged(new PrivilegedAction(){
                 public Object run(){
                     doRemoveAttribute(name, scope);
@@ -404,7 +405,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
                     Localizer.getMessage("jsp.error.attribute.null_name"));
         }
 
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             return ((Integer)AccessController.doPrivileged(new PrivilegedAction(){
                 public Object run(){
                     return new Integer(doGetAttributeScope(name));
@@ -434,7 +435,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
     }
 
     public Object findAttribute(final String name) {
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             return AccessController.doPrivileged(new PrivilegedAction(){
                 public Object run(){
                     if (name == null) {
@@ -476,7 +477,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
 
 
     public Enumeration getAttributeNamesInScope(final int scope) {
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             return (Enumeration)
                     AccessController.doPrivileged(new PrivilegedAction(){
                 public Object run(){
@@ -518,7 +519,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
                     Localizer.getMessage("jsp.error.attribute.null_name"));
         }
 
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             AccessController.doPrivileged(new PrivilegedAction(){
                 public Object run(){
                     doRemoveAttribute(name);
@@ -558,26 +559,28 @@ public class PageContextImpl extends PageContext implements VariableResolver {
     public ServletRequest getRequest() { return request; }
     public ServletResponse getResponse() { return response; }
 
+
     /**
      * Returns the exception associated with this page
      * context, if any.
      * <p/>
-     * Added wrapping for Throwables to avoid ClassCaseException:
+     * Added wrapping for Throwables to avoid ClassCastException:
      * see Bugzilla 31171 for details.
      *
      * @return The Exception associated with this page context, if any.
      */
     public Exception getException() {
-        Throwable exc = (Throwable) request.getAttribute(EXCEPTION);
+        Throwable t = JspRuntimeLibrary.getThrowable(request);
 
         // Only wrap if needed
-        if((exc != null) && (! (exc instanceof Exception))) {
-            exc = new JspException(exc);
+        if((t != null) && (! (t instanceof Exception))) {
+            t = new JspException(t);
         }
 
-        return (Exception) exc;
+        return (Exception) t;
     }
 
+
     public Object getPage() { return servlet; }
 
 
@@ -604,7 +607,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
 
     public void include(final String relativeUrlPath, final boolean flush)
 	        throws ServletException, IOException {
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             try{
                 AccessController.doPrivileged(new PrivilegedExceptionAction(){
                     public Object run() throws Exception{
@@ -637,7 +640,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
 
     public void forward(final String relativeUrlPath)
         throws ServletException, IOException {
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             try{
                 AccessController.doPrivileged(new PrivilegedExceptionAction(){
                     public Object run() throws Exception{
@@ -758,7 +761,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
         if (t == null)
             throw new NullPointerException("null Throwable");
 
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             try{
                 AccessController.doPrivileged(new PrivilegedExceptionAction(){
                     public Object run() throws Exception{
@@ -801,26 +804,26 @@ public class PageContextImpl extends PageContext implements VariableResolver {
             ((HttpServletRequest) request).getRequestURI());
             request.setAttribute("javax.servlet.error.servlet_name",
                      config.getServletName());
-                try {
-                    forward(errorPageURL);
-                } catch (IllegalStateException ise) {
-                    include(errorPageURL);
-                }
+            try {
+                forward(errorPageURL);
+            } catch (IllegalStateException ise) {
+                include(errorPageURL);
+            }
 
-                // The error page could be inside an include.
+            // The error page could be inside an include.
 
-                Object newException = request.getAttribute("javax.servlet.error.exception");
+            Object newException = request.getAttribute("javax.servlet.error.exception");
 
-                // t==null means the attribute was not set.
-                if( (newException!= null) && (newException==t) ) {
-                    request.removeAttribute("javax.servlet.error.exception");
-                }
+            // t==null means the attribute was not set.
+            if( (newException!= null) && (newException==t) ) {
+                request.removeAttribute("javax.servlet.error.exception");
+            }
 
-                // now clear the error code - to prevent double handling.
-                request.removeAttribute("javax.servlet.error.status_code");
-                request.removeAttribute("javax.servlet.error.request_uri");
-                request.removeAttribute("javax.servlet.error.status_code");
-                request.removeAttribute("javax.servlet.jsp.jspException");
+            // now clear the error code - to prevent double handling.
+            request.removeAttribute("javax.servlet.error.status_code");
+            request.removeAttribute("javax.servlet.error.request_uri");
+            request.removeAttribute("javax.servlet.error.status_code");
+            request.removeAttribute("javax.servlet.jsp.jspException");
 
         } else {
             // Otherwise throw the exception wrapped inside a ServletException.
@@ -895,7 +898,7 @@ public class PageContextImpl extends PageContext implements VariableResolver {
 	throws ELException
     {
 	Object retValue;
-        if (System.getSecurityManager() != null){
+        if (SecurityUtil.isPackageProtectionEnabled()){
             try {
                 retValue = AccessController.doPrivileged(
 			new PrivilegedExceptionAction(){
